@@ -1,8 +1,10 @@
 import os,re,sys
 
 try:
-	suffix=sys.argv[1]
+	prefix=sys.argv[1]
+	suffix=sys.argv[2]
 except IndexError:
+	prefix='_'
 	suffix='so'
 
 incexpr=re.compile('^\s*[%#]include "(\S+)"',re.MULTILINE)
@@ -10,10 +12,21 @@ deps=dict();
 deps['carrays.i']=[]
 deps['cpointer.i']=[]
 deps['exception.i']=[]
+deps['lib/Library_doxygen.i']=[]
+deps['features/Features_doxygen.i']=[]
+deps['classifier/Classifier_doxygen.i']=[]
+deps['structure/Structure_doxygen.i']=[]
+deps['regression/Regression_doxygen.i']=[]
+deps['kernel/Kernel_doxygen.i']=[]
+deps['preproc/PreProc_doxygen.i']=[]
+deps['distributions/Distribution_doxygen.i']=[]
+deps['classifier/Classifier_doxygen.i']=[]
+deps['clustering/Clustering_doxygen.i']=[]
+deps['distance/Distance_doxygen.i']=[]
+deps['evaluation/Evaluation_doxygen.i']=[]
 
 initial_deps=deps.copy()
 
-
 def get_deps(f):
 	global deps
 	global fdep
@@ -40,7 +53,7 @@ for f in files:
 for f in deps.iterkeys():
 	if f[-1] == 'i' and not initial_deps.has_key(f):
 		if file(f).read().find('%module')>-1:
-			str1=os.path.join(os.path.dirname(f), '_' + os.path.basename(f)[:-2]) + '.' + suffix + ': ' + f[:-2]+'_wrap.cxx.o'
+			str1=os.path.join(os.path.dirname(f), prefix + os.path.basename(f)[:-2]) + '.' + suffix + ': ' + f[:-2]+'_wrap.cxx.o'
 			str2=os.path.join(os.path.dirname(f), os.path.basename(f)[:-2]) + '_wrap.cxx: ' + f
 
 			fdep=list();
