@@ -1,11 +1,11 @@
 /*
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
- * Written (W) 1999-2007 Soeren Sonnenburg
- * Copyright (C) 1999-2007 Fraunhofer Institute FIRST and Max-Planck-Society
+ * Written (W) 1999-2008 Soeren Sonnenburg
+ * Copyright (C) 1999-2008 Fraunhofer Institute FIRST and Max-Planck-Society
  */
 
 #include "lib/common.h"
@@ -24,8 +24,8 @@ public:
 	static inline void print_version()
 	{
 		SG_SPRINT("shogun (%s/%s/%s%d)\n\n", TARGET, MACHINE, VERSION_RELEASE, version_revision);
-		SG_SPRINT("Copyright (C) 1999-2007 Fraunhofer Institute FIRST and Max-Planck-Society\n");
-		SG_SPRINT("Written   (W) 2000-2007 Soeren Sonnenburg, Gunnar Raetsch et.al.\n\n");
+		SG_SPRINT("Copyright (C) 1999-2008 Fraunhofer Institute FIRST and Max-Planck-Society\n");
+		SG_SPRINT("Written   (W) 2000-2008 Soeren Sonnenburg, Gunnar Raetsch et al.\n\n");
 #ifdef GPL
 		SG_SPRINT("This is free software; see the source for copying conditions.  There is NO\n");
 		SG_SPRINT("warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n\n");
@@ -78,8 +78,8 @@ public:
 		return ((((version_year)*12 + version_month)*30 + version_day)* 24 + version_hour)*60 + version_minute;
 	}
 
-	static const CHAR* version_release;
-	static const CHAR* version_extra;
+	static const CHAR version_release[128];
+	static const CHAR version_extra[128];
 
 	static const INT version_revision;
 	static const INT version_year;
