@@ -1,23 +1,32 @@
 /*
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
- * 
- * Written (W) 2006 Soeren Sonnenburg
+ *
  * Written (W) 2006 Christian Gehl
- * Copyright (C) 1999-2007 Fraunhofer Institute FIRST and Max-Planck-Society
+ * Written (W) 2006-2008 Soeren Sonnenburg
+ * Copyright (C) 1999-2008 Fraunhofer Institute FIRST and Max-Planck-Society
  */
 
 #include "classifier/KNN.h"
 #include "features/Labels.h"
 #include "lib/Mathematics.h"
 
-CKNN::CKNN(): k(3), num_classes(0), num_train_labels(0), train_labels(NULL)
+CKNN::CKNN()
+: CDistanceMachine(), k(3), num_classes(0), num_train_labels(0), train_labels(NULL)
 {
 }
 
+CKNN::CKNN(INT k_, CDistance* d, CLabels* trainlab)
+: CDistanceMachine(), k(k_), num_classes(0), train_labels(NULL)
+{
+    set_distance(d);
+    set_labels(trainlab);
+    num_train_labels=trainlab->get_num_labels();
+}
+
 
 CKNN::~CKNN()
 {
@@ -26,10 +35,9 @@ CKNN::~CKNN()
 
 bool CKNN::train()
 {
-	ASSERT(CDistanceMachine::get_labels());
-	
-	train_labels=CDistanceMachine::get_labels()->get_int_labels(num_train_labels);
+	ASSERT(labels);
 
+	train_labels=labels->get_int_labels(num_train_labels);
 	ASSERT(train_labels);
 	ASSERT(num_train_labels>0);
 
@@ -55,13 +63,13 @@ bool CKNN::train()
 
 CLabels* CKNN::classify(CLabels* output)
 {
-	ASSERT(CDistanceMachine::get_distance());
-	ASSERT(CDistanceMachine::get_labels());
-	ASSERT(CDistanceMachine::get_labels()->get_num_labels());
+	ASSERT(num_classes>0);
+	ASSERT(distance);
+	ASSERT(labels);
+	ASSERT(labels->get_num_labels());
 
-	int num_lab=CDistanceMachine::get_labels()->get_num_labels();
-	
-	CDistance* _distance=CDistanceMachine::get_distance();
+	INT num_lab=labels->get_num_labels();
+	ASSERT(k<=num_lab);
 
 	//distances to train data and working buffer of train_labels
 	DREAL* dists=new DREAL[num_train_labels];
@@ -78,18 +86,18 @@ CLabels* CKNN::classify(CLabels* output)
 	ASSERT(classes);
 
 	SG_INFO( "%d test examples\n", num_lab);
-	for (int i=0; i<num_lab; i++)
+	for (INT i=0; i<num_lab; i++)
 	{
-		if ( (i% (num_lab/10+1))== 0)
-			SG_PRINT( "%i%%..",100*i/(num_lab+1));
+		if ((i%(num_lab/10+1))== 0)
+			SG_PROGRESS(i, 0, num_lab);
 
-		int j;
+		INT j;
 		for (j=0; j<num_train_labels; j++)
 		{
 			//copy back train labels and compute distance
 			train_lab[j]=train_labels[j];
 			
-			dists[j]=_distance->distance(j,i);
+			dists[j]=distance->distance(j,i);
 		}
 
 		//sort the distance vector for test example j to all train examples
