@@ -1,11 +1,11 @@
 /*
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
- * Written (W) 1999-2007 Soeren Sonnenburg
- * Copyright (C) 1999-2007 Fraunhofer Institute FIRST and Max-Planck-Society
+ * Written (W) 1999-2008 Soeren Sonnenburg
+ * Copyright (C) 1999-2008 Fraunhofer Institute FIRST and Max-Planck-Society
  */
 
 #include "classifier/KernelPerceptron.h"
@@ -13,6 +13,7 @@
 #include "lib/Mathematics.h"
 
 CKernelPerceptron::CKernelPerceptron()
+: CKernelMachine()
 {
 }
 
@@ -23,8 +24,8 @@ CKernelPerceptron::~CKernelPerceptron()
 
 bool CKernelPerceptron::train()
 {
-	ASSERT(CKernelMachine::get_labels());
-	//CLabels* train_labels=CKernelMachine::get_labels()->get_int_labels(num_train_labels);
+	ASSERT(labels);
+	//CLabels* train_labels=labels->get_int_labels(num_train_labels);
 
 //
 //# compute output activation y = f(w x)
