@@ -1,11 +1,11 @@
 /*
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
- * Written (W) 1999-2007 Soeren Sonnenburg
- * Copyright (C) 1999-2007 Fraunhofer Institute FIRST and Max-Planck-Society
+ * Written (W) 1999-2008 Soeren Sonnenburg
+ * Copyright (C) 1999-2008 Fraunhofer Institute FIRST and Max-Planck-Society
  */
 
 #include "lib/common.h"
@@ -18,14 +18,16 @@
 #include "lib/Mathematics.h"
 #include "lib/lapack.h"
 
-CLDA::CLDA(DREAL gamma) : CLinearClassifier(), m_gamma(gamma)
+CLDA::CLDA(DREAL gamma)
+: CLinearClassifier(), m_gamma(gamma)
 {
 }
 
-CLDA::CLDA(DREAL gamma, CRealFeatures* traindat, CLabels* trainlab) : CLinearClassifier(), m_gamma(gamma)
+CLDA::CLDA(DREAL gamma, CRealFeatures* traindat, CLabels* trainlab)
+: CLinearClassifier(), m_gamma(gamma)
 {
-	CLinearClassifier::features=traindat;
-	CClassifier::labels=trainlab;
+	set_features(traindat);
+	set_labels(trainlab);
 }
 
 
@@ -35,10 +37,10 @@ CLDA::~CLDA()
 
 bool CLDA::train()
 {
-	ASSERT(get_labels());
-	ASSERT(get_features());
+	ASSERT(labels);
+	ASSERT(features);
 	INT num_train_labels=0;
-	INT* train_labels=get_labels()->get_int_labels(num_train_labels);
+	INT* train_labels=labels->get_int_labels(num_train_labels);
 	ASSERT(train_labels);
 
 	INT num_feat=features->get_num_features();
@@ -46,9 +48,7 @@ bool CLDA::train()
 	ASSERT(num_vec==num_train_labels);
 
 	INT* classidx_neg=new INT[num_vec];
-	ASSERT(classidx_neg);
 	INT* classidx_pos=new INT[num_vec];
-	ASSERT(classidx_pos);
 
 	INT i=0;
 	INT j=0;
@@ -75,22 +75,16 @@ bool CLDA::train()
 
 	delete[] w;
 	w=new DREAL[num_feat];
-	ASSERT(w);
 	w_dim=num_feat;
 
 	DREAL* mean_neg=new DREAL[num_feat];
-	ASSERT(mean_neg);
 	memset(mean_neg,0,num_feat*sizeof(DREAL));
 
 	DREAL* mean_pos=new DREAL[num_feat];
-	ASSERT(mean_pos);
 	memset(mean_pos,0,num_feat*sizeof(DREAL));
 
 	DREAL* scatter=new DREAL[num_feat*num_feat];
-	ASSERT(scatter);
-
 	DREAL* buffer=new DREAL[num_feat*CMath::max(num_neg, num_pos)];
-	ASSERT(buffer);
 
 	//mean neg
 	for (i=0; i<num_neg; i++)
