@@ -1,11 +1,11 @@
 /*
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
- * Written (W) 1999-2007 Soeren Sonnenburg
- * Copyright (C) 1999-2007 Fraunhofer Institute FIRST and Max-Planck-Society
+ * Written (W) 1999-2008 Soeren Sonnenburg
+ * Copyright (C) 1999-2008 Fraunhofer Institute FIRST and Max-Planck-Society
  */
 
 #ifndef _PERCEPTRON_H___
@@ -16,13 +16,31 @@
 #include "features/Features.h"
 #include "classifier/LinearClassifier.h"
 
+/** class Perceptron */
 class CPerceptron : public CLinearClassifier
 {
 	public:
+		/** default constructor */
 		CPerceptron();
+
+		/** constructor
+		 *
+		 * @param traindat training features
+		 * @param trainlab labels for training features
+		 */
+		CPerceptron(CRealFeatures* traindat, CLabels* trainlab);
 		virtual ~CPerceptron();
 
+		/** get classifier type
+		 *
+		 * @return classifier type PERCEPTRON
+		 */
 		virtual inline EClassifierType get_classifier_type() { return CT_PERCEPTRON; }
+
+		/** train classifier
+		 *
+		 * @return if training was successful
+		 */
 		virtual bool train();
 
 		/// set learn rate of gradient descent training algorithm
@@ -36,8 +54,11 @@ class CPerceptron : public CLinearClassifier
 		{
 			max_iter=i;
 		}
+
 	protected:
+		/** learning rate */
 		DREAL learn_rate;
+		/** maximum number of iterations */
 		INT max_iter;
 };
 #endif
