@@ -1,15 +1,16 @@
 /*
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
- * Written (W) 1999-2007 Soeren Sonnenburg
- * Copyright (C) 1999-2007 Fraunhofer Institute FIRST and Max-Planck-Society
+ * Written (W) 1999-2008 Soeren Sonnenburg
+ * Copyright (C) 1999-2008 Fraunhofer Institute FIRST and Max-Planck-Society
  */
 
 #include "classifier/svm/CPLEXSVM.h"
 #include "lib/common.h"
+
 #ifdef USE_CPLEX
 #include "lib/io.h"
 #include "lib/Mathematics.h"
@@ -17,6 +18,7 @@
 #include "features/Labels.h"
 
 CCPLEXSVM::CCPLEXSVM()
+: CSVM()
 {
 }
 
@@ -27,20 +29,18 @@ CCPLEXSVM::~CCPLEXSVM()
 bool CCPLEXSVM::train()
 {
 	bool result = false;
-	CLabels* lab = CKernelMachine::get_labels();
 	CCplex cplex;
 
 	if (cplex.init(E_QP))
 	{
 		INT n,m;
 		INT num_label=0;
-		DREAL* y = lab->get_labels(num_label);
+		DREAL* y = labels->get_labels(num_label);
 		DREAL* H = kernel->get_kernel_matrix_real(m, n, NULL);
 		ASSERT(n>0 && n==m && n==num_label);
 		DREAL* alphas=new DREAL[n];
 		DREAL* lb=new DREAL[n];
 		DREAL* ub=new DREAL[n];
-		ASSERT(lb && ub);
 
 		//hessian y'y.*K
 		for (int i=0; i<n; i++)
@@ -60,8 +60,8 @@ bool CCPLEXSVM::train()
 		{
 			if (alphas[i]>0)
 			{
-				//set_alpha(j, alphas[i]*lab->get_label(i)/etas[1]);
-				set_alpha(j, alphas[i]*lab->get_label(i));
+				//set_alpha(j, alphas[i]*labels->get_label(i)/etas[1]);
+				set_alpha(j, alphas[i]*labels->get_label(i));
 				set_support_vector(j, i);
 				j++;
 			}
