@@ -1,11 +1,11 @@
 /*
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
- * Written (W) 1999-2007 Soeren Sonnenburg
- * Copyright (C) 1999-2007 Fraunhofer Institute FIRST and Max-Planck-Society
+ * Written (W) 1999-2008 Soeren Sonnenburg
+ * Copyright (C) 1999-2008 Fraunhofer Institute FIRST and Max-Planck-Society
  */
 
 #include "classifier/svm/GPBTSVM.h"
@@ -13,11 +13,13 @@
 #include "classifier/svm/gpdtsolve.h"
 #include "lib/io.h"
 
-CGPBTSVM::CGPBTSVM() : CSVM(), model(NULL)
+CGPBTSVM::CGPBTSVM()
+: CSVM(), model(NULL)
 {
 }
 
-CGPBTSVM::CGPBTSVM(DREAL C, CKernel* k, CLabels* lab) : CSVM(C, k, lab), model(NULL)
+CGPBTSVM::CGPBTSVM(DREAL C, CKernel* k, CLabels* lab)
+: CSVM(C, k, lab), model(NULL)
 {
 }
 
@@ -31,21 +33,20 @@ bool CGPBTSVM::train()
 	double     *solution;                     /* store the solution found       */
 	QPproblem  prob;                          /* object containing the solvers  */
 
-	ASSERT(get_kernel());
-	ASSERT(get_labels() && get_labels()->get_num_labels());
-	ASSERT(get_labels()->is_two_class_labeling());
+	ASSERT(kernel);
+	ASSERT(labels && labels->get_num_labels());
+	ASSERT(labels->is_two_class_labeling());
 
 	int num_lab = 0;
 	int* lab=get_labels()->get_int_labels(num_lab);
-	prob.KER=new sKernel(get_kernel(), num_lab);
+	prob.KER=new sKernel(kernel, num_lab);
 	prob.y=lab;
-	ASSERT(prob.KER);
 	prob.ell=get_labels()->get_num_labels();
 	SG_INFO( "%d trainlabels\n", prob.ell);
 
 	//  /*** set options defaults ***/
 	prob.delta = epsilon;
-	prob.maxmw = get_kernel()->get_cache_size();
+	prob.maxmw = kernel->get_cache_size();
 	prob.verbosity       = 0;
 	prob.preprocess_size = -1;
 	prob.projection_projector = -1;
@@ -86,7 +87,7 @@ bool CGPBTSVM::train()
 	prob.gpdtsolve(solution);
 	/****************************************************************************/
 
-  CSVM::set_objective(prob.objective_value);
+	CSVM::set_objective(prob.objective_value);
 
 	int num_sv=0;
 	int bsv=0;
@@ -116,8 +117,9 @@ bool CGPBTSVM::train()
 		}
 	}
 
+	delete prob.KER;
+	delete[] prob.y;
 	delete[] solution;
-	delete[] lab;
 
 	return true;
 }
