@@ -1,21 +1,23 @@
 /*
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
- * Written (W) 1999-2007 Soeren Sonnenburg
- * Copyright (C) 1999-2007 Fraunhofer Institute FIRST and Max-Planck-Society
+ * Written (W) 1999-2008 Soeren Sonnenburg
+ * Copyright (C) 1999-2008 Fraunhofer Institute FIRST and Max-Planck-Society
  */
 
 #include "classifier/svm/LibSVM.h"
 #include "lib/io.h"
 
-CLibSVM::CLibSVM() : CSVM(), model(NULL)
+CLibSVM::CLibSVM()
+: CSVM(), model(NULL)
 {
 }
 
-CLibSVM::CLibSVM(DREAL C, CKernel* k, CLabels* lab) : CSVM(C, k, lab), model(NULL)
+CLibSVM::CLibSVM(DREAL C, CKernel* k, CLabels* lab)
+: CSVM(C, k, lab), model(NULL)
 {
 }
 
@@ -26,26 +28,21 @@ CLibSVM::~CLibSVM()
 
 bool CLibSVM::train()
 {
-
 	struct svm_node* x_space;
 
-	ASSERT(get_labels() && get_labels()->get_num_labels());
-	ASSERT(get_labels()->is_two_class_labeling());
+	ASSERT(labels && labels->get_num_labels());
+	ASSERT(labels->is_two_class_labeling());
 
-	problem.l=get_labels()->get_num_labels();
+	problem.l=labels->get_num_labels();
 	SG_INFO( "%d trainlabels\n", problem.l);
 
 	problem.y=new double[problem.l];
 	problem.x=new struct svm_node*[problem.l];
 	x_space=new struct svm_node[2*problem.l];
 
-	ASSERT(problem.y);
-	ASSERT(problem.x);
-	ASSERT(x_space);
-
 	for (int i=0; i<problem.l; i++)
 	{
-		problem.y[i]=get_labels()->get_label(i);
+		problem.y[i]=labels->get_label(i);
 		problem.x[i]=&x_space[2*i];
 		x_space[2*i].index=i;
 		x_space[2*i+1].index=-1;
@@ -54,8 +51,8 @@ bool CLibSVM::train()
 	int weights_label[2]={-1,+1};
 	double weights[2]={1.0,get_C2()/get_C1()};
 
-	ASSERT(get_kernel() && get_kernel()->get_lhs());
-    ASSERT(get_kernel()->get_lhs()->get_num_vectors() == problem.l);
+	ASSERT(kernel && kernel->has_features());
+    ASSERT(kernel->get_num_vec_lhs()==problem.l);
 
 	param.svm_type=C_SVC; // C SVM
 	param.kernel_type = LINEAR;
@@ -63,8 +60,8 @@ bool CLibSVM::train()
 	param.gamma = 0;	// 1/k
 	param.coef0 = 0;
 	param.nu = 0.5;
-	param.kernel=get_kernel();
-	param.cache_size = get_kernel()->get_cache_size();
+	param.kernel=kernel;
+	param.cache_size = kernel->get_cache_size();
 	param.C = get_C1();
 	param.eps = epsilon;
 	param.p = 0.1;
@@ -86,7 +83,7 @@ bool CLibSVM::train()
 	if (model)
 	{
 		ASSERT(model->nr_class==2);
-		ASSERT( (model->l==0) || (model->l > 0 && model->SV && model->sv_coef && model->sv_coef[0]) );
+		ASSERT((model->l==0) || (model->l>0 && model->SV && model->sv_coef && model->sv_coef[0]));
 
 		int num_sv=model->l;
 
@@ -107,7 +104,7 @@ bool CLibSVM::train()
 		delete[] problem.y;
 		delete[] x_space;
 
-		free(model);
+		svm_destroy_model(model);
 		model=NULL;
 		return true;
 	}
