@@ -1,11 +1,11 @@
 /*
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
- * Written (W) 1999-2007 Soeren Sonnenburg
- * Copyright (C) 1999-2007 Fraunhofer Institute FIRST and Max-Planck-Society
+ * Written (W) 1999-2008 Soeren Sonnenburg
+ * Copyright (C) 1999-2008 Fraunhofer Institute FIRST and Max-Planck-Society
  */
 
 #include "lib/common.h"
@@ -21,7 +21,7 @@
 #include <pthread.h>
 #endif
 
-struct S_THREAD_PARAM 
+struct S_THREAD_PARAM
 {
 	CSVM* svm;
 	CLabels* result;
@@ -30,12 +30,14 @@ struct S_THREAD_PARAM
 	bool verbose;
 };
 
-CSVM::CSVM(INT num_sv) : CKernelMachine()
+CSVM::CSVM(INT num_sv)
+: CKernelMachine()
 {
 	set_defaults(num_sv);
 }
 
-CSVM::CSVM(DREAL C, CKernel* k, CLabels* lab) : CKernelMachine()
+CSVM::CSVM(DREAL C, CKernel* k, CLabels* lab)
+: CKernelMachine()
 {
 	set_defaults();
 	set_C(C,C);
@@ -45,10 +47,10 @@ CSVM::CSVM(DREAL C, CKernel* k, CLabels* lab) : CKernelMachine()
 
 CSVM::~CSVM()
 {
-  delete[] svm_model.alpha ;
-  delete[] svm_model.svs ;
+	delete[] svm_model.alpha;
+	delete[] svm_model.svs;
 
-  SG_DEBUG( "SVM object destroyed\n") ;
+	SG_DEBUG("SVM object destroyed\n");
 }
 
 void CSVM::set_defaults(INT num_sv)
@@ -200,7 +202,7 @@ bool CSVM::save(FILE* modelfl)
 	SG_INFO( "Writing model file...");
 	fprintf(modelfl,"%%SVM\n");
 	fprintf(modelfl,"numsv=%d;\n", get_num_support_vectors());
-	fprintf(modelfl,"kernel='%s';\n", CKernelMachine::get_kernel()->get_name());
+	fprintf(modelfl,"kernel='%s';\n", kernel->get_name());
 	fprintf(modelfl,"b=%+10.16e;\n",get_bias());
 
 	fprintf(modelfl, "alphas=\[\n");
@@ -211,7 +213,7 @@ bool CSVM::save(FILE* modelfl)
 
 	fprintf(modelfl, "];\n");
 
-	SG_INFO( "done\n");
+	SG_DONE();
 	return true ;
 } 
 
@@ -219,7 +221,7 @@ bool CSVM::init_kernel_optimization()
 {
 	INT num_sv=get_num_support_vectors();
 
-	if (get_kernel() && get_kernel()->has_property(KP_LINADD) && num_sv>0)
+	if (kernel && kernel->has_property(KP_LINADD) && num_sv>0)
 	{
 		INT * sv_idx    = new INT[num_sv] ;
 		DREAL* sv_weight = new DREAL[num_sv] ;
@@ -255,6 +257,7 @@ void* CSVM::classify_example_helper(void* p)
 #ifdef WIN32
 	for (INT vec=params->start; vec<params->end; vec++)
 #else
+	CSignal::clear() ;
 	for (INT vec=params->start; vec<params->end && 
 			!CSignal::cancel_computations(); vec++)
 #endif
@@ -273,40 +276,35 @@ void* CSVM::classify_example_helper(void* p)
 	return NULL;
 }
 
-CLabels* CSVM::classify(CLabels* result)
+CLabels* CSVM::classify(CLabels* lab)
 {
-	if (!CKernelMachine::get_kernel())
+	if (!kernel)
 	{
 		SG_ERROR( "SVM can not proceed without kernel!\n");
 		return false ;
 	}
 
-	if ( (CKernelMachine::get_kernel()) &&
-		 (CKernelMachine::get_kernel())->get_rhs() &&
-		 (CKernelMachine::get_kernel())->get_rhs()->get_num_vectors())
+	if ( kernel && kernel->get_num_vec_rhs()>0 )
 	{
-		INT num_vectors=get_kernel()->get_rhs()->get_num_vectors();
+		INT num_vectors=kernel->get_num_vec_rhs();
 
-		if (!result)
-			result=new CLabels(num_vectors);
-
-		ASSERT(result);
+		if (!lab)
+			lab=new CLabels(num_vectors);
 		SG_DEBUG( "computing output on %d test examples\n", num_vectors);
 
-		if (CKernelMachine::get_kernel()->has_property(KP_BATCHEVALUATION) &&
+		if (this->io.get_show_progress())
+			this->io.enable_progress();
+		else
+			this->io.disable_progress();
+
+		if (kernel->has_property(KP_BATCHEVALUATION) &&
 				get_batch_computation_enabled())
 		{
 			ASSERT(get_num_support_vectors()>0);
-			INT * sv_idx    = new INT[get_num_support_vectors()] ;
-			DREAL* sv_weight = new DREAL[get_num_support_vectors()] ;
-			INT* idx = new INT[num_vectors];
-			DREAL* output = new DREAL[num_vectors];
-
-			ASSERT(sv_idx);
-			ASSERT(sv_weight);
-
-			ASSERT(idx);
-			ASSERT(output);
+			INT* sv_idx=new INT[get_num_support_vectors()];
+			DREAL* sv_weight=new DREAL[get_num_support_vectors()];
+			INT* idx=new INT[num_vectors];
+			DREAL* output=new DREAL[num_vectors];
 			memset(output, 0, sizeof(DREAL)*num_vectors);
 
 			//compute output for all vectors v[0]...v[num_vectors-1]
@@ -319,11 +317,11 @@ CLabels* CSVM::classify(CLabels* result)
 				sv_weight[i] = get_alpha(i) ;
 			}
 
-			CKernelMachine::get_kernel()->compute_batch(num_vectors, idx,
+			kernel->compute_batch(num_vectors, idx,
 					output, get_num_support_vectors(), sv_idx, sv_weight);
 
 			for (INT i=0; i<num_vectors; i++)
-				result->set_label(i, get_bias()+output[i]);
+				lab->set_label(i, get_bias()+output[i]);
 
 			delete[] sv_idx ;
 			delete[] sv_weight ;
@@ -339,7 +337,7 @@ CLabels* CSVM::classify(CLabels* result)
 			{
 				S_THREAD_PARAM params;
 				params.svm=this;
-				params.result=result;
+				params.result=lab;
 				params.start=0;
 				params.end=num_vectors;
 				params.verbose=true;
@@ -357,7 +355,7 @@ CLabels* CSVM::classify(CLabels* result)
 				for (t=0; t<num_threads-1; t++)
 				{
 					params[t].svm = this;
-					params[t].result = result;
+					params[t].result = lab;
 					params[t].start = t*step;
 					params[t].end = (t+1)*step;
 					params[t].verbose = false;
@@ -366,7 +364,7 @@ CLabels* CSVM::classify(CLabels* result)
 				}
 
 				params[t].svm = this;
-				params[t].result = result;
+				params[t].result = lab;
 				params[t].start = t*step;
 				params[t].end = num_vectors;
 				params[t].verbose = true;
@@ -383,29 +381,28 @@ CLabels* CSVM::classify(CLabels* result)
 			SG_INFO( "prematurely stopped.           \n");
 		else
 #endif
-			SG_INFO( "done.           \n");
+			SG_DONE();
 	}
 	else 
 		return NULL;
 
-	return result;
+	return lab;
 }
 
 DREAL CSVM::classify_example(INT num)
 {
-	ASSERT(CKernelMachine::get_kernel());
+	ASSERT(kernel);
 
-	if (CKernelMachine::get_kernel()->has_property(KP_LINADD) &&
-			(CKernelMachine::get_kernel()->get_is_initialized()))
+	if (kernel->has_property(KP_LINADD) && (kernel->get_is_initialized()))
 	{
-		DREAL dist = CKernelMachine::get_kernel()->compute_optimized(num);
+		DREAL dist = kernel->compute_optimized(num);
 		return (dist+get_bias());
 	}
 	else
 	{
 		DREAL dist=0;
 		for(INT i=0; i<get_num_support_vectors(); i++)
-			dist+=get_kernel()->kernel(get_support_vector(i), num)*get_alpha(i);
+			dist+=kernel->kernel(get_support_vector(i), num)*get_alpha(i);
 
 		return (dist+get_bias());
 	}
@@ -414,21 +411,16 @@ DREAL CSVM::classify_example(INT num)
 
 DREAL CSVM::compute_objective()
 {
-	CLabels* lab=CKernelMachine::get_labels();
 	INT n=get_num_support_vectors();
-	CKernel* k=CKernelMachine::get_kernel();
 
-	if (lab && k)
+	if (labels && kernel)
 	{
-		ASSERT(lab);
-		ASSERT(k);
-
 		objective=0;
 		for (int i=0; i<n; i++)
 		{
-			objective-=get_alpha(i)*lab->get_label(i);
+			objective-=get_alpha(i)*labels->get_label(i);
 			for (int j=0; j<n; j++)
-				objective+=0.5*get_alpha(i)*get_alpha(j)*k->kernel(i,j);
+				objective+=0.5*get_alpha(i)*get_alpha(j)*kernel->kernel(i,j);
 		}
 	}
 	else
