@@ -2,13 +2,13 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * Library of solvers for Generalized Nearest Point Problem (GNPP).
  *
- * Written (W) 1999-2007 Vojtech Franc, xfrancv@cmp.felk.cvut.cz
- * Copyright (C) 1999-2007 Center for Machine Perception, CTU FEL Prague 
+ * Written (W) 1999-2008 Vojtech Franc, xfrancv@cmp.felk.cvut.cz
+ * Copyright (C) 1999-2008 Center for Machine Perception, CTU FEL Prague 
  *
 -------------------------------------------------------------------- */
 
@@ -29,7 +29,8 @@
 #define INDEX(ROW,COL,DIM) ((COL*DIM)+ROW)
 
 
-CGNPPLib::CGNPPLib(DREAL* vector_y, CKernel* kernel, INT num_data, DREAL reg_const) : CSGObject()
+CGNPPLib::CGNPPLib(DREAL* vector_y, CKernel* kernel, INT num_data, DREAL reg_const)
+: CSGObject()
 {
   m_reg_const = reg_const;
   m_num_data = num_data;
@@ -40,18 +41,15 @@ CGNPPLib::CGNPPLib(DREAL* vector_y, CKernel* kernel, INT num_data, DREAL reg_con
   Cache_Size = CMath::min(Cache_Size, (LONG) num_data);
 
   SG_INFO("using %d kernel cache lines\n", Cache_Size);
-  ASSERT(Cache_Size > 2);
+  ASSERT(Cache_Size>=2);
 
   /* allocates memory for kernel cache */
   kernel_columns = new DREAL*[Cache_Size];
-  ASSERT(kernel_columns);
   cache_index = new DREAL[Cache_Size];
-  ASSERT(cache_index);
 
   for(INT i = 0; i < Cache_Size; i++ ) 
   {
     kernel_columns[i] = new DREAL[num_data];
-    ASSERT(kernel_columns[i]);
     cache_index[i] = -2;
   }
   first_kernel_inx = 0;
