@@ -33,7 +33,7 @@
  ***      'Parallel Algorithms and Numerical Nonlinear Optimization'        ***
  ***      (grant RBAU01JYPN), http://dm.unife.it/pn2o                       ***
  ***                                                                        ***
- *** Copyright (C) 2004 by T. Serafini, G. Zanghirati, L. Zanni.            ***
+ *** Copyright (C) 2004-2008 by T. Serafini, G. Zanghirati, L. Zanni.       ***
  ***                                                                        ***
  ***                     COPYRIGHT NOTIFICATION                             ***
  ***                                                                        ***
@@ -48,7 +48,7 @@
  ***                                                                        ***
  *** This program is free software; you can redistribute it and/or modify   ***
  *** it under the terms of the GNU General Public License as published by   ***
- *** the Free Software Foundation; either version 2 of the License, or      ***
+ *** the Free Software Foundation; either version 3 of the License, or      ***
  *** (at your option) any later version.                                    ***
  ***                                                                        ***
  *** This program is distributed in the hope that it will be useful,        ***
@@ -66,13 +66,14 @@
  *** Date:     October, 2005                                                ***
  *** Revision: 1                                                            ***
  ***                                                                        ***
- *** SHOGUN adaptions  Written (W) 2006-2007 Soeren Sonnenburg              ***
+ *** SHOGUN adaptions  Written (W) 2006-2008 Soeren Sonnenburg              ***
  ******************************************************************************/
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <math.h>
 #include "classifier/svm/gpdt.h"
+#include "lib/io.h"
 
 #define maxvpm           30000  /* max number of method iterations allowed  */
 #define maxprojections   200
@@ -412,7 +413,7 @@ int gvpm(int Projector, int n, float *vecA, double *b, double c, double e,
       } // stopping rule based on the norm of d_k
   }
 
-  printf("GVPM exits after maxvpm = %d iterations.\n", maxvpm);
+  SG_SWARNING("GVPM exits after maxvpm = %d iterations.\n", maxvpm);
 
 Clean:
 
@@ -748,7 +749,7 @@ int FletcherAlg2A(int Projector, int n, float *vecA, double *b, double c,
       }
   }
 
-  printf("\nDai-Fletcher method exits after maxvpm = %d iterations.\n", maxvpm);
+  SG_SWARNING("Dai-Fletcher method exits after maxvpm = %d iterations.\n", maxvpm);
 
 Clean:
 
@@ -969,7 +970,7 @@ int ProjectDai(int n, int *a, double b, double *c, double l, double u,
 
   lam_ext = lambda;
   if (iter >= maxprojections)
-      printf("  error: Projector exits after max iterations: %d\n", iter);
+      SG_SERROR("Projector exits after max iterations: %d\n", iter);
 
   return (iter);
 }
@@ -1180,7 +1181,7 @@ int Pardalos(int n, int *iy, double e,   double *qk,
       else if ((a[i]<=xmin) && (xmax<=b[i]))
           x[i] = xx;
       else
-          printf("\nWarning: inner solver troubles...\n");
+          SG_SWARNING("Inner solver troubles...\n");
   }
 
   for (i = 0; i < n; i++)
