@@ -1,16 +1,16 @@
 /*
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * Purpose:     solves quadratic programming problem for pattern recognition
  *              for support vectors
  *
  * Written (W) 1997-1998 Alex J. Smola
- * Written (W) 1999-2007 Soeren Sonnenburg
- * Written (W) 1999-2007 Gunnar Raetsch
- * Copyright (C) 1997-2007 Fraunhofer Institute FIRST and Max-Planck-Society
+ * Written (W) 1999-2008 Soeren Sonnenburg
+ * Written (W) 1999-2008 Gunnar Raetsch
+ * Copyright (C) 1997-2008 Fraunhofer Institute FIRST and Max-Planck-Society
  */
 
 #include "lib/common.h"
@@ -25,7 +25,7 @@
 #include <stdio.h>
 #ifdef HAVE_LAPACK
 extern "C" {
-#ifndef DARWIN
+#ifdef HAVE_ATLAS
 #include <clapack.h>
 #endif
 }
@@ -108,7 +108,7 @@ bool choldc(double a[], int n, double p[])
 			{
 				if (sum <= 0.0)
 				{
-					nrerror("Choldc failed, matrix not positive definite");
+					SG_SERROR("Choldc failed, matrix not positive definite");
 					sum = 0.0;
 					return false;
 				}
@@ -566,7 +566,10 @@ int pr_loqo(int n, int m, double c[], double h_x[], double a[], double b[],
       
       /* and do it */
       if (!solve_reduced(n, m, h_x, h_y, a, delta_x, delta_y, c_x, c_y, workspace, PREDICTOR))
-	  	  return status=INCONSISTENT;
+	  {
+	  	  status=INCONSISTENT;
+		  goto exit_optimizer;
+	  }
       
       for (i=0; i<n; i++) {
 	/* backsubstitution */
@@ -634,6 +637,8 @@ int pr_loqo(int n, int m, double c[], double h_x[], double a[], double b[],
 	y[i] += alfa * delta_y[i];
     }
   }
+
+exit_optimizer: 
   if ((status == 1) && (verb >= STATUS)) {
 	  SG_SDEBUG("----------------------------------------------------------------------------------\n");
 	  SG_SDEBUG("optimization converged\n");
