@@ -1,9 +1,10 @@
 /*
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
- * Written (W) 2007 Vojtech Franc 
+ *
+ * Written (W) 2007-2008 Vojtech Franc
  */
 
 int qpssvm_solver(const void* (*get_col)(uint32_t),
@@ -18,5 +19,4 @@ int qpssvm_solver(const void* (*get_col)(uint32_t),
                   double tolrel,
                   double *QP,
                   double *QD,
-				  int (*qpssvmlib_print)(const char *format, ...),
                   uint32_t verb);
