@@ -1,12 +1,12 @@
 /*
  * this program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * Written (W) 2006 Christian Gehl
- * Written (W) 2006 Soeren Sonnenburg
- * Copyright (C) 1999-2007 Fraunhofer Institute FIRST and Max-Planck-Society
+ * Written (W) 2006-2008 Soeren Sonnenburg
+ * Copyright (C) 1999-2008 Fraunhofer Institute FIRST and Max-Planck-Society
  */
 
 #include "lib/config.h"
@@ -27,15 +27,15 @@
 #include <pthread.h>
 #endif
 
-CDistance::CDistance() 
-: CSGObject(), precomputed_matrix(NULL), precompute_matrix(false), 
+CDistance::CDistance()
+: CSGObject(), precomputed_matrix(NULL), precompute_matrix(false),
 	lhs(NULL), rhs(NULL)
 {
 }
 
 		
 CDistance::CDistance(CFeatures* p_lhs, CFeatures* p_rhs)
-:  precomputed_matrix(NULL), precompute_matrix(false), 
+: CSGObject(), precomputed_matrix(NULL), precompute_matrix(false),
 	lhs(NULL), rhs(NULL)
 {
 	init(p_lhs, p_rhs);
@@ -43,8 +43,8 @@ CDistance::CDistance(CFeatures* p_lhs, CFeatures* p_rhs)
 
 CDistance::~CDistance()
 {
-	delete[] precomputed_matrix ;
-	precomputed_matrix=NULL ;
+	delete[] precomputed_matrix;
+	precomputed_matrix=NULL;
 }
 
 bool CDistance::init(CFeatures* l, CFeatures* r)
@@ -54,8 +54,8 @@ bool CDistance::init(CFeatures* l, CFeatures* r)
 	ASSERT(r);
 
 	//make sure features are compatible
-	ASSERT(l->get_feature_class() == r->get_feature_class());
-	ASSERT(l->get_feature_type() == r->get_feature_type());
+	ASSERT(l->get_feature_class()==r->get_feature_class());
+	ASSERT(l->get_feature_type()==r->get_feature_type());
 
 	lhs=l;
 	rhs=r;
@@ -74,8 +74,8 @@ bool CDistance::load(CHAR* fname)
 bool CDistance::save(CHAR* fname)
 {
 	INT i=0;
-	INT num_left=get_lhs()->get_num_vectors();
-	INT num_right=get_rhs()->get_num_vectors();
+	INT num_left=lhs->get_num_vectors();
+	INT num_right=rhs->get_num_vectors();
 	KERNELCACHE_IDX num_total=num_left*num_right;
 
 	CFile f(fname, 'w', F_DREAL);
@@ -104,31 +104,30 @@ bool CDistance::save(CHAR* fname)
 
 void CDistance::remove_lhs()
 { 
+	SG_UNREF(lhs);
 	lhs = NULL;
 }
 
 /// takes all necessary steps if the rhs is removed from kernel
 void CDistance::remove_rhs()
 {
+	SG_UNREF(rhs);
 	rhs = NULL;
 }
 
 
 void CDistance::do_precompute_matrix()
 {
-	INT num_left=get_lhs()->get_num_vectors();
-	INT num_right=get_rhs()->get_num_vectors();
-
+	INT num_left=lhs->get_num_vectors();
+	INT num_right=rhs->get_num_vectors();
 	SG_INFO( "precomputing distance matrix (%ix%i)\n", num_left, num_right) ;
 
-	ASSERT(num_left == num_right) ;
-	ASSERT(get_lhs()==get_rhs()) ;
-	INT num=num_left ;
+	ASSERT(num_left==num_right);
+	ASSERT(lhs==rhs);
+	INT num=num_left;
 	
-	delete[] precomputed_matrix ;
-	precomputed_matrix=new SHORTREAL[num*(num+1)/2] ;
-
-	ASSERT(precomputed_matrix!=NULL) ;
+	delete[] precomputed_matrix;
+	precomputed_matrix=new SHORTREAL[num*(num+1)/2];
 
 	for (INT i=0; i<num; i++)
 	{
@@ -138,7 +137,7 @@ void CDistance::do_precompute_matrix()
 	}
 
 	SG_PROGRESS(num*num,0,num*num);
-	SG_INFO( "\ndone.\n") ;
+	SG_DONE();
 }
 
 void CDistance::get_distance_matrix(DREAL** dst, INT* m, INT* n)
@@ -146,8 +145,8 @@ void CDistance::get_distance_matrix(DREAL** dst, INT* m, INT* n)
 	ASSERT(dst && m && n);
 
 	DREAL* result = NULL;
-	CFeatures* f1 = get_lhs();
-	CFeatures* f2 = get_rhs();
+	CFeatures* f1 = lhs;
+	CFeatures* f2 = rhs;
 
 	if (f1 && f2)
 	{
@@ -156,13 +155,11 @@ void CDistance::get_distance_matrix(DREAL** dst, INT* m, INT* n)
 		*m=num_vec1;
 		*n=num_vec2;
 
-		LONG total_num = num_vec1 * num_vec2;
-		INT num_done = 0;
-		SG_PRINT( "returning distance matrix of size %dx%d\n", num_vec1, num_vec2);
+		LONG total_num=num_vec1*num_vec2;
+		INT num_done=0;
+		SG_DEBUG("returning distance matrix of size %dx%d\n", num_vec1, num_vec2);
 
-		SG_DEBUG( "returning distance matrix of size %dx%d\n", num_vec1, num_vec2);
-
-		result=new DREAL[total_num];
+		result=(DREAL*) malloc(total_num*sizeof(DREAL));
 		ASSERT(result);
 
 		if ( (f1 == f2) && (num_vec1 == num_vec2) )
@@ -202,7 +199,7 @@ void CDistance::get_distance_matrix(DREAL** dst, INT* m, INT* n)
 			}
 		}
 
-		SG_PRINT( "done.           \n");
+		SG_DONE();
 	}
 	else
       SG_ERROR( "no features assigned to distance\n");
@@ -213,28 +210,26 @@ void CDistance::get_distance_matrix(DREAL** dst, INT* m, INT* n)
 SHORTREAL* CDistance::get_distance_matrix_shortreal(int &num_vec1, int &num_vec2, SHORTREAL* target)
 {
 	SHORTREAL* result = NULL;
-	CFeatures* f1 = get_lhs();
-	CFeatures* f2 = get_rhs();
+	CFeatures* f1 = lhs;
+	CFeatures* f2 = rhs;
 
 	if (f1 && f2)
 	{
-		if (target && (num_vec1!=f1->get_num_vectors() || num_vec2!=f2->get_num_vectors()) )
-         SG_ERROR( "distance matrix does not fit into target\n");
+		if (target && (num_vec1!=f1->get_num_vectors() || num_vec2!=f2->get_num_vectors()))
+			SG_ERROR("distance matrix does not fit into target\n");
 
 		num_vec1=f1->get_num_vectors();
 		num_vec2=f2->get_num_vectors();
-		LONG total_num = num_vec1 * num_vec2;
-		int num_done = 0;
+		LONG total_num=num_vec1*num_vec2;
+		int num_done=0;
 
-		SG_DEBUG( "returning distance matrix of size %dx%d\n", num_vec1, num_vec2);
+		SG_DEBUG("returning distance matrix of size %dx%d\n", num_vec1, num_vec2);
 
 		if (target)
 			result=target;
 		else
 			result=new SHORTREAL[total_num];
 
-		ASSERT(result);
-
 		if ( (f1 == f2) && (num_vec1 == num_vec2) )
 		{
 			for (int i=0; i<num_vec1; i++)
@@ -272,7 +267,7 @@ SHORTREAL* CDistance::get_distance_matrix_shortreal(int &num_vec1, int &num_vec2
 			}
 		}
 
-		SG_PRINT( "done.           \n");
+		SG_DONE();
 	}
 	else
       SG_ERROR( "no features assigned to distance\n");
@@ -283,28 +278,26 @@ SHORTREAL* CDistance::get_distance_matrix_shortreal(int &num_vec1, int &num_vec2
 DREAL* CDistance::get_distance_matrix_real(int &num_vec1, int &num_vec2, DREAL* target)
 {
 	DREAL* result = NULL;
-	CFeatures* f1 = get_lhs();
-	CFeatures* f2 = get_rhs();
+	CFeatures* f1 = lhs;
+	CFeatures* f2 = rhs;
 
 	if (f1 && f2)
 	{
-		if (target && (num_vec1!=f1->get_num_vectors() || num_vec2!=f2->get_num_vectors()) )
-         SG_ERROR( "distance matrix does not fit into target\n");
+		if (target && (num_vec1!=f1->get_num_vectors() || num_vec2!=f2->get_num_vectors()))
+			SG_ERROR("distance matrix does not fit into target\n");
 
 		num_vec1=f1->get_num_vectors();
 		num_vec2=f2->get_num_vectors();
-		LONG total_num = num_vec1 * num_vec2;
-		int num_done = 0;
+		LONG total_num=num_vec1*num_vec2;
+		int num_done=0;
 
-		SG_DEBUG( "returning distance matrix of size %dx%d\n", num_vec1, num_vec2);
+		SG_DEBUG("returning distance matrix of size %dx%d\n", num_vec1, num_vec2);
 
 		if (target)
 			result=target;
 		else
 			result=new DREAL[total_num];
 
-		ASSERT(result);
-
 		if ( (f1 == f2) && (num_vec1 == num_vec2) )
 		{
 			for (int i=0; i<num_vec1; i++)
@@ -342,7 +335,7 @@ DREAL* CDistance::get_distance_matrix_real(int &num_vec1, int &num_vec2, DREAL*
 			}
 		}
 
-		SG_PRINT( "done.           \n");
+		SG_DONE();
 	}
 	else
       SG_ERROR( "no features assigned to distance\n");
