@@ -1,12 +1,12 @@
 /*
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * Written (W) 2006 Christian Gehl
- * Written (W) 2006 Soeren Sonnenburg
- * Copyright (C) 1999-2007 Fraunhofer Institute FIRST and Max-Planck-Society
+ * Written (W) 2006-2008 Soeren Sonnenburg
+ * Copyright (C) 1999-2008 Fraunhofer Institute FIRST and Max-Planck-Society
  */
 
 #ifndef _DISTANCE_MACHINE_H__
@@ -19,16 +19,33 @@
 
 #include <stdio.h>
 
+/** class DistanceMachine */
 class CDistanceMachine : public CClassifier
 {
 	public:
+		/** default constructor */
 		CDistanceMachine();
 		virtual ~CDistanceMachine();
 
-		inline void set_distance(CDistance* d) { distance=d; }
-		inline CDistance* get_distance() { return distance; }
+		/** set distance
+		 *
+		 * @param d distance to set
+		 */
+		inline void set_distance(CDistance* d)
+		{
+			SG_UNREF(distance);
+			SG_REF(d);
+			distance=d;
+		}
+
+		/** get distance
+		 *
+		 * @return distance
+		 */
+		inline CDistance* get_distance() { SG_REF(distance); return distance; }
 		
 	protected:
+		/** the distance */
 		CDistance* distance;
 };
 #endif
