@@ -1,11 +1,11 @@
 /*
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
- * Written (W) 1999-2007 Soeren Sonnenburg
- * Copyright (C) 1999-2007 Fraunhofer Institute FIRST and Max-Planck-Society
+ * Written (W) 1999-2008 Soeren Sonnenburg
+ * Copyright (C) 1999-2008 Fraunhofer Institute FIRST and Max-Planck-Society
  */
 
 #ifndef _REALDISTANCE_H__
@@ -14,13 +14,19 @@
 #include "distance/SimpleDistance.h"
 #include "lib/common.h"
 
+/** class RealDistance */
 class CRealDistance : public CSimpleDistance<DREAL>
 {
 	public:
-		CRealDistance() : CSimpleDistance<DREAL>()
-		{
-		}
+		/** default constructor */
+		CRealDistance() : CSimpleDistance<DREAL>() {}
 
+		/** init distance
+		 *
+		 * @param l features of left-hand side
+		 * @param r features of right-hand side
+		 * @return if init was successful
+		 */
 		virtual bool init(CFeatures* l, CFeatures* r)
 		{
 			CSimpleDistance<DREAL>::init(l,r);
@@ -31,8 +37,10 @@ class CRealDistance : public CSimpleDistance<DREAL>
 			return true;
 		}
 
-		/** return feature type the distance can deal with
-		  */
+		/** get feature type the distance can deal with
+		 *
+		 * @return feature type DREAL
+		 */
 		inline virtual EFeatureType get_feature_type() { return F_DREAL; }
 };
 #endif
