@@ -1,17 +1,18 @@
 /*
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
- * Written (W) 1999-2007 Soeren Sonnenburg
- * Copyright (C) 1999-2007 Fraunhofer Institute FIRST and Max-Planck-Society
+ * Written (W) 1999-2008 Soeren Sonnenburg
+ * Copyright (C) 1999-2008 Fraunhofer Institute FIRST and Max-Planck-Society
  */
 
 #include "distributions/Distribution.h"
 #include "lib/Mathematics.h"
 
-CDistribution::CDistribution() : CSGObject(), features(NULL), pseudo_count(0)
+CDistribution::CDistribution()
+: CSGObject(), features(NULL), pseudo_count(1e-10)
 {
 }
 
@@ -31,17 +32,17 @@ DREAL CDistribution::get_log_likelihood_sample()
 	return sum/features->get_num_vectors();
 }
 
-DREAL* CDistribution::get_log_likelihood_all_examples()
+void CDistribution::get_log_likelihood(DREAL **dst, INT *num)
 {
 	ASSERT(features);
 
-	DREAL* output=new DREAL[features->get_num_vectors()];
-	ASSERT(output);
+	*num=features->get_num_vectors();
+	size_t sz=sizeof(DREAL)*(*num);
+	*dst=(DREAL*) malloc(sz);
+	ASSERT(dst);
 
-	for (INT i=0; i<features->get_num_vectors(); i++)
-		output[i]=get_log_likelihood_example(i);
-
-	return output;
+	for (INT i=0; i<(*num); i++)
+		*(*dst+i)=get_log_likelihood_example(i);
 }
 
 INT CDistribution::get_num_relevant_model_parameters()
