@@ -1,25 +1,32 @@
 /*
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
- * Written (W) 1999-2007 Soeren Sonnenburg
- * Written (W) 1999-2007 Gunnar Raetsch
- * Copyright (C) 1999-2007 Fraunhofer Institute FIRST and Max-Planck-Society
+ * Written (W) 1999-2008 Soeren Sonnenburg
+ * Written (W) 1999-2008 Gunnar Raetsch
+ * Copyright (C) 1999-2008 Fraunhofer Institute FIRST and Max-Planck-Society
  */
 
 #include "distributions/histogram/Histogram.h"
 #include "lib/common.h"
-#include "features/WordFeatures.h"
+#include "features/StringFeatures.h"
 #include "lib/io.h"
 #include "lib/Mathematics.h"
 
 
 CHistogram::CHistogram()
+: CDistribution()
 {
 	hist=new DREAL[1<<16];
-	ASSERT(hist);
+}
+
+CHistogram::CHistogram(CStringFeatures<WORD> *f)
+: CDistribution()
+{
+	hist=new DREAL[1<<16];
+	features=f;
 }
 
 CHistogram::~CHistogram()
@@ -34,7 +41,7 @@ bool CHistogram::train()
 	INT i;
 
 	ASSERT(features);
-	ASSERT(features->get_feature_class()==C_SIMPLE);
+	ASSERT(features->get_feature_class()==C_STRING);
 	ASSERT(features->get_feature_type()==F_WORD);
 
 	for (i=0; i< (INT) (1<<16); i++)
@@ -43,15 +50,11 @@ bool CHistogram::train()
 	for (vec=0; vec<features->get_num_vectors(); vec++)
 	{
 		INT len;
-		bool to_free;
 
-		WORD* vector=((CWordFeatures*) features)->get_feature_vector(vec, len, to_free);
+		WORD* vector=((CStringFeatures<WORD>*) features)->get_feature_vector(vec, len);
 
 		for (feat=0; feat<len ; feat++)
-		{
 			hist[vector[feat]]++;
-		}
-		((CWordFeatures*) features)->free_feature_vector(vector, len, to_free);
 	}
 
 	for (i=0; i< (INT) (1<<16); i++)
@@ -63,40 +66,34 @@ bool CHistogram::train()
 DREAL CHistogram::get_log_likelihood_example(INT num_example)
 {
 	ASSERT(features);
-	ASSERT(features->get_feature_class()==C_SIMPLE);
+	ASSERT(features->get_feature_class()==C_STRING);
 	ASSERT(features->get_feature_type()==F_WORD);
 
 	INT len;
-	bool to_free;
 	DREAL loglik=0;
 
-	WORD* vector=((CWordFeatures*) features)->get_feature_vector(num_example, len, to_free);
+	WORD* vector=((CStringFeatures<WORD>*) features)->get_feature_vector(num_example, len);
 
 	for (INT i=0; i<len; i++)
-	{
 		loglik+=hist[vector[i]];
-	}
-
-	((CWordFeatures*) features)->free_feature_vector(vector, len, to_free);
 
 	return loglik;
 }
 
-DREAL CHistogram::get_log_derivative(INT num_example, INT param_num)
+DREAL CHistogram::get_log_derivative(INT num_param, INT num_example)
 {
-	if (hist[param_num] < CMath::ALMOST_NEG_INFTY)
+	if (hist[num_param] < CMath::ALMOST_NEG_INFTY)
 		return -CMath::INFTY;
 	else
 	{
 		ASSERT(features);
-		ASSERT(features->get_feature_class()==C_SIMPLE);
+		ASSERT(features->get_feature_class()==C_STRING);
 		ASSERT(features->get_feature_type()==F_WORD);
 
 		INT len;
-		bool to_free;
 		DREAL deriv=0;
 
-		WORD* vector=((CWordFeatures*) features)->get_feature_vector(num_example, len, to_free);
+		WORD* vector=((CStringFeatures<WORD>*) features)->get_feature_vector(num_example, len);
 
 		INT num_occurences=0;
 
@@ -104,22 +101,44 @@ DREAL CHistogram::get_log_derivative(INT num_example, INT param_num)
 		{
 			deriv+=hist[vector[i]];
 
-			if (vector[i]==param_num)
+			if (vector[i]==num_param)
 				num_occurences++;
 		}
 
 		if (num_occurences>0)
-			deriv+=log(num_occurences)-hist[param_num];
+			deriv+=log(num_occurences)-hist[num_param];
 		else
 			deriv=-CMath::INFTY;
 
-		((CWordFeatures*) features)->free_feature_vector(vector, len, to_free);
-
 		return deriv;
 	}
 }
 
-DREAL CHistogram::get_log_model_parameter(INT param_num)
+DREAL CHistogram::get_log_model_parameter(INT num_param)
 {
-	return hist[param_num];
+	return hist[num_param];
 }
+
+bool CHistogram::set_histogram(DREAL* src, INT num)
+{
+	ASSERT(num==get_num_model_parameters());
+
+	delete[] hist;
+	hist=new DREAL[num];
+	for (INT i=0; i<num; i++) {
+		hist[i]=src[i];
+	}
+
+	return true;
+}
+
+void CHistogram::get_histogram(DREAL** dst, INT* num)
+{
+	*num=get_num_model_parameters();
+	size_t sz=sizeof(*hist)*(*num);
+	*dst=(DREAL*) malloc(sz);
+	ASSERT(dst);
+
+	memcpy(*dst, hist, sz);
+}
+
