@@ -1,11 +1,11 @@
 /*
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
- * Written (W) 2006 Soeren Sonnenburg
- * Copyright (C) 2006 Fraunhofer Institute FIRST and Max-Planck-Society
+ * Written (W) 2006-2008 Soeren Sonnenburg
+ * Copyright (C) 2006-2008 Fraunhofer Institute FIRST and Max-Planck-Society
  */
 
 #include <string.h>
@@ -20,14 +20,17 @@ const BYTE CAlphabet::B_C=1;
 const BYTE CAlphabet::B_G=2;
 const BYTE CAlphabet::B_T=3;
 const BYTE CAlphabet::MAPTABLE_UNDEF=0xff;
-const CHAR* CAlphabet::alphabet_names[10]={"DNA", "RNA", "PROTEIN", "ALPHANUM", "CUBE", "RAW", "IUPAC_NUCLEIC_ACID", "IUPAC_AMINO_ACID", "NONE", "UNKNOWN"};
+const CHAR* CAlphabet::alphabet_names[11]={"DNA", "RAWDNA", "RNA", "PROTEIN", "ALPHANUM", "CUBE", "RAW", "IUPAC_NUCLEIC_ACID", "IUPAC_AMINO_ACID", "NONE", "UNKNOWN"};
 
-CAlphabet::CAlphabet(CHAR* al, INT len) : CSGObject()
+CAlphabet::CAlphabet(CHAR* al, INT len)
+: CSGObject()
 {
 	E_ALPHABET alpha=NONE;
 
 	if (len>=(INT) strlen("DNA") && !strncmp(al, "DNA", strlen("DNA")))
 		alpha = DNA;
+	else if (len>=(INT) strlen("RAWDNA") && !strncmp(al, "RAWDNA", strlen("RAWDNA")))
+		alpha = RAWDNA;
 	else if (len>=(INT) strlen("RNA") && !strncmp(al, "RNA", strlen("RNA")))
 		alpha = RNA;
 	else if (len>=(INT) strlen("PROTEIN") && !strncmp(al, "PROTEIN", strlen("PROTEIN")))
@@ -36,7 +39,7 @@ CAlphabet::CAlphabet(CHAR* al, INT len) : CSGObject()
 		alpha = ALPHANUM;
 	else if (len>=(INT) strlen("CUBE") && !strncmp(al, "CUBE", strlen("CUBE")))
 		alpha = CUBE;
-	else if (len>=(INT) strlen("BYTE") && !strncmp(al, "BYTE", strlen("BYTE")) || 
+	else if ((len>=(INT) strlen("BYTE") && !strncmp(al, "BYTE", strlen("BYTE"))) || 
 			(len>=(INT) strlen("RAW") && !strncmp(al, "RAW", strlen("RAW"))))
 		alpha = RAWBYTE;
 	else if (len>=(INT) strlen("IUPAC_NUCLEIC_ACID") && !strncmp(al, "IUPAC_NUCLEIC_ACID", strlen("IUPAC_NUCLEIC_ACID")))
@@ -50,15 +53,18 @@ CAlphabet::CAlphabet(CHAR* al, INT len) : CSGObject()
 	set_alphabet(alpha);
 }
 
-CAlphabet::CAlphabet(E_ALPHABET alpha) : CSGObject()
+CAlphabet::CAlphabet(E_ALPHABET alpha)
+: CSGObject()
 {
 	set_alphabet(alpha);
 }
 
-CAlphabet::CAlphabet(CAlphabet* a) : CSGObject()
+CAlphabet::CAlphabet(CAlphabet* a)
+: CSGObject()
 {
+	ASSERT(a);
 	set_alphabet(a->get_alphabet());
-    copy_histogram(a);
+	copy_histogram(a);
 }
 
 CAlphabet::~CAlphabet()
@@ -73,6 +79,7 @@ bool CAlphabet::set_alphabet(E_ALPHABET alpha)
 	switch (alphabet)
 	{
 		case DNA:
+		case RAWDNA:
 			num_symbols = 4;
 			break;
 		case RNA:
@@ -210,6 +217,17 @@ void CAlphabet::init_map_table()
 			maptable_to_char[B_G]='G';
 			maptable_to_char[B_T]='T';
 			break;
+		case RAWDNA:
+			{
+				//identity
+				for (i=0; i<4; i++)
+				{
+					valid_chars[i]=1;
+					maptable_to_bin[i]=i;
+					maptable_to_char[i]=i;
+				}
+			}
+			break;
 
 		case RNA:
 			valid_chars[(BYTE) 'A']=1;
@@ -522,33 +540,36 @@ const CHAR* CAlphabet::get_alphabet_name(E_ALPHABET alphabet)
 		case DNA:
 			idx=0;
 			break;
-		case RNA:
+		case RAWDNA:
 			idx=1;
 			break;
-		case PROTEIN:
+		case RNA:
 			idx=2;
 			break;
-		case ALPHANUM:
+		case PROTEIN:
 			idx=3;
 			break;
-		case CUBE:
+		case ALPHANUM:
 			idx=4;
 			break;
-		case RAWBYTE:
+		case CUBE:
 			idx=5;
 			break;
-		case IUPAC_NUCLEIC_ACID:
+		case RAWBYTE:
 			idx=6;
 			break;
-		case IUPAC_AMINO_ACID:
+		case IUPAC_NUCLEIC_ACID:
 			idx=7;
 			break;
-		case NONE:
+		case IUPAC_AMINO_ACID:
 			idx=8;
 			break;
-		default:
+		case NONE:
 			idx=9;
 			break;
+		default:
+			idx=10;
+			break;
 	}
 	return alphabet_names[idx];
 }
