@@ -1,38 +1,43 @@
 /*
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
- * Written (W) 1999-2007 Soeren Sonnenburg
- * Copyright (C) 1999-2007 Fraunhofer Institute FIRST and Max-Planck-Society
+ * Written (W) 1999-2008 Soeren Sonnenburg
+ * Copyright (C) 1999-2008 Fraunhofer Institute FIRST and Max-Planck-Society
  */
 
 #include "features/CharFeatures.h"
 #include "lib/common.h"
 #include "lib/File.h"
 
-CCharFeatures::CCharFeatures(E_ALPHABET a, INT size) : CSimpleFeatures<CHAR>(size)
+CCharFeatures::CCharFeatures(E_ALPHABET a, INT size)
+: CSimpleFeatures<CHAR>(size)
 {
 	alphabet=new CAlphabet(a);
 }
 
-CCharFeatures::CCharFeatures(CAlphabet* a, INT size) : CSimpleFeatures<CHAR>(size)
+CCharFeatures::CCharFeatures(CAlphabet* a, INT size)
+: CSimpleFeatures<CHAR>(size)
 {
 	alphabet= new CAlphabet(a);
 }
 
-CCharFeatures::CCharFeatures(const CCharFeatures & orig) : CSimpleFeatures<CHAR>(orig)
+CCharFeatures::CCharFeatures(const CCharFeatures & orig)
+: CSimpleFeatures<CHAR>(orig)
 {
 	alphabet=orig.alphabet;
 }
 
-CCharFeatures::CCharFeatures(E_ALPHABET a, CHAR* fm, INT num_feat, INT num_vec) : CSimpleFeatures<CHAR>(fm, num_feat, num_vec)
+CCharFeatures::CCharFeatures(E_ALPHABET a, CHAR* fm, INT num_feat, INT num_vec)
+: CSimpleFeatures<CHAR>(fm, num_feat, num_vec)
 {
 	alphabet=new CAlphabet(a);
 }
 
-CCharFeatures::CCharFeatures(E_ALPHABET a, CHAR* fname) : CSimpleFeatures<CHAR>(fname)
+CCharFeatures::CCharFeatures(E_ALPHABET a, CHAR* fname)
+: CSimpleFeatures<CHAR>(fname)
 {
 	alphabet=new CAlphabet(a);
 	load(fname);
@@ -51,6 +56,7 @@ bool CCharFeatures::load(CHAR* fname)
 	LONG linelen=0;
 
 	CFile f(fname, 'r', F_CHAR);
+	free_feature_matrix();
 	feature_matrix=f.load_char_data(NULL, length);
 
     if (f.is_ok())
