@@ -1,19 +1,20 @@
 /*
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
- * Written (W) 1999-2007 Soeren Sonnenburg
- * Written (W) 1999-2007 Gunnar Raetsch
- * Copyright (C) 1999-2007 Fraunhofer Institute FIRST and Max-Planck-Society
+ * Written (W) 1999-2008 Soeren Sonnenburg
+ * Written (W) 1999-2008 Gunnar Raetsch
+ * Copyright (C) 1999-2008 Fraunhofer Institute FIRST and Max-Planck-Society
  */
 
 #include "features/FKFeatures.h"
 #include "features/StringFeatures.h"
 #include "lib/io.h"
 
-CFKFeatures::CFKFeatures(INT size, CHMM* p, CHMM* n) : CRealFeatures(size)
+CFKFeatures::CFKFeatures(INT size, CHMM* p, CHMM* n)
+: CRealFeatures(size)
 {
 	pos_prob=NULL;
 	neg_prob=NULL;
@@ -21,13 +22,15 @@ CFKFeatures::CFKFeatures(INT size, CHMM* p, CHMM* n) : CRealFeatures(size)
 	set_models(p,n);
 }
 
- CFKFeatures::CFKFeatures(const CFKFeatures &orig): 
-	CRealFeatures(orig), pos(orig.pos), neg(orig.neg), weight_a(orig.weight_a)
+CFKFeatures::CFKFeatures(const CFKFeatures &orig)
+: CRealFeatures(orig), pos(orig.pos), neg(orig.neg), weight_a(orig.weight_a)
 { 
 }
 
 CFKFeatures::~CFKFeatures()
 {
+	SG_UNREF(pos);
+	SG_UNREF(neg);
 }
 
 double CFKFeatures::deriv_a(double a, INT dimension)
@@ -85,8 +88,6 @@ double CFKFeatures::set_opt_a(double a)
 		SG_INFO( "estimating a.\n");
 		pos_prob=new double[pos->get_observations()->get_num_vectors()];
 		neg_prob=new double[pos->get_observations()->get_num_vectors()];
-		ASSERT(pos_prob!=NULL);
-		ASSERT(neg_prob!=NULL);
 		for (INT i=0; i<pos->get_observations()->get_num_vectors(); i++)
 		{
 			pos_prob[i]=pos->model_probability(i) ;
@@ -96,7 +97,7 @@ double CFKFeatures::set_opt_a(double a)
 		double la=0;
 		double ua=1;
 		a=(la+ua)/2;
-		while (ua-la>1e-6)
+		while (CMath::abs(ua-la)>1e-6)
 		{
 			double da=deriv_a(a);
 			if (da>0)
@@ -119,14 +120,15 @@ double CFKFeatures::set_opt_a(double a)
 
 void CFKFeatures::set_models(CHMM* p, CHMM* n)
 {
-	ASSERT(p!=NULL && n!=NULL);
+	ASSERT(p && n);
+	SG_REF(p);
+	SG_REF(n);
 
 	pos=p; 
 	neg=n;
 	set_num_vectors(0);
 
-	delete[] feature_matrix  ;
-	feature_matrix=NULL ;
+	free_feature_matrix();
 
 	SG_INFO( "pos_feat=[%i,%i,%i,%i],neg_feat=[%i,%i,%i,%i]\n", pos->get_N(), pos->get_N(), pos->get_N()*pos->get_N(), pos->get_N()*pos->get_M(), neg->get_N(), neg->get_N(), neg->get_N()*neg->get_N(), neg->get_N()*neg->get_M()) ;
 
@@ -197,13 +199,19 @@ void CFKFeatures::compute_feature_vector(DREAL* featurevector, INT num, INT& len
 
 DREAL* CFKFeatures::set_feature_matrix()
 {
-	INT len=0;
+	ASSERT(pos);
+	ASSERT(pos->get_observations());
+	ASSERT(neg);
+	ASSERT(neg->get_observations());
 
+	INT len=0;
 	num_features=1+ pos->get_N()*(1+pos->get_N()+1+pos->get_M()) + neg->get_N()*(1+neg->get_N()+1+neg->get_M());
 
 	num_vectors=pos->get_observations()->get_num_vectors();
+	ASSERT(num_vectors);
+
 	SG_INFO( "allocating FK feature cache of size %.2fM\n", sizeof(double)*num_features*num_vectors/1024.0/1024.0);
-	delete[] feature_matrix;
+	free_feature_matrix();
 	feature_matrix=new DREAL[num_features*num_vectors];
 
 	SG_INFO( "calculating FK feature matrix\n");
@@ -211,14 +219,14 @@ DREAL* CFKFeatures::set_feature_matrix()
 	for (INT x=0; x<num_vectors; x++)
 	{
 		if (!(x % (num_vectors/10+1)))
-			printf("%02d%%.", (int) (100.0*x/num_vectors));
+			SG_DEBUG("%02d%%.", (int) (100.0*x/num_vectors));
 		else if (!(x % (num_vectors/200+1)))
-			printf(".");
+			SG_DEBUG(".");
 
 		compute_feature_vector(&feature_matrix[x*num_features], x, len);
 	}
 
-	printf(".done.\n");
+	SG_DONE();
 	
 	num_vectors=get_num_vectors() ;
 	num_features=get_num_features() ;
