@@ -1,12 +1,12 @@
 /*
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
- * Written (W) 1999-2007 Soeren Sonnenburg
- * Written (W) 1999-2007 Gunnar Raetsch
- * Copyright (C) 1999-2007 Fraunhofer Institute FIRST and Max-Planck-Society
+ * Written (W) 1999-2008 Soeren Sonnenburg
+ * Written (W) 1999-2008 Gunnar Raetsch
+ * Copyright (C) 1999-2008 Fraunhofer Institute FIRST and Max-Planck-Society
  */
 
 #include "features/Features.h"
@@ -15,19 +15,24 @@
 
 #include <string.h>
 
-CFeatures::CFeatures(INT size) : CSGObject(), cache_size(size), preproc(NULL), num_preproc(0), preprocessed(NULL) 
+CFeatures::CFeatures(INT size)
+: CSGObject(), cache_size(size), preproc(NULL), num_preproc(0),
+	preprocessed(NULL)
 {
 	SG_INFO("Feature object created (%p)\n",this);
 }
 
-CFeatures::CFeatures(const CFeatures& orig) : CSGObject(orig), preproc(orig.preproc), num_preproc(orig.num_preproc), preprocessed(orig.preprocessed)
+CFeatures::CFeatures(const CFeatures& orig)
+: CSGObject(orig), preproc(orig.preproc),
+	num_preproc(orig.num_preproc), preprocessed(orig.preprocessed)
 {
 	preprocessed=new bool[orig.num_preproc];
-	ASSERT(preprocessed);
 	memcpy(preprocessed, orig.preprocessed, sizeof(bool)*orig.num_preproc);
 }
 
-CFeatures::CFeatures(CHAR* fname) : cache_size(0), preproc(NULL), num_preproc(0), preprocessed(false)
+CFeatures::CFeatures(CHAR* fname)
+: CSGObject(), cache_size(0), preproc(NULL), num_preproc(0),
+	preprocessed(false)
 {
 	load(fname);
 	SG_INFO("Feature object loaded (%p)\n",this) ;
