@@ -1,12 +1,12 @@
 /*
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
- * Written (W) 1999-2007 Soeren Sonnenburg
- * Written (W) 1999-2007 Gunnar Raetsch
- * Copyright (C) 1999-2007 Fraunhofer Institute FIRST and Max-Planck-Society
+ * Written (W) 1999-2008 Soeren Sonnenburg
+ * Written (W) 1999-2008 Gunnar Raetsch
+ * Copyright (C) 1999-2008 Fraunhofer Institute FIRST and Max-Planck-Society
  */
 
 #include "features/Labels.h"
@@ -15,22 +15,32 @@
 #include "lib/io.h"
 #include "lib/Mathematics.h"
 
-CLabels::CLabels() : CSGObject()
+CLabels::CLabels()
+: CSGObject()
 {
 	labels = NULL;
 	num_labels = 0;
 }
 
-CLabels::CLabels(INT num_lab) : CSGObject(), num_labels(num_lab)
+CLabels::CLabels(INT num_lab)
+: CSGObject(), num_labels(num_lab)
 {
 	labels=new DREAL[num_lab];
-	ASSERT(labels);
-
 	for (INT i=0; i<num_lab; i++)
 		labels[i]=0;
 }
 
-CLabels::CLabels(CHAR* fname) : CSGObject()
+CLabels::CLabels(DREAL* p_labels, INT len)
+: CSGObject()
+{
+	labels = NULL;
+	num_labels = 0;
+
+    set_labels(p_labels, len);
+}
+
+CLabels::CLabels(CHAR* fname)
+: CSGObject()
 {
 	num_labels=0;
 	labels=NULL;
@@ -48,18 +58,18 @@ CLabels::~CLabels()
 void CLabels::set_labels(DREAL* p_labels, INT len)
 {
 	ASSERT(len>0);
-	num_labels = len;
+	num_labels=len;
 
-	this->labels = new DREAL[len];
-	ASSERT(this->labels);
+	delete[] labels;
+	labels=new DREAL[len];
 
 	for (INT i=0; i<len; i++)
-		this->labels[i] = p_labels[i];
+		labels[i]=p_labels[i];
 }
 
 bool CLabels::is_two_class_labeling()
 {
-	ASSERT(this->labels);
+	ASSERT(labels);
 
 	for (INT i=0; i<num_labels; i++)
 	{
@@ -81,6 +91,8 @@ INT CLabels::get_num_classes()
 	for (INT i=0; i<n; i++)
 		num_classes=CMath::max(num_classes,lab[i]);
 
+	delete[] lab;
+
 	return num_classes+1;
 }
 
@@ -107,7 +119,7 @@ void CLabels::get_labels(DREAL** p_labels, INT* len)
 
 	if (num_labels>0)
 	{
-		*p_labels=new DREAL[num_labels];
+		*p_labels=(DREAL*) malloc(sizeof(DREAL)*num_labels);
 
 		for (INT i=0; i<num_labels; i++)
 			(*p_labels)[i]=get_label(i);
@@ -132,9 +144,7 @@ INT* CLabels::get_int_labels(INT &len)
 void CLabels::set_int_labels(INT * mylabels, INT len)
 {
 	num_labels = len ;
-#ifndef HAVE_SWIG
 	delete[] labels ;
-#endif
 	
 	labels = new DREAL[num_labels] ;
 	for (INT i=0; i<num_labels; i++)
