@@ -1,12 +1,12 @@
 /*
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
- * Written (W) 1999-2007 Soeren Sonnenburg
- * Written (W) 1999-2007 Gunnar Raetsch
- * Copyright (C) 1999-2007 Fraunhofer Institute FIRST and Max-Planck-Society
+ * Written (W) 1999-2008 Soeren Sonnenburg
+ * Written (W) 1999-2008 Gunnar Raetsch
+ * Copyright (C) 1999-2008 Fraunhofer Institute FIRST and Max-Planck-Society
  */
 
 #include "features/RealFeatures.h"
@@ -19,6 +19,7 @@ bool CRealFeatures::load(CHAR* fname)
     num_features=0;
 	CFile f(fname, 'r', F_DREAL);
 	LONG numf=0 ;
+	free_feature_matrix();
 	feature_matrix=f.load_real_data(NULL, numf);
 	num_features=numf;
 
@@ -63,21 +64,18 @@ bool CRealFeatures::Align_char_features(CCharFeatures* cf, CCharFeatures* Ref, D
 {
 	ASSERT(cf);
 
-	num_vectors  = cf->get_num_vectors();
-	num_features = Ref->get_num_vectors();
+	num_vectors=cf->get_num_vectors();
+	num_features=Ref->get_num_vectors();
 
 	INT len=num_vectors*num_features;
-	delete[] feature_matrix;
+	free_feature_matrix();
 	feature_matrix=new DREAL[len];
-	ASSERT(feature_matrix);
-
-	INT num_cf_feat;
-	INT num_cf_vec;
-	INT num_ref_feat;
-	INT num_ref_vec;
-
-	CHAR* fm_cf  = cf->get_feature_matrix(num_cf_feat, num_cf_vec);
-	CHAR* fm_ref = Ref->get_feature_matrix(num_ref_feat, num_ref_vec);
+	INT num_cf_feat=0;
+	INT num_cf_vec=0;
+	INT num_ref_feat=0;
+	INT num_ref_vec=0;
+	CHAR* fm_cf=cf->get_feature_matrix(num_cf_feat, num_cf_vec);
+	CHAR* fm_ref=Ref->get_feature_matrix(num_ref_feat, num_ref_vec);
 
 	ASSERT(num_cf_vec==num_vectors);
 	ASSERT(num_ref_vec==num_features);
