@@ -1,29 +1,30 @@
 /*
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
- * Written (W) 1999-2007 Soeren Sonnenburg
- * Written (W) 1999-2007 Gunnar Raetsch
- * Copyright (C) 1999-2007 Fraunhofer Institute FIRST and Max-Planck-Society
+ * Written (W) 1999-2008 Soeren Sonnenburg
+ * Written (W) 1999-2008 Gunnar Raetsch
+ * Copyright (C) 1999-2008 Fraunhofer Institute FIRST and Max-Planck-Society
  */
 
 #include "features/TOPFeatures.h"
 #include "lib/io.h"
 #include "lib/Mathematics.h"
 
-CTOPFeatures::CTOPFeatures(INT size, CHMM* p, CHMM* n, bool neglin, bool poslin) : CRealFeatures(size), neglinear(neglin), poslinear(poslin)
+CTOPFeatures::CTOPFeatures(INT size, CHMM* p, CHMM* n, bool neglin, bool poslin)
+: CRealFeatures(size), neglinear(neglin), poslinear(poslin)
 {
 	memset(&pos_relevant_indizes, 0, sizeof(pos_relevant_indizes));
 	memset(&neg_relevant_indizes, 0, sizeof(neg_relevant_indizes));
 	set_models(p,n);
 }
 
-CTOPFeatures::CTOPFeatures(const CTOPFeatures &orig): 
-CRealFeatures(orig), pos(orig.pos), neg(orig.neg), neglinear(orig.neglinear), poslinear(orig.poslinear)
-{ 
-
+CTOPFeatures::CTOPFeatures(const CTOPFeatures &orig)
+: CRealFeatures(orig), pos(orig.pos), neg(orig.neg), neglinear(orig.neglinear),
+	poslinear(orig.poslinear)
+{
 }
 
 CTOPFeatures::~CTOPFeatures()
@@ -41,11 +42,16 @@ CTOPFeatures::~CTOPFeatures()
 	delete[] neg_relevant_indizes.idx_a_rows;
 	delete[] neg_relevant_indizes.idx_b_cols;
 	delete[] neg_relevant_indizes.idx_b_rows;
+
+	SG_UNREF(pos);
+	SG_UNREF(neg);
 }
 
 void CTOPFeatures::set_models(CHMM* p, CHMM* n)
 {
-	ASSERT(p!=NULL && n!=NULL);
+	ASSERT(p && n);
+	SG_REF(p);
+	SG_REF(n);
 
 	pos=p; 
 	neg=n;
@@ -171,6 +177,9 @@ DREAL* CTOPFeatures::set_feature_matrix()
 	INT len=0;
 
 	num_features=get_num_features();
+	ASSERT(num_features);
+	ASSERT(pos);
+	ASSERT(pos->get_observations());
 
 	num_vectors=pos->get_observations()->get_num_vectors();
 	SG_INFO( "allocating top feature cache of size %.2fM\n", sizeof(double)*num_features*num_vectors/1024.0/1024.0);
@@ -187,14 +196,14 @@ DREAL* CTOPFeatures::set_feature_matrix()
 	for (INT x=0; x<num_vectors; x++)
 	{
 		if (!(x % (num_vectors/10+1)))
-			SG_PRINT( "%02d%%.", (int) (100.0*x/num_vectors));
+			SG_DEBUG( "%02d%%.", (int) (100.0*x/num_vectors));
 		else if (!(x % (num_vectors/200+1)))
-			SG_PRINT( ".");
+			SG_DEBUG( ".");
 
 		compute_feature_vector(&feature_matrix[x*num_features], x, len);
 	}
 
-	SG_PRINT( ".done.\n");
+	SG_DONE();
 
 	num_vectors=get_num_vectors() ;
 	num_features=get_num_features() ;
