@@ -1,27 +1,32 @@
 /*
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
- * Written (W) 1999-2007 Soeren Sonnenburg
- * Written (W) 1999-2007 Gunnar Raetsch
- * Copyright (C) 1999-2007 Fraunhofer Institute FIRST and Max-Planck-Society
+ * Written (W) 1999-2008 Soeren Sonnenburg
+ * Written (W) 1999-2008 Gunnar Raetsch
+ * Copyright (C) 1999-2008 Fraunhofer Institute FIRST and Max-Planck-Society
  */
 
 #include "features/WordFeatures.h"
 #include "features/CharFeatures.h"
 #include "lib/File.h"
 
-CWordFeatures::CWordFeatures(INT size, INT num_sym) : CSimpleFeatures<WORD>(size), num_symbols(num_sym),original_num_symbols(num_sym),order(0),symbol_mask_table(NULL)
+CWordFeatures::CWordFeatures(INT size, INT num_sym)
+: CSimpleFeatures<WORD>(size), num_symbols(num_sym),
+	original_num_symbols(num_sym), order(0), symbol_mask_table(NULL)
 {
 }
 
-CWordFeatures::CWordFeatures(const CWordFeatures & orig) : CSimpleFeatures<WORD>(orig)
+CWordFeatures::CWordFeatures(const CWordFeatures & orig)
+: CSimpleFeatures<WORD>(orig)
 {
 }
 
-CWordFeatures::CWordFeatures(CHAR* fname, INT num_sym) : CSimpleFeatures<WORD>(fname), num_symbols(num_sym),original_num_symbols(num_sym),order(0),symbol_mask_table(NULL)
+CWordFeatures::CWordFeatures(CHAR* fname, INT num_sym)
+: CSimpleFeatures<WORD>(fname), num_symbols(num_sym),
+	original_num_symbols(num_sym), order(0), symbol_mask_table(NULL)
 {
 }
 
@@ -47,11 +52,8 @@ bool CWordFeatures::obtain_from_char_features(CCharFeatures* cf, INT start, INT
 	INT len=num_vectors*num_features;
 	delete[] feature_matrix;
 	feature_matrix=new WORD[len];
-	ASSERT(feature_matrix);
-
-	INT num_cf_feat;
-	INT num_cf_vec;
-
+	INT num_cf_feat=0;
+	INT num_cf_vec=0;
 	CHAR* fm=cf->get_feature_matrix(num_cf_feat, num_cf_vec);
 
 	ASSERT(num_cf_vec==num_vectors);
@@ -99,7 +101,7 @@ bool CWordFeatures::obtain_from_char_features(CCharFeatures* cf, INT start, INT
 	if (start+gap!=0)
 	{
 		// condensing feature matrix ... 
-		ASSERT(start+gap>=0) ;
+		ASSERT(start+gap>=0);
 		for (INT line=0; line<num_vectors; line++)
 			for (INT j=0; j<num_features-start-gap; j++)
 				feature_matrix[line*(num_features-(start+gap))+j]=feature_matrix[line*num_features+j] ;
@@ -132,7 +134,7 @@ bool CWordFeatures::obtain_from_char_features(CCharFeatures* cf, INT start, INT
 
 void CWordFeatures::translate_from_single_order(WORD* obs, INT sequence_length, INT start, INT p_order, INT max_val, INT gap)
 {
-	ASSERT(gap>=0) ;
+	ASSERT(gap>=0);
 	
 	const INT start_gap = (p_order - gap)/2 ;
 	const INT end_gap = start_gap + gap ;
