@@ -1,12 +1,12 @@
 /*
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
- * Written (W) 1999-2007 Soeren Sonnenburg
- * Written (W) 1999-2007 Gunnar Raetsch
- * Copyright (C) 1999-2007 Fraunhofer Institute FIRST and Max-Planck-Society
+ * Written (W) 1999-2008 Soeren Sonnenburg
+ * Written (W) 1999-2008 Gunnar Raetsch
+ * Copyright (C) 1999-2008 Fraunhofer Institute FIRST and Max-Planck-Society
  */
 
 #ifndef __GUILABELS__H_
@@ -19,12 +19,12 @@
 #include "base/SGObject.h"
 #include "features/Labels.h"
 
-class CGUI;
+class CSGInterface;
 
 class CGUILabels : public CSGObject
 {
 	public:
-		CGUILabels(CGUI *);
+		CGUILabels(CSGInterface* interface);
 		~CGUILabels();
 
 		CLabels *get_train_labels() { return train_labels; }
@@ -33,11 +33,12 @@ class CGUILabels : public CSGObject
 		bool set_train_labels(CLabels* lab) { delete train_labels; train_labels=lab; return true;}
 		bool set_test_labels(CLabels* lab) { delete test_labels; test_labels=lab; return true;}
 
-		bool load(CHAR* param);
+		/** load labels from file */
+		bool load(CHAR* filename, CHAR* target);
 		bool save(CHAR* param);
 
 	protected:
-		CGUI* gui;
+		CSGInterface* ui;
 		CLabels *train_labels;
 		CLabels *test_labels;
 };
