@@ -1,30 +1,32 @@
 /*
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
- * Written (W) 1999-2007 Soeren Sonnenburg
- * Copyright (C) 1999-2007 Fraunhofer Institute FIRST and Max-Planck-Society
+ * Written (W) 1999-2008 Soeren Sonnenburg
+ * Copyright (C) 1999-2008 Fraunhofer Institute FIRST and Max-Planck-Society
  */
 
 #include "lib/config.h"
 
 #ifndef HAVE_SWIG
-#include "lib/Mathematics.h"
-#include "guilib/GUIMath.h"
 #include "lib/io.h"
+#include "lib/Mathematics.h"
 
-CGUIMath::CGUIMath(CGUI* g) : CSGObject(), gui(g), threshold(0.0)
+#include "guilib/GUIMath.h"
+#include "interface/SGInterface.h"
+
+CGUIMath::CGUIMath(CSGInterface* ui_)
+: CSGObject(), ui(ui_), threshold(0.0)
 {
 }
 
-void CGUIMath::set_threshold(CHAR* param)
+void CGUIMath::set_threshold(DREAL value)
 {
-	param=CIO::skip_spaces(param);
-	SG_INFO( "old threshold: %f", threshold);
-	sscanf(param,"%le", &threshold);
-	SG_INFO( " new threshold: %f\n", threshold);
+	SG_INFO("Old threshold: %f.\n", threshold);
+	threshold=value;
+	SG_INFO("New threshold: %f.\n", threshold);
 }
 
 void CGUIMath::evaluate_results(DREAL* output, INT* label, INT total, FILE* outputfile, FILE* rocfile)
