@@ -1,22 +1,23 @@
 /*
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
- * Written (W) 1999-2007 Soeren Sonnenburg
- * Copyright (C) 1999-2007 Fraunhofer Institute FIRST and Max-Planck-Society
+ * Written (W) 1999-2008 Soeren Sonnenburg
+ * Copyright (C) 1999-2008 Fraunhofer Institute FIRST and Max-Planck-Society
  */
 
 #include "lib/config.h"
 
 #ifndef HAVE_SWIG
 #include "guilib/GUITime.h"
+#include "interface/SGInterface.h"
 
-CGUITime::CGUITime(CGUI* g) : CSGObject(), gui(g)
+CGUITime::CGUITime(CSGInterface* ui_)
+: CSGObject(), ui(ui_)
 {
 	time=new CTime();
-	ASSERT(time);
 }
 
 CGUITime::~CGUITime()
