@@ -1,11 +1,11 @@
 /*
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
- * Written (W) 1999-2007 Soeren Sonnenburg
- * Copyright (C) 1999-2007 Fraunhofer Institute FIRST and Max-Planck-Society
+ * Written (W) 1999-2008 Soeren Sonnenburg
+ * Copyright (C) 1999-2008 Fraunhofer Institute FIRST and Max-Planck-Society
  */
 
 #ifndef __GUITIME__H_
@@ -18,19 +18,19 @@
 #include "base/SGObject.h"
 #include "lib/Time.h"
 
-class CGUI;
+class CSGInterface;
 
 class CGUITime : public CSGObject
 {
 	public:
-		CGUITime(CGUI *);
+		CGUITime(CSGInterface* interface);
 		~CGUITime();
 
 		void start();
 		void stop();
 
 	protected:
-		CGUI* gui;
+		CSGInterface* ui;
 		CTime* time;
 };
 #endif //HAVE_SWIG
