@@ -1,11 +1,11 @@
 /*
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
- * Written (W) 1999-2007 Gunnar Raetsch
- * Copyright (C) 1999-2007 Fraunhofer Institute FIRST and Max-Planck-Society
+ * Written (W) 1999-2008 Gunnar Raetsch
+ * Copyright (C) 1999-2008 Fraunhofer Institute FIRST and Max-Planck-Society
  */
 
 #include "lib/common.h"
@@ -15,12 +15,12 @@
 #include "lib/io.h"
 
 CAUCKernel::CAUCKernel(INT size, CKernel* s)
-	: CSimpleKernel<WORD>(size),subkernel(s)
+: CSimpleKernel<WORD>(size), subkernel(s)
 {
 }
 
 CAUCKernel::CAUCKernel(CWordFeatures* l, CWordFeatures* r, CKernel* s)
-	: CSimpleKernel<WORD>(10),subkernel(s)
+: CSimpleKernel<WORD>(10), subkernel(s)
 {
 	init(l, r);
 }
@@ -36,11 +36,6 @@ bool CAUCKernel::init(CFeatures* l, CFeatures* r)
 	return true;
 }
 
-
-void CAUCKernel::cleanup()
-{
-}
-
 bool CAUCKernel::load_init(FILE* src)
 {
 	return false;
@@ -62,9 +57,8 @@ DREAL CAUCKernel::compute(INT idx_a, INT idx_b)
   ASSERT(alen==2);
   ASSERT(blen==2);
 
-  ASSERT(subkernel!=NULL);
-  ASSERT(subkernel->get_lhs()!=NULL);
-  ASSERT(subkernel->get_rhs()!=NULL);
+  ASSERT(subkernel && subkernel->has_features());
+
   DREAL k11,k12,k21,k22 ;
   INT idx_a1=avec[0], idx_a2=avec[1], idx_b1=bvec[0], idx_b2=bvec[1] ;
   
