@@ -1,11 +1,11 @@
 /*
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
- * Written (W) 1999-2007 Soeren Sonnenburg
- * Copyright (C) 1999-2007 Fraunhofer Institute FIRST and Max-Planck-Society
+ * Written (W) 1999-2008 Soeren Sonnenburg
+ * Copyright (C) 1999-2008 Fraunhofer Institute FIRST and Max-Planck-Society
  */
 
 #include "lib/common.h"
@@ -15,12 +15,12 @@
 #include "lib/io.h"
 
 CChi2Kernel::CChi2Kernel(INT size, DREAL w)
-	: CSimpleKernel<DREAL>(size), width(w)
+: CSimpleKernel<DREAL>(size), width(w)
 {
 }
 
 CChi2Kernel::CChi2Kernel(CRealFeatures* l, CRealFeatures* r, DREAL w, INT size)
-	: CSimpleKernel<DREAL>(size), width(w)
+: CSimpleKernel<DREAL>(size), width(w)
 {
 	init(l,r);
 }
@@ -36,10 +36,6 @@ bool CChi2Kernel::init(CFeatures* l, CFeatures* r)
 	return result;
 }
 
-void CChi2Kernel::cleanup()
-{
-}
-
 bool CChi2Kernel::load_init(FILE* src)
 {
 	return false;
@@ -57,7 +53,6 @@ DREAL CChi2Kernel::compute(INT idx_a, INT idx_b)
 
 	double* avec=((CRealFeatures*) lhs)->get_feature_vector(idx_a, alen, afree);
 	double* bvec=((CRealFeatures*) rhs)->get_feature_vector(idx_b, blen, bfree);
-
 	ASSERT(alen==blen);
 
 	DREAL result=0;
@@ -65,7 +60,8 @@ DREAL CChi2Kernel::compute(INT idx_a, INT idx_b)
 	{
 		DREAL n=avec[i]-bvec[i];
 		DREAL d=avec[i]+bvec[i];
-		result+=n*n/d;
+		if (d!=0)
+			result+=n*n/d;
 	}
 
 	result=exp(-result/width);
