@@ -1,12 +1,12 @@
 /*
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
- * Written (W) 1999-2007 Soeren Sonnenburg
- * Written (W) 1999-2007 Gunnar Raetsch
- * Copyright (C) 1999-2007 Fraunhofer Institute FIRST and Max-Planck-Society
+ * Written (W) 1999-2008 Soeren Sonnenburg
+ * Written (W) 1999-2008 Gunnar Raetsch
+ * Copyright (C) 1999-2008 Fraunhofer Institute FIRST and Max-Planck-Society
  */
 
 #include "lib/common.h"
@@ -38,8 +38,8 @@ struct S_THREAD_PARAM
 };
 
 CCombinedKernel::CCombinedKernel(INT size, bool asw)
-	: CKernel(size), sv_count(0), sv_idx(NULL), sv_weight(NULL),
-	  subkernel_weights_buffer(NULL), append_subkernel_weights(asw)
+: CKernel(size), sv_count(0), sv_idx(NULL), sv_weight(NULL),
+	subkernel_weights_buffer(NULL), append_subkernel_weights(asw)
 {
 	properties |= KP_LINADD | KP_KERNCOMBINATION | KP_BATCHEVALUATION;
 	kernel_list=new CList<CKernel*>(true);
@@ -49,8 +49,8 @@ CCombinedKernel::CCombinedKernel(INT size, bool asw)
 }
 
 CCombinedKernel::CCombinedKernel(CCombinedFeatures *l, CCombinedFeatures *r, bool asw)
-	: CKernel(10), sv_count(0), sv_idx(NULL), sv_weight(NULL),
-		subkernel_weights_buffer(NULL), append_subkernel_weights(asw)
+: CKernel(10), sv_count(0), sv_idx(NULL), sv_weight(NULL),
+	subkernel_weights_buffer(NULL), append_subkernel_weights(asw)
 {
 	properties |= KP_LINADD | KP_KERNCOMBINATION | KP_BATCHEVALUATION;
 	kernel_list=new CList<CKernel*>(true);
@@ -62,24 +62,24 @@ CCombinedKernel::CCombinedKernel(CCombinedFeatures *l, CCombinedFeatures *r, boo
 	init(l, r);
 }
 
-CCombinedKernel::~CCombinedKernel() 
+CCombinedKernel::~CCombinedKernel()
 {
-	delete[] subkernel_weights_buffer ;
-	subkernel_weights_buffer=NULL ;
+	delete[] subkernel_weights_buffer;
+	subkernel_weights_buffer=NULL;
 	
 	cleanup();
 	delete kernel_list;
 
-	SG_INFO("Combined kernel deleted (%p)\n", this);
+	SG_INFO("Combined kernel deleted (%p).\n", this);
 }
 
 bool CCombinedKernel::init(CFeatures* l, CFeatures* r)
 {
 	CKernel::init(l,r);
-	ASSERT(l->get_feature_class() == C_COMBINED);
-	ASSERT(r->get_feature_class() == C_COMBINED);
-	ASSERT(l->get_feature_type() == F_UNKNOWN);
-	ASSERT(r->get_feature_type() == F_UNKNOWN);
+	ASSERT(l->get_feature_class()==C_COMBINED);
+	ASSERT(r->get_feature_class()==C_COMBINED);
+	ASSERT(l->get_feature_type()==F_UNKNOWN);
+	ASSERT(r->get_feature_type()==F_UNKNOWN);
 
 	CFeatures* lf=NULL;
 	CFeatures* rf=NULL;
@@ -101,6 +101,7 @@ bool CCombinedKernel::init(CFeatures* l, CFeatures* r)
 		{
 			while ( result && lf && rf && k )
 			{
+				SG_DEBUG( "Initializing 0x%p - \"%s\"\n", this, k->get_name());
 				result=k->init(lf,rf);
 
 				lf=((CCombinedFeatures*) l)->get_next_feature_obj(lfc) ;
@@ -112,9 +113,9 @@ bool CCombinedKernel::init(CFeatures* l, CFeatures* r)
 		{
 			while ( result && lf && k )
 			{
+				SG_DEBUG( "Initializing 0x%p - \"%s\"\n", this, k->get_name());
 				result=k->init(lf,rf);
 
-
 				lf=((CCombinedFeatures*) l)->get_next_feature_obj(lfc) ;
 				k=get_next_kernel(current) ;
 				rf=lf ;
@@ -191,6 +192,8 @@ void CCombinedKernel::cleanup()
 	}
 
 	delete_optimization();
+
+	CKernel::cleanup();
 }
 
 void CCombinedKernel::list_kernels()
@@ -302,8 +305,8 @@ bool CCombinedKernel::delete_optimization()
 
 void CCombinedKernel::compute_batch(INT num_vec, INT* vec_idx, DREAL* result, INT num_suppvec, INT* IDX, DREAL* weights, DREAL factor)
 {
-	ASSERT(get_rhs());
-	ASSERT(num_vec<=get_rhs()->get_num_vectors())
+	ASSERT(rhs);
+	ASSERT(num_vec<=rhs->get_num_vectors())
 	ASSERT(num_vec>0);
 	ASSERT(vec_idx);
 	ASSERT(result);
@@ -429,8 +432,8 @@ void CCombinedKernel::emulate_compute_batch(CKernel* k, INT num_vec, INT* vec_id
 	}
 	else
 	{
-		ASSERT(IDX!=NULL || num_suppvec==0) ;
-		ASSERT(weights!=NULL || num_suppvec==0) ;
+		ASSERT(IDX!=NULL || num_suppvec==0);
+		ASSERT(weights!=NULL || num_suppvec==0);
 
 		if (k->get_combined_kernel_weight()!=0)
 		{ // compute the usual way for any non-optimized kernel
@@ -514,8 +517,8 @@ DREAL CCombinedKernel::compute_optimized(INT idx)
 		}
 		else
 		{
-			ASSERT(sv_idx!=NULL || sv_count==0) ;
-			ASSERT(sv_weight!=NULL || sv_count==0) ;
+			ASSERT(sv_idx!=NULL || sv_count==0);
+			ASSERT(sv_weight!=NULL || sv_count==0);
 
 			if (k->get_combined_kernel_weight()!=0)
 			{ // compute the usual way for any non-optimized kernel
@@ -610,7 +613,7 @@ const DREAL* CCombinedKernel::get_subkernel_weights(INT& num_weights)
 		{
 			INT num = -1 ;
 			const DREAL *w = k->get_subkernel_weights(num);
-			ASSERT(num==k->get_num_subkernels()) ;
+			ASSERT(num==k->get_num_subkernels());
 			for (INT j=0; j<num; j++)
 				subkernel_weights_buffer[i+j]=w[j] ;
 			k = get_next_kernel(current);
