@@ -1,11 +1,11 @@
 /*
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
- * Written (W) 1999-2007 Soeren Sonnenburg
- * Copyright (C) 1999-2007 Fraunhofer Institute FIRST and Max-Planck-Society
+ * Written (W) 1999-2008 Soeren Sonnenburg
+ * Copyright (C) 1999-2008 Fraunhofer Institute FIRST and Max-Planck-Society
  */
 
 #include "lib/common.h"
@@ -15,7 +15,7 @@
 
 CCommUlongStringKernel::CCommUlongStringKernel(
 	INT size, bool us, ENormalizationType n)
-	: CStringKernel<ULONG>(size), sqrtdiag_lhs(NULL), sqrtdiag_rhs(NULL),
+: CStringKernel<ULONG>(size), sqrtdiag_lhs(NULL), sqrtdiag_rhs(NULL),
 	initialized(false), use_sign(us), normalization(n)
 {
 	properties |= KP_LINADD;
@@ -25,7 +25,7 @@ CCommUlongStringKernel::CCommUlongStringKernel(
 CCommUlongStringKernel::CCommUlongStringKernel(
 	CStringFeatures<ULONG>* l, CStringFeatures<ULONG>* r, bool us,
 	ENormalizationType n, INT size)
-	: CStringKernel<ULONG>(size), sqrtdiag_lhs(NULL), sqrtdiag_rhs(NULL),
+: CStringKernel<ULONG>(size), sqrtdiag_lhs(NULL), sqrtdiag_rhs(NULL),
 	initialized(false), use_sign(us), normalization(n)
 {
 	properties |= KP_LINADD;
@@ -33,13 +33,13 @@ CCommUlongStringKernel::CCommUlongStringKernel(
 	init(l,r);
 }
 
-CCommUlongStringKernel::~CCommUlongStringKernel() 
+CCommUlongStringKernel::~CCommUlongStringKernel()
 {
 	cleanup();
 }
 
-void CCommUlongStringKernel::remove_lhs() 
-{ 
+void CCommUlongStringKernel::remove_lhs()
+{
 	delete_optimization();
 
 #ifdef SVMLIGHT
@@ -77,13 +77,11 @@ bool CCommUlongStringKernel::init(CFeatures* l, CFeatures* r)
 	INT i;
 
 	initialized=false;
-	if (sqrtdiag_lhs != sqrtdiag_rhs)
-	  delete[] sqrtdiag_rhs;
+	if (sqrtdiag_lhs!=sqrtdiag_rhs)
+		delete[] sqrtdiag_rhs;
 	sqrtdiag_rhs=NULL;
 	delete[] sqrtdiag_lhs;
-	sqrtdiag_lhs=NULL;
-
-	sqrtdiag_lhs= new DREAL[lhs->get_num_vectors()];
+	sqrtdiag_lhs=new DREAL[lhs->get_num_vectors()];
 
 	for (i=0; i<lhs->get_num_vectors(); i++)
 		sqrtdiag_lhs[i]=1;
@@ -92,14 +90,11 @@ bool CCommUlongStringKernel::init(CFeatures* l, CFeatures* r)
 		sqrtdiag_rhs=sqrtdiag_lhs;
 	else
 	{
-		sqrtdiag_rhs= new DREAL[rhs->get_num_vectors()];
+		sqrtdiag_rhs=new DREAL[rhs->get_num_vectors()];
 		for (i=0; i<rhs->get_num_vectors(); i++)
 			sqrtdiag_rhs[i]=1;
 	}
 
-	ASSERT(sqrtdiag_lhs);
-	ASSERT(sqrtdiag_rhs);
-
 	this->lhs=(CStringFeatures<ULONG>*) l;
 	this->rhs=(CStringFeatures<ULONG>*) l;
 
@@ -152,6 +147,8 @@ void CCommUlongStringKernel::cleanup()
 
 	delete[] sqrtdiag_lhs;
 	sqrtdiag_lhs=NULL;
+
+	CKernel::cleanup();
 }
 
 bool CCommUlongStringKernel::load_init(FILE* src)
