@@ -1,11 +1,11 @@
 /*
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
- * Written (W) 1999-2007 Soeren Sonnenburg
- * Copyright (C) 1999-2007 Fraunhofer Institute FIRST and Max-Planck-Society
+ * Written (W) 1999-2008 Soeren Sonnenburg
+ * Copyright (C) 1999-2008 Fraunhofer Institute FIRST and Max-Planck-Society
  */
 
 #include "lib/common.h"
@@ -15,9 +15,9 @@
 
 CCommWordStringKernel::CCommWordStringKernel(
 	INT size, bool s, ENormalizationType n)
-	: CStringKernel<WORD>(size), sqrtdiag_lhs(NULL), sqrtdiag_rhs(NULL),
+: CStringKernel<WORD>(size), sqrtdiag_lhs(NULL), sqrtdiag_rhs(NULL),
 	initialized(false), dictionary_size(0), dictionary_weights(NULL),
-	use_sign(s), normalization(n)
+	use_sign(s), normalization(n), use_dict_diagonal_optimization(false), dict_diagonal_optimization(NULL)
 {
 	properties |= KP_LINADD;
 	init_dictionary(1<<(sizeof(WORD)*8));
@@ -26,9 +26,9 @@ CCommWordStringKernel::CCommWordStringKernel(
 CCommWordStringKernel::CCommWordStringKernel(
 	CStringFeatures<WORD>* l, CStringFeatures<WORD>* r,
 	bool s, ENormalizationType n, INT size)
-	: CStringKernel<WORD>(size), sqrtdiag_lhs(NULL), sqrtdiag_rhs(NULL),
+: CStringKernel<WORD>(size), sqrtdiag_lhs(NULL), sqrtdiag_rhs(NULL),
 	initialized(false), dictionary_size(0), dictionary_weights(NULL),
-	use_sign(s), normalization(n)
+	use_sign(s), normalization(n), use_dict_diagonal_optimization(false), dict_diagonal_optimization(NULL)
 {
 	properties |= KP_LINADD;
 
@@ -41,8 +41,7 @@ bool CCommWordStringKernel::init_dictionary(INT size)
 {
 	dictionary_size= size;
 	delete[] dictionary_weights;
-	dictionary_weights = new DREAL[size];
-	ASSERT(dictionary_weights);
+	dictionary_weights=new DREAL[size];
 	SG_DEBUG( "using dictionary of %d words\n", size);
 	clear_normal();
 
@@ -54,6 +53,7 @@ CCommWordStringKernel::~CCommWordStringKernel()
 	cleanup();
 
 	delete[] dictionary_weights;
+	delete[] dict_diagonal_optimization ;
 }
   
 void CCommWordStringKernel::remove_lhs() 
@@ -97,13 +97,11 @@ bool CCommWordStringKernel::init(CFeatures* l, CFeatures* r)
 	initialized = false;
 	INT i;
 
-	if (sqrtdiag_lhs != sqrtdiag_rhs)
-	  delete[] sqrtdiag_rhs;
+	if (sqrtdiag_lhs!=sqrtdiag_rhs)
+		delete[] sqrtdiag_rhs;
 	sqrtdiag_rhs=NULL;
 	delete[] sqrtdiag_lhs;
-	sqrtdiag_lhs=NULL;
-
-	sqrtdiag_lhs= new DREAL[lhs->get_num_vectors()];
+	sqrtdiag_lhs=new DREAL[lhs->get_num_vectors()];
 
 	for (i=0; i<lhs->get_num_vectors(); i++)
 		sqrtdiag_lhs[i]=1;
@@ -112,22 +110,29 @@ bool CCommWordStringKernel::init(CFeatures* l, CFeatures* r)
 		sqrtdiag_rhs=sqrtdiag_lhs;
 	else
 	{
-		sqrtdiag_rhs= new DREAL[rhs->get_num_vectors()];
+		sqrtdiag_rhs=new DREAL[rhs->get_num_vectors()];
 		for (i=0; i<rhs->get_num_vectors(); i++)
 			sqrtdiag_rhs[i]=1;
 	}
 
-	ASSERT(sqrtdiag_lhs);
-	ASSERT(sqrtdiag_rhs);
-
 	this->lhs=(CStringFeatures<WORD>*) l;
 	this->rhs=(CStringFeatures<WORD>*) l;
 
+	if (use_dict_diagonal_optimization)
+	{
+		delete[] dict_diagonal_optimization ;
+		dict_diagonal_optimization=new INT[INT(((CStringFeatures<WORD>*)l)->get_num_symbols())];
+		ASSERT(((CStringFeatures<WORD>*)l)->get_num_symbols() == ((CStringFeatures<WORD>*)r)->get_num_symbols()) ;
+	}
+
 	//compute normalize to 1 values
 	for (i=0; i<lhs->get_num_vectors(); i++)
 	{
-		sqrtdiag_lhs[i]=sqrt(compute_helper(i,i, true));
-
+		if (use_dict_diagonal_optimization)
+			sqrtdiag_lhs[i]=sqrt(compute_diag(i));
+		else
+			sqrtdiag_lhs[i]=sqrt(compute_helper(i,i, true));
+		
 		//trap divide by zero exception
 		if (sqrtdiag_lhs[i]==0)
 			sqrtdiag_lhs[i]=1e-16;
@@ -139,12 +144,15 @@ bool CCommWordStringKernel::init(CFeatures* l, CFeatures* r)
 	{
 		this->lhs=(CStringFeatures<WORD>*) r;
 		this->rhs=(CStringFeatures<WORD>*) r;
-
+		
 		//compute normalize to 1 values
 		for (i=0; i<rhs->get_num_vectors(); i++)
 		{
-			sqrtdiag_rhs[i]=sqrt(compute_helper(i,i, true));
-
+			if (use_dict_diagonal_optimization)
+				sqrtdiag_rhs[i]=sqrt(compute_diag(i));
+			else
+				sqrtdiag_rhs[i]=sqrt(compute_helper(i,i, true));
+				
 			//trap divide by zero exception
 			if (sqrtdiag_rhs[i]==0)
 				sqrtdiag_rhs[i]=1e-16;
@@ -172,6 +180,8 @@ void CCommWordStringKernel::cleanup()
 
 	delete[] sqrtdiag_lhs;
 	sqrtdiag_lhs=NULL;
+
+	CKernel::cleanup();
 }
 
 bool CCommWordStringKernel::load_init(FILE* src)
@@ -184,6 +194,48 @@ bool CCommWordStringKernel::save_init(FILE* dest)
 	return false;
 }
 
+DREAL CCommWordStringKernel::compute_diag(INT idx_a)
+{
+	INT alen;
+	CStringFeatures<WORD>* l = (CStringFeatures<WORD>*) lhs;
+	CStringFeatures<WORD>* r = (CStringFeatures<WORD>*) rhs;
+
+	WORD* av=l->get_feature_vector(idx_a, alen);
+
+	DREAL result=0.0 ;
+	ASSERT(l==r);
+	ASSERT(sizeof(WORD)<=sizeof(DREAL));
+	ASSERT((1<<(sizeof(WORD)*8)) > alen);
+
+	INT num_symbols=(INT) l->get_num_symbols();
+	ASSERT(num_symbols<=dictionary_size);
+
+	INT* dic = dict_diagonal_optimization;
+	memset(dic, 0, num_symbols*sizeof(INT));
+
+	for (INT i=0; i<alen; i++)
+		dic[av[i]]++;
+
+	if (use_sign)
+	{
+		for (INT i=0; i<(INT) l->get_num_symbols(); i++)
+		{
+			if (dic[i]!=0)
+				result++;
+		}
+	}
+	else
+	{
+		for (INT i=0; i<num_symbols; i++)
+		{
+			if (dic[i]!=0)
+				result+=dic[i]*dic[i];
+		}
+	}
+
+	return result;
+}
+
 DREAL CCommWordStringKernel::compute_helper(INT idx_a, INT idx_b, bool do_sort)
 {
 	INT alen, blen;
@@ -200,8 +252,7 @@ DREAL CCommWordStringKernel::compute_helper(INT idx_a, INT idx_b, bool do_sort)
 	{
 		if (alen>0)
 		{
-			avec= new WORD[alen];
-			ASSERT(avec);
+			avec=new WORD[alen];
 			memcpy(avec, av, sizeof(WORD)*alen);
 			CMath::radix_sort(avec, alen);
 		}
@@ -210,8 +261,7 @@ DREAL CCommWordStringKernel::compute_helper(INT idx_a, INT idx_b, bool do_sort)
 
 		if (blen>0)
 		{
-			bvec= new WORD[blen];
-			ASSERT(bvec);
+			bvec=new WORD[blen];
 			memcpy(bvec, bv, sizeof(WORD)*blen);
 			CMath::radix_sort(bvec, blen);
 		}
@@ -377,7 +427,7 @@ bool CCommWordStringKernel::init_optimization(INT count, INT *IDX, DREAL * weigh
 		add_to_normal(IDX[i], weights[i]);
 	}
 
-	SG_PRINT( "Done.         \n");
+	//SG_PRINT( "Done.         \n");
 	
 	set_is_initialized(true);
 	return true;
@@ -463,9 +513,6 @@ DREAL* CCommWordStringKernel::compute_scoring(INT max_degree, INT& num_feat,
 
 	if (!target)
 		target=new DREAL[num_feat*num_sym];
-
-	ASSERT(target);
-
 	memset(target, 0, num_feat*num_sym*sizeof(DREAL));
 
 	if (do_init)
@@ -556,6 +603,8 @@ DREAL* CCommWordStringKernel::compute_scoring(INT max_degree, INT& num_feat,
 	for (INT i=1; i<num_feat; i++)
 		memcpy(&target[num_sym*i], target, num_sym*sizeof(DREAL));
 
+	SG_UNREF(alpha);
+
 	return target;
 }
 
@@ -583,11 +632,8 @@ CHAR* CCommWordStringKernel::compute_consensus(INT &result_len, INT num_suppvec,
 	init_optimization(num_suppvec, IDX, alphas);
 
 	CHAR* result=new CHAR[result_len];
-	ASSERT(result);
 	INT* bt=new INT[total_len];
-	ASSERT(bt);
 	DREAL* score=new DREAL[total_len];
-	ASSERT(score);
 
 	for (LONG i=0; i<total_len; i++)
 	{
@@ -664,5 +710,6 @@ CHAR* CCommWordStringKernel::compute_consensus(INT &result_len, INT num_suppvec,
 
 	delete[] bt;
 	delete[] score;
+	SG_UNREF(alpha);
 	return result;
 }
