@@ -1,11 +1,11 @@
 /*
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
- * Written (W) 1999-2007 Soeren Sonnenburg
- * Copyright (C) 1999-2007 Fraunhofer Institute FIRST and Max-Planck-Society
+ * Written (W) 1999-2008 Soeren Sonnenburg
+ * Copyright (C) 1999-2008 Fraunhofer Institute FIRST and Max-Planck-Society
  */
 
 #include "lib/common.h"
@@ -15,13 +15,13 @@
 #include "lib/io.h"
 
 CGaussianKernel::CGaussianKernel(INT size, DREAL w)
-	: CSimpleKernel<DREAL>(size), width(w)
+: CSimpleKernel<DREAL>(size), width(w)
 {
 }
 
 CGaussianKernel::CGaussianKernel(
 	CRealFeatures* l, CRealFeatures* r, DREAL w, INT size)
-	: CSimpleKernel<DREAL>(size), width(w)
+: CSimpleKernel<DREAL>(size), width(w)
 {
 	init(l,r);
 }
@@ -29,17 +29,13 @@ CGaussianKernel::CGaussianKernel(
 CGaussianKernel::~CGaussianKernel()
 {
 }
-  
+
 bool CGaussianKernel::init(CFeatures* l, CFeatures* r)
 {
 	CSimpleKernel<DREAL>::init(l, r);
 	return true;
 }
 
-void CGaussianKernel::cleanup()
-{
-}
-
 bool CGaussianKernel::load_init(FILE* src)
 {
 	return false;
@@ -49,27 +45,24 @@ bool CGaussianKernel::save_init(FILE* dest)
 {
 	return false;
 }
-  
+
 DREAL CGaussianKernel::compute(INT idx_a, INT idx_b)
 {
-  INT alen, blen;
-  bool afree, bfree;
+	INT alen, blen;
+	bool afree, bfree;
 
-  DREAL* avec=((CRealFeatures*) lhs)->get_feature_vector(idx_a, alen, afree);
-  DREAL* bvec=((CRealFeatures*) rhs)->get_feature_vector(idx_b, blen, bfree);
-  
-  ASSERT(alen==blen);
+	DREAL* avec=((CRealFeatures*) lhs)->get_feature_vector(idx_a, alen, afree);
+	DREAL* bvec=((CRealFeatures*) rhs)->get_feature_vector(idx_b, blen, bfree);
+	ASSERT(alen==blen);
 
-  INT ialen=(int) alen;
+	DREAL result=0;
+	for (INT i=0; i<alen; i++)
+		result+=CMath::sq(avec[i]-bvec[i]);
 
-  DREAL result=0;
-  for (INT i=0; i<ialen; i++)
-	  result+=(avec[i]-bvec[i])*(avec[i]-bvec[i]);
+	result=exp(-result/width);
 
-  result=exp(-result/width);
+	((CRealFeatures*) lhs)->free_feature_vector(avec, idx_a, afree);
+	((CRealFeatures*) rhs)->free_feature_vector(bvec, idx_b, bfree);
 
-  ((CRealFeatures*) lhs)->free_feature_vector(avec, idx_a, afree);
-  ((CRealFeatures*) rhs)->free_feature_vector(bvec, idx_b, bfree);
-
-  return result;
+	return result;
 }
