@@ -1,12 +1,12 @@
 /*
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
- * Written (W) 1999-2007 Soeren Sonnenburg
- * Written (W) 1999-2007 Gunnar Raetsch
- * Copyright (C) 1999-2007 Fraunhofer Institute FIRST and Max-Planck-Society
+ * Written (W) 1999-2008 Soeren Sonnenburg
+ * Written (W) 1999-2008 Gunnar Raetsch
+ * Copyright (C) 1999-2008 Fraunhofer Institute FIRST and Max-Planck-Society
  */
 
 #include "lib/config.h"
@@ -20,6 +20,8 @@
 #include "kernel/Kernel.h"
 #include "features/Features.h"
 
+#include "classifier/svm/SVM.h"
+
 #include <string.h>
 #include <unistd.h>
 #include <math.h>
@@ -29,7 +31,7 @@
 #endif
 
 CKernel::CKernel(INT size)
-	: CSGObject(), kernel_matrix(NULL), precomputed_matrix(NULL),
+: CSGObject(), kernel_matrix(NULL), precomputed_matrix(NULL),
 	precompute_subkernel_matrix(false), precompute_matrix(false), lhs(NULL),
 	rhs(NULL), combined_kernel_weight(1), optimization_initialized(false),
 	opt_type(FASTBUTMEMHUNGRY), properties(KP_NONE)
@@ -39,14 +41,14 @@ CKernel::CKernel(INT size)
 
 	cache_size=size;
 
-	if (get_is_initialized()) 
-      SG_ERROR( "COptimizableKernel still initialized on destruction");
+	if (get_is_initialized())
+		SG_ERROR( "COptimizableKernel still initialized on destruction");
 }
 
-		
+
 CKernel::CKernel(CFeatures* p_lhs, CFeatures* p_rhs, INT size)
-  : CSGObject(), kernel_matrix(NULL), precomputed_matrix(NULL),
-	precompute_subkernel_matrix(false), precompute_matrix(false), 
+: CSGObject(), kernel_matrix(NULL), precomputed_matrix(NULL),
+	precompute_subkernel_matrix(false), precompute_matrix(false),
 	lhs(NULL), rhs(NULL), combined_kernel_weight(1), optimization_initialized(false),
 	opt_type(FASTBUTMEMHUNGRY), properties(KP_NONE)
 {
@@ -55,27 +57,23 @@ CKernel::CKernel(CFeatures* p_lhs, CFeatures* p_rhs, INT size)
 
 	cache_size=size;
 
-	if (get_is_initialized()) 
-      SG_ERROR( "COptimizableKernel still initialized on destruction");
+	if (get_is_initialized())
+		SG_ERROR("Kernel initialized on construction.\n");
 
 	init(p_lhs, p_rhs);
 }
 
 CKernel::~CKernel()
 {
-	if (get_is_initialized()) 
-      SG_ERROR( "COptimizableKernel still initialized on destruction");
+	if (get_is_initialized())
+		SG_ERROR("Kernel still initialized on destruction.\n");
 
-    SG_UNREF(lhs);
-    if (lhs!=rhs)
-        SG_UNREF(rhs);
+	remove_lhs_and_rhs();
 
+	delete[] precomputed_matrix;
+	precomputed_matrix=NULL;
 
-
-	delete[] precomputed_matrix ;
-	precomputed_matrix=NULL ;
-
-	SG_INFO("Kernel deleted (%p)\n", this);
+	SG_INFO("Kernel deleted (%p).\n", this);
 }
 
 void CKernel::get_kernel_matrix(DREAL** dst, INT* m, INT* n)
@@ -83,8 +81,8 @@ void CKernel::get_kernel_matrix(DREAL** dst, INT* m, INT* n)
 	ASSERT(dst && m && n);
 
 	DREAL* result = NULL;
-	CFeatures* f1 = get_lhs();
-	CFeatures* f2 = get_rhs();
+	CFeatures* f1 = lhs;
+	CFeatures* f2 = rhs;
 
 	if (f1 && f2)
 	{
@@ -95,11 +93,9 @@ void CKernel::get_kernel_matrix(DREAL** dst, INT* m, INT* n)
 
 		LONG total_num = num_vec1 * num_vec2;
 		INT num_done = 0;
-		SG_PRINT( "returning kernel matrix of size %dx%d\n", num_vec1, num_vec2);
-
 		SG_DEBUG( "returning kernel matrix of size %dx%d\n", num_vec1, num_vec2);
 
-		result=new DREAL[total_num];
+		result=(DREAL*) malloc(sizeof(DREAL)*total_num);
 		ASSERT(result);
 
 		if ( (f1 == f2) && (num_vec1 == num_vec2) )
@@ -139,7 +135,7 @@ void CKernel::get_kernel_matrix(DREAL** dst, INT* m, INT* n)
 			}
 		}
 
-		SG_PRINT( "done.           \n");
+		SG_DONE();
 	}
 	else
       SG_ERROR( "no features assigned to kernel\n");
@@ -151,8 +147,8 @@ SHORTREAL* CKernel::get_kernel_matrix_shortreal(int &num_vec1, int &num_vec2,
 		SHORTREAL* target)
 {
 	SHORTREAL* result = NULL;
-	CFeatures* f1 = get_lhs();
-	CFeatures* f2 = get_rhs();
+	CFeatures* f1 = lhs;
+	CFeatures* f2 = rhs;
 
 	if (f1 && f2)
 	{
@@ -172,9 +168,7 @@ SHORTREAL* CKernel::get_kernel_matrix_shortreal(int &num_vec1, int &num_vec2,
 		else
 			result=new SHORTREAL[total_num];
 
-		ASSERT(result);
-
-		if ( (f1 == f2) && (num_vec1 == num_vec2) )
+		if (f1==f2 && num_vec1==num_vec2)
 		{
 			for (int i=0; i<num_vec1; i++)
 			{
@@ -211,7 +205,7 @@ SHORTREAL* CKernel::get_kernel_matrix_shortreal(int &num_vec1, int &num_vec2,
 			}
 		}
 
-		SG_PRINT( "done.           \n");
+		SG_DONE();
 	}
 	else
       SG_ERROR( "no features assigned to kernel\n");
@@ -222,8 +216,8 @@ SHORTREAL* CKernel::get_kernel_matrix_shortreal(int &num_vec1, int &num_vec2,
 DREAL* CKernel::get_kernel_matrix_real(int &num_vec1, int &num_vec2, DREAL* target)
 {
 	DREAL* result = NULL;
-	CFeatures* f1 = get_lhs();
-	CFeatures* f2 = get_rhs();
+	CFeatures* f1 = lhs;
+	CFeatures* f2 = rhs;
 
 	if (f1 && f2)
 	{
@@ -243,9 +237,7 @@ DREAL* CKernel::get_kernel_matrix_real(int &num_vec1, int &num_vec2, DREAL* targ
 		else
 			result=new DREAL[total_num];
 
-		ASSERT(result);
-
-		if ( (f1 == f2) && (num_vec1 == num_vec2) )
+		if (f1==f2 && num_vec1==num_vec2)
 		{
 			for (int i=0; i<num_vec1; i++)
 			{
@@ -282,7 +274,7 @@ DREAL* CKernel::get_kernel_matrix_real(int &num_vec1, int &num_vec2, DREAL* targ
 			}
 		}
 
-		SG_PRINT( "done.           \n");
+		SG_DONE();
 	}
 	else
       SG_ERROR( "no features assigned to kernel\n");
@@ -300,8 +292,11 @@ bool CKernel::init(CFeatures* l, CFeatures* r)
 	ASSERT(r);
 
 	//make sure features are compatible
-	ASSERT(l->get_feature_class() == r->get_feature_class());
-	ASSERT(l->get_feature_type() == r->get_feature_type());
+	ASSERT(l->get_feature_class()==r->get_feature_class());
+	ASSERT(l->get_feature_type()==r->get_feature_type());
+
+	//remove references to previous features
+	remove_lhs_and_rhs();
 
     //increase reference counts
     SG_REF(l);
@@ -311,14 +306,17 @@ bool CKernel::init(CFeatures* l, CFeatures* r)
 	lhs=l;
 	rhs=r;
 
-
-
 	delete[] precomputed_matrix ;
 	precomputed_matrix=NULL ;
 
 	return true;
 }
 
+void CKernel::cleanup()
+{
+	remove_lhs_and_rhs();
+}
+
 
 
 bool CKernel::load(CHAR* fname)
@@ -329,8 +327,8 @@ bool CKernel::load(CHAR* fname)
 bool CKernel::save(CHAR* fname)
 {
 	INT i=0;
-	INT num_left=get_lhs()->get_num_vectors();
-	INT num_right=get_rhs()->get_num_vectors();
+	INT num_left=lhs->get_num_vectors();
+	INT num_right=rhs->get_num_vectors();
 	KERNELCACHE_IDX num_total=num_left*num_right;
 
 	CFile f(fname, 'w', F_DREAL);
@@ -357,18 +355,34 @@ bool CKernel::save(CHAR* fname)
     return (f.is_ok());
 }
 
+void CKernel::remove_lhs_and_rhs()
+{
+	if (rhs!=lhs)
+		SG_UNREF(rhs);
+	rhs = NULL;
+
+	SG_UNREF(lhs);
+	lhs = NULL;
+
+
+}
+
 void CKernel::remove_lhs()
 { 
-
-
+	SG_UNREF(lhs);
 	lhs = NULL;
+
+
 }
 
 /// takes all necessary steps if the rhs is removed from kernel
 void CKernel::remove_rhs()
 {
-
+	if (rhs!=lhs)
+		SG_UNREF(rhs);
 	rhs = NULL;
+
+
 }
 
 
@@ -459,15 +473,6 @@ void CKernel::list_kernel()
 		case K_CONST:
 			SG_INFO( "K_CONST ");
 			break;
-		case K_HAMMINGWORD:
-			SG_INFO( "K_HAMMINGWORD ");
-			break;
-		case K_MANHATTANWORD:
-			SG_INFO( "K_MANHATTANWORD ");
-			break;
-		case K_CANBERRAWORD:
-			SG_INFO( "K_CANBERRAWORD ");
-			break;
 		case K_MINDYGRAM:
 			SG_INFO( "K_MINDYGRAM ");
 			break;
@@ -554,7 +559,7 @@ bool CKernel::delete_optimization()
 	return false;
 }
 
-DREAL CKernel::compute_optimized(INT idx)
+DREAL CKernel::compute_optimized(INT vector_idx)
 {
    SG_ERROR( "kernel does not support linadd optimization\n");
 	return 0;
@@ -565,7 +570,7 @@ void CKernel::compute_batch(INT num_vec, INT* vec_idx, DREAL* target, INT num_su
    SG_ERROR( "kernel does not support batch computation\n");
 }
 
-void CKernel::add_to_normal(INT idx, DREAL weight)
+void CKernel::add_to_normal(INT vector_idx, DREAL weight)
 {
    SG_ERROR( "kernel does not support linadd optimization, add_to_normal not implemented\n");
 }
@@ -580,7 +585,7 @@ INT CKernel::get_num_subkernels()
 	return 1;
 }
 
-void CKernel::compute_by_subkernel(INT idx, DREAL * subkernel_contrib)
+void CKernel::compute_by_subkernel(INT vector_idx, DREAL * subkernel_contrib)
 {
    SG_ERROR( "kernel compute_by_subkernel not implemented\n");
 }
@@ -600,17 +605,16 @@ void CKernel::set_subkernel_weights(DREAL* weights, INT num_weights)
 
 void CKernel::do_precompute_matrix()
 {
-	INT num_left=get_lhs()->get_num_vectors();
-	INT num_right=get_rhs()->get_num_vectors();
+	INT num_left=lhs->get_num_vectors();
+	INT num_right=rhs->get_num_vectors();
 	SG_INFO( "precomputing kernel matrix (%ix%i)\n", num_left, num_right) ;
 
-	ASSERT(num_left == num_right) ;
-	ASSERT(get_lhs()==get_rhs()) ;
-	INT num=num_left ;
+	ASSERT(num_left==num_right);
+	ASSERT(lhs==rhs);
+	INT num=num_left;
 	
-	delete[] precomputed_matrix ;
-	precomputed_matrix=new SHORTREAL[num*(num+1)/2] ;
-	ASSERT(precomputed_matrix!=NULL) ;
+	delete[] precomputed_matrix;
+	precomputed_matrix=new SHORTREAL[num*(num+1)/2];
 
 	for (INT i=0; i<num; i++)
 	{
@@ -636,3 +640,22 @@ void CKernel::init_sqrt_diag(DREAL *v, INT num)
 			v[i] = 1e-16; /* avoid divide by zero exception */
 	}
 }
+
+bool CKernel::init_optimization_svm(CSVM * svm)
+{
+	INT num_suppvec=svm->get_num_support_vectors();
+	INT* sv_idx=new INT[num_suppvec];
+	DREAL* sv_weight=new DREAL[num_suppvec];
+
+	for (INT i=0; i<num_suppvec; i++)
+	{
+		sv_idx[i]    = svm->get_support_vector(i);
+		sv_weight[i] = svm->get_alpha(i);
+	}
+	bool ret = init_optimization(num_suppvec, sv_idx, sv_weight);
+
+	delete[] sv_idx;
+	delete[] sv_weight;
+	return ret;
+}
+
