@@ -1,11 +1,11 @@
 /*
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
- * Written (W) 1999-2007 Soeren Sonnenburg
- * Copyright (C) 1999-2007 Fraunhofer Institute FIRST and Max-Planck-Society
+ * Written (W) 1999-2008 Soeren Sonnenburg
+ * Copyright (C) 1999-2008 Fraunhofer Institute FIRST and Max-Planck-Society
  */
 
 #include "lib/common.h"
@@ -15,24 +15,24 @@
 #include "features/ByteFeatures.h"
 
 CLinearByteKernel::CLinearByteKernel(INT size, bool dr, DREAL s)
-	: CSimpleKernel<BYTE>(size), scale(s), do_rescale(dr), initialized(false),
+: CSimpleKernel<BYTE>(size), scale(s), do_rescale(dr), initialized(false),
 	normal(NULL)
 {
 }
 
 CLinearByteKernel::CLinearByteKernel(
 	CByteFeatures* l, CByteFeatures* r, bool dr, DREAL s)
-	: CSimpleKernel<BYTE>(10), scale(s), do_rescale(dr), initialized(false),
+: CSimpleKernel<BYTE>(10), scale(s), do_rescale(dr), initialized(false),
 	normal(NULL)
 {
 	init(l, r);
 }
 
-CLinearByteKernel::~CLinearByteKernel() 
+CLinearByteKernel::~CLinearByteKernel()
 {
 	cleanup();
 }
-  
+
 bool CLinearByteKernel::init(CFeatures* l, CFeatures* r)
 {
 	CSimpleKernel<BYTE>::init(l, r);
@@ -64,6 +64,8 @@ void CLinearByteKernel::init_rescale()
 void CLinearByteKernel::cleanup()
 {
 	delete_optimization();
+
+	CKernel::cleanup();
 }
 
 bool CLinearByteKernel::load_init(FILE* src)
@@ -126,8 +128,6 @@ bool CLinearByteKernel::init_optimization(INT num_suppvec, INT* sv_idx, DREAL* a
 	ASSERT(num_feat);
 
 	normal=new DREAL[num_feat];
-	ASSERT(normal);
-
 	for (INT i=0; i<num_feat; i++)
 		normal[i]=0;
 
