@@ -1,11 +1,11 @@
 /*
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
- * Written (W) 1999-2007 Soeren Sonnenburg
- * Copyright (C) 1999-2007 Fraunhofer Institute FIRST and Max-Planck-Society
+ * Written (W) 1999-2008 Soeren Sonnenburg
+ * Copyright (C) 1999-2008 Fraunhofer Institute FIRST and Max-Planck-Society
  */
 
 #include "lib/common.h"
@@ -15,15 +15,15 @@
 #include "kernel/LinearKernel.h"
 
 CLinearKernel::CLinearKernel(INT size, DREAL s)
-	: CSimpleKernel<DREAL>(size), scale(s), initialized(false),
-	normal_length(0), normal(NULL)
+: CSimpleKernel<DREAL>(size), scale(s), initialized(false),
+	normal(NULL), normal_length(0)
 {
 	properties |= KP_LINADD;
 }
 
 CLinearKernel::CLinearKernel(CRealFeatures* l, CRealFeatures* r, DREAL s, INT size)
-	: CSimpleKernel<DREAL>(size), scale(s), initialized(false),
-	normal_length(0), normal(NULL)
+: CSimpleKernel<DREAL>(size), scale(s), initialized(false),
+	normal(NULL), normal_length(0)
 {
 	properties |= KP_LINADD;
 	init(l,r);
@@ -33,7 +33,7 @@ CLinearKernel::~CLinearKernel()
 {
 	cleanup();
 }
-  
+
 bool CLinearKernel::init(CFeatures* l, CFeatures* r)
 {
 	CSimpleKernel<DREAL>::init(l, r);
@@ -62,6 +62,8 @@ void CLinearKernel::init_rescale()
 void CLinearKernel::cleanup()
 {
 	delete_optimization();
+
+	CKernel::cleanup();
 }
 
 bool CLinearKernel::load_init(FILE* src)
