@@ -1,11 +1,11 @@
 /*
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
- * Written (W) 1999-2007 Soeren Sonnenburg
- * Copyright (C) 1999-2007 Fraunhofer Institute FIRST and Max-Planck-Society
+ * Written (W) 1999-2008 Soeren Sonnenburg
+ * Copyright (C) 1999-2008 Fraunhofer Institute FIRST and Max-Planck-Society
  */
 
 #include "lib/common.h"
@@ -15,14 +15,14 @@
 #include "features/WordFeatures.h"
 
 CLinearWordKernel::CLinearWordKernel(INT size, bool dr, DREAL s)
-	: CSimpleKernel<WORD>(size), scale(s), do_rescale(dr), initialized(false),
+: CSimpleKernel<WORD>(size), scale(s), do_rescale(dr), initialized(false),
 	normal(NULL)
 {
 }
 
 CLinearWordKernel::CLinearWordKernel(
 	CWordFeatures* l, CWordFeatures* r, bool dr, DREAL s)
-	: CSimpleKernel<WORD>(10), scale(s), do_rescale(dr), initialized(false),
+: CSimpleKernel<WORD>(10), scale(s), do_rescale(dr), initialized(false),
 	normal(NULL)
 {
 	init(l, r);
@@ -64,6 +64,8 @@ void CLinearWordKernel::init_rescale()
 void CLinearWordKernel::cleanup()
 {
 	delete_optimization();
+
+	CKernel::cleanup();
 }
 
 bool CLinearWordKernel::load_init(FILE* src)
@@ -103,8 +105,8 @@ DREAL CLinearWordKernel::compute(INT idx_a, INT idx_b)
 
 	WORD* avec=((CWordFeatures*) lhs)->get_feature_vector(idx_a, alen, afree);
 	WORD* bvec=((CWordFeatures*) rhs)->get_feature_vector(idx_b, blen, bfree);
-
 	ASSERT(alen==blen);
+
 	double sum=0;
 	for (LONG i=0; i<alen; i++)
 		sum+=((LONG) avec[i])*((LONG) bvec[i]);
@@ -126,8 +128,6 @@ bool CLinearWordKernel::init_optimization(INT num_suppvec, INT* sv_idx, DREAL* a
 	ASSERT(num_feat);
 
 	normal=new DREAL[num_feat];
-	ASSERT(normal);
-
 	for (INT i=0; i<num_feat; i++)
 		normal[i]=0;
 
