@@ -1,11 +1,11 @@
 /*
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * Written (W) 2006 Konrad Rieck
- * Copyright (C) 1999-2007 Fraunhofer Institute FIRST and Max-Planck-Society
+ * Copyright (C) 1999-2008 Fraunhofer Institute FIRST and Max-Planck-Society
  *
  * Indentation: bcpp -f 1 -s -ylcnc -bcl -i 4
  */
@@ -38,65 +38,68 @@ param_spec_t p_map[] = {
  * @param meas Similarity measure to use
  * @param w Kernel width
  */
-CMindyGramKernel::CMindyGramKernel(INT ch, CHAR *meas, DREAL w) : CKernel(ch)
+CMindyGramKernel::CMindyGramKernel(INT ch, CHAR *meas, DREAL w)
+: CKernel(ch)
 {
-    /* Init attributes */
-    sdiag_lhs = NULL;
-    sdiag_rhs = NULL;
-    initialized = false;
-    measure = meas;
-    norm = NO_NORMALIZATION;
-    width = w;
-    cache = 0;
-    
-    /* Check for similarity coefficients */
-    simcof = sico_get_type(measure);
+	/* Init attributes */
+	sdiag_lhs=NULL;
+	sdiag_rhs=NULL;
+	initialized=false;
+	measure=meas;
+	norm=NO_NORMALIZATION;
+	width=w;
+	cache=0;
+	
+	/* Check for similarity coefficients */
+	simcof=sico_get_type(measure);
 
-    /* Create similarity measure */
-    SG_INFO( "Initializing Mindy kernel\n");
-    if (simcof == SC_NONE)
-        kernel = sm_create(sm_get_type(measure));
-    else
-        kernel = sm_create(ST_MINKERN);
+	/* Create similarity measure */
+	SG_INFO("Initializing Mindy kernel.\n");
+	if (simcof==SC_NONE)
+		kernel=sm_create(sm_get_type(measure));
+	else
+		kernel=sm_create(ST_MINKERN);
    
-    SG_INFO( "Mindy similarity measure: %s (using %s)\n", 
-	     measure, sm_get_descr(kernel->type));
+	SG_INFO("Mindy similarity measure: %s (using %s).\n",
+		measure, sm_get_descr(kernel->type));
 
-    /* Initialize optimization */
-    if (kernel->type == ST_LINEAR) {
-	SG_INFO( "Optimization supported\n");
-        properties |= KP_LINADD;
-    }
+	/* Initialize optimization */
+	if (kernel->type == ST_LINEAR)
+	{
+		SG_INFO("Optimization supported.\n");
+		properties |= KP_LINADD;
+	}
 
-    normal = NULL;
-    clear_normal();
+	normal=NULL;
+	clear_normal();
 }
 
 CMindyGramKernel::CMindyGramKernel(
 	CFeatures* l, CFeatures* r, CHAR *m, DREAL w)
-	: CKernel(10), sdiag_lhs(NULL), sdiag_rhs(NULL), initialized(false),
-		measure(m), norm(NO_NORMALIZATION), width(w)
+: CKernel(10), sdiag_lhs(NULL), sdiag_rhs(NULL), initialized(false),
+	measure(m), norm(NO_NORMALIZATION), width(w)
 {
 	/* Check for similarity coefficients */
-	simcof = sico_get_type(measure);
+	simcof=sico_get_type(measure);
 
 	/* Create similarity measure */
-	SG_INFO( "Initializing Mindy kernel\n");
-	if (simcof == SC_NONE)
-		kernel = sm_create(sm_get_type(measure));
+	SG_INFO("Initializing Mindy kernel.\n");
+	if (simcof==SC_NONE)
+		kernel=sm_create(sm_get_type(measure));
 	else
-		kernel = sm_create(ST_MINKERN);
+		kernel=sm_create(ST_MINKERN);
    
-	SG_INFO( "Mindy similarity measure: %s (using %s)\n", 
+	SG_INFO("Mindy similarity measure: %s (using %s).\n",
 		 measure, sm_get_descr(kernel->type));
 
 	/* Initialize optimization */
-	if (kernel->type == ST_LINEAR) {
-	SG_INFO( "Optimization supported\n");
+	if (kernel->type == ST_LINEAR)
+	{
+		SG_INFO("Optimization supported.\n");
 		properties |= KP_LINADD;
 	}
 
-	normal = NULL;
+	normal=NULL;
 	clear_normal();
 
 	init(l, r);
@@ -197,7 +200,8 @@ void CMindyGramKernel::cleanup()
 {
     delete_optimization();
     clear_normal();
-    remove_lhs();
+
+	CKernel::cleanup();
 }
 
 /**
@@ -260,10 +264,10 @@ bool CMindyGramKernel::init(CFeatures* l, CFeatures* r)
     INT i;
 
     /* Assert correct types of features */
-    ASSERT(l->get_feature_class() == C_MINDYGRAM);
-    ASSERT(r->get_feature_class() == C_MINDYGRAM);
-    ASSERT(l->get_feature_type() == F_ULONG);
-    ASSERT(r->get_feature_type() == F_ULONG);
+    ASSERT(l->get_feature_class()== C_MINDYGRAM);
+    ASSERT(r->get_feature_class()==C_MINDYGRAM);
+    ASSERT(l->get_feature_type()==F_ULONG);
+    ASSERT(r->get_feature_type()==F_ULONG);
 
     /* Clean diagonals */
     if (sdiag_lhs != sdiag_rhs)
@@ -286,9 +290,6 @@ bool CMindyGramKernel::init(CFeatures* l, CFeatures* r)
             sdiag_rhs[i]=1;
     }
 
-    ASSERT(sdiag_lhs);
-    ASSERT(sdiag_rhs);
-
     this->lhs=(CMindyGramFeatures *) l;
     this->rhs=(CMindyGramFeatures *) l;
 
