@@ -1,11 +1,11 @@
 /*
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
- * Written (W) 1999-2007 Soeren Sonnenburg
- * Copyright (C) 1999-2007 Fraunhofer Institute FIRST and Max-Planck-Society
+ * Written (W) 1999-2008 Soeren Sonnenburg
+ * Copyright (C) 1999-2008 Fraunhofer Institute FIRST and Max-Planck-Society
  */
 
 #include "lib/config.h"
@@ -15,16 +15,15 @@
 #include "features/RealFeatures.h"
 
 CPolyKernel::CPolyKernel(INT size, INT d, bool i, bool un)
-	: CSimpleKernel<DREAL>(size), degree(d), inhomogene(i),
-	sqrtdiag_lhs(NULL), sqrtdiag_rhs(NULL), initialized(false),
-	use_normalization(un)
+: CSimpleKernel<DREAL>(size), degree(d), inhomogene(i),
+	use_normalization(un), sqrtdiag_lhs(NULL), sqrtdiag_rhs(NULL),
+	initialized(false)
 {
 }
 
 CPolyKernel::CPolyKernel(CRealFeatures* l, CRealFeatures* r, INT d, bool i, bool un, INT size)
-	: CSimpleKernel<DREAL>(size),degree(d),inhomogene(i),
-	sqrtdiag_lhs(NULL), sqrtdiag_rhs(NULL), initialized(false),
-	use_normalization(un)
+: CSimpleKernel<DREAL>(size),degree(d),inhomogene(i), use_normalization(un),
+	sqrtdiag_lhs(NULL), sqrtdiag_rhs(NULL), initialized(false)
 {
 	init(l,r);
 }
@@ -41,15 +40,15 @@ bool CPolyKernel::init(CFeatures* l, CFeatures* r)
 	initialized = false ;
 	INT i;
 
-	if (sqrtdiag_lhs != sqrtdiag_rhs)
+	if (sqrtdiag_lhs!=sqrtdiag_rhs)
 	  delete[] sqrtdiag_rhs;
-	sqrtdiag_rhs=NULL ;
+	sqrtdiag_rhs=NULL;
 	delete[] sqrtdiag_lhs;
-	sqrtdiag_lhs=NULL ;
+	sqrtdiag_lhs=NULL;
 
 	if (use_normalization)
 	{
-		sqrtdiag_lhs= new DREAL[lhs->get_num_vectors()];
+		sqrtdiag_lhs=new DREAL[lhs->get_num_vectors()];
 
 		for (i=0; i<lhs->get_num_vectors(); i++)
 			sqrtdiag_lhs[i]=1;
@@ -58,14 +57,11 @@ bool CPolyKernel::init(CFeatures* l, CFeatures* r)
 			sqrtdiag_rhs=sqrtdiag_lhs;
 		else
 		{
-			sqrtdiag_rhs= new DREAL[rhs->get_num_vectors()];
+			sqrtdiag_rhs=new DREAL[rhs->get_num_vectors()];
 			for (i=0; i<rhs->get_num_vectors(); i++)
 				sqrtdiag_rhs[i]=1;
 		}
 
-		ASSERT(sqrtdiag_lhs);
-		ASSERT(sqrtdiag_rhs);
-
 		this->lhs=(CRealFeatures*) l;
 		this->rhs=(CRealFeatures*) l;
 
@@ -115,6 +111,7 @@ void CPolyKernel::cleanup()
 	sqrtdiag_lhs=NULL;
 
 	initialized=false;
+	CKernel::cleanup();
 }
 
 bool CPolyKernel::load_init(FILE* src)
@@ -136,7 +133,6 @@ DREAL CPolyKernel::compute(INT idx_a, INT idx_b)
 
   double* avec=((CRealFeatures*) lhs)->get_feature_vector(idx_a, alen, afree);
   double* bvec=((CRealFeatures*) rhs)->get_feature_vector(idx_b, blen, bfree);
-  
   ASSERT(alen==blen);
 
   DREAL sqrt_a= 1.0;
