@@ -1,11 +1,11 @@
 /*
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
- * Written (W) 1999-2007 Soeren Sonnenburg
- * Copyright (C) 1999-2007 Fraunhofer Institute FIRST and Max-Planck-Society
+ * Written (W) 1999-2008 Soeren Sonnenburg
+ * Copyright (C) 1999-2008 Fraunhofer Institute FIRST and Max-Planck-Society
  */
 
 #include "lib/common.h"
@@ -16,26 +16,26 @@
 #include "kernel/SparseKernel.h"
 
 CSparseLinearKernel::CSparseLinearKernel(INT size, DREAL s)
-	: CSparseKernel<DREAL>(size), scale(s), initialized(false),
-	normal_length(0), normal(NULL)
+: CSparseKernel<DREAL>(size), scale(s), initialized(false), normal(NULL),
+	normal_length(0)
 {
 	properties |= KP_LINADD;
 }
 
 CSparseLinearKernel::CSparseLinearKernel(
 	CSparseFeatures<DREAL>* l, CSparseFeatures<DREAL>* r, DREAL s, INT size)
-	: CSparseKernel<DREAL>(size), scale(s), initialized(false),
-	normal_length(0), normal(NULL)
+: CSparseKernel<DREAL>(size), scale(s), initialized(false), normal(NULL),
+	normal_length(0)
 {
 	properties |= KP_LINADD;
 	init(l,r);
 }
 
-CSparseLinearKernel::~CSparseLinearKernel() 
+CSparseLinearKernel::~CSparseLinearKernel()
 {
 	cleanup();
 }
-  
+
 bool CSparseLinearKernel::init(CFeatures* l, CFeatures* r)
 {
 	CSparseKernel<DREAL>::init(l, r);
@@ -64,6 +64,8 @@ void CSparseLinearKernel::init_rescale()
 void CSparseLinearKernel::cleanup()
 {
 	delete_optimization();
+
+	CKernel::cleanup();
 }
 
 bool CSparseLinearKernel::load_init(FILE* src)
