@@ -1,11 +1,11 @@
 /*
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
- * Written (W) 1999-2007 Soeren Sonnenburg
- * Copyright (C) 1999-2007 Fraunhofer Institute FIRST and Max-Planck-Society
+ * Written (W) 1999-2008 Soeren Sonnenburg
+ * Copyright (C) 1999-2008 Fraunhofer Institute FIRST and Max-Planck-Society
  */
 
 #include "lib/common.h"
@@ -14,22 +14,22 @@
 #include "features/SparseFeatures.h"
 
 CSparsePolyKernel::CSparsePolyKernel(INT size, INT d, bool i, bool un)
-	: CSparseKernel<DREAL>(size), degree(d), inhomogene(i),
-	sqrtdiag_lhs(NULL), sqrtdiag_rhs(NULL), initialized(false),
-	use_normalization(un)
+: CSparseKernel<DREAL>(size), degree(d),
+	inhomogene(i),use_normalization(un), sqrtdiag_lhs(NULL), sqrtdiag_rhs(NULL),
+	initialized(false)
 {
 }
 
 CSparsePolyKernel::CSparsePolyKernel(
 	CSparseFeatures<DREAL>* l, CSparseFeatures<DREAL>* r,
 	INT size, INT d, bool i, bool un)
-	: CSparseKernel<DREAL>(size),degree(d),inhomogene(i), sqrtdiag_lhs(NULL),
-	sqrtdiag_rhs(NULL), initialized(false), use_normalization(un)
+: CSparseKernel<DREAL>(size),degree(d),inhomogene(i), use_normalization(un),
+	sqrtdiag_lhs(NULL), sqrtdiag_rhs(NULL), initialized(false)
 {
 	init(l,r);
 }
 
-CSparsePolyKernel::~CSparsePolyKernel() 
+CSparsePolyKernel::~CSparsePolyKernel()
 {
 	cleanup();
 }
@@ -63,9 +63,6 @@ bool CSparsePolyKernel::init(CFeatures* l, CFeatures* r)
 				sqrtdiag_rhs[i]=1;
 		}
 
-		ASSERT(sqrtdiag_lhs);
-		ASSERT(sqrtdiag_rhs);
-
 		this->lhs=(CSparseFeatures<DREAL>*) l;
 		this->rhs=(CSparseFeatures<DREAL>*) l;
 
@@ -115,6 +112,8 @@ void CSparsePolyKernel::cleanup()
 	sqrtdiag_lhs=NULL;
 
 	initialized=false;
+
+	CKernel::cleanup();
 }
 
 bool CSparsePolyKernel::load_init(FILE* src)
