@@ -1,12 +1,12 @@
 /*
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
- * Written (W) 1999-2007 Soeren Sonnenburg
- * Written (W) 1999-2007 Gunnar Raetsch
- * Copyright (C) 1999-2007 Fraunhofer Institute FIRST and Max-Planck-Society
+ * Written (W) 1999-2008 Soeren Sonnenburg
+ * Written (W) 1999-2008 Gunnar Raetsch
+ * Copyright (C) 1999-2008 Fraunhofer Institute FIRST and Max-Planck-Society
  */
 
 #include "lib/common.h"
@@ -19,6 +19,8 @@
 #include "features/Features.h"
 #include "features/StringFeatures.h"
 
+#include "classifier/svm/SVM.h"
+
 #ifndef WIN32
 #include <pthread.h>
 #endif
@@ -44,37 +46,38 @@ template <class Trie> struct S_THREAD_PARAM
 
 CWeightedDegreePositionStringKernel::CWeightedDegreePositionStringKernel(
 	INT size, INT d, INT mm, bool un, INT mkls)
-	: CStringKernel<CHAR>(size), weights(NULL), position_weights(NULL),
+: CStringKernel<CHAR>(size), weights(NULL), position_weights(NULL),
 	position_weights_lhs(NULL), position_weights_rhs(NULL),
 	weights_buffer(NULL), mkl_stepsize(mkls), degree(d), length(0),
 	max_mismatch(mm), seq_length(0), shift(NULL), shift_len(0),
 	initialized(false), use_normalization(un),
 	normalization_const(1.0), num_block_weights_external(0),
 	block_weights_external(NULL), block_weights(NULL), type(E_EXTERNAL),
-	tries(d), poim_tries(d), tree_initialized(false), use_poim_tries(false)
+	tries(d), poim_tries(d), tree_initialized(false), use_poim_tries(false),
+	m_poim_distrib(NULL), m_poim(NULL), m_poim_num_sym(0), m_poim_num_feat(0),
+	m_poim_result_len(0), alphabet(NULL)
 {
 	properties |= KP_LINADD | KP_KERNCOMBINATION | KP_BATCHEVALUATION;
 	set_wd_weights();
 	ASSERT(weights);
-
 }
 
 CWeightedDegreePositionStringKernel::CWeightedDegreePositionStringKernel(
 	INT size, DREAL* w, INT d, INT mm, INT* s, INT sl, bool un, INT mkls)
-	: CStringKernel<CHAR>(size), weights(NULL), position_weights(NULL),
+: CStringKernel<CHAR>(size), weights(NULL), position_weights(NULL),
 	position_weights_lhs(NULL), position_weights_rhs(NULL),
 	weights_buffer(NULL), mkl_stepsize(mkls), degree(d), length(0),
-	max_mismatch(mm), seq_length(0), shift(NULL),
+	max_mismatch(mm), seq_length(0), shift(NULL), shift_len(0),
 	initialized(false), use_normalization(un),
 	normalization_const(1.0), num_block_weights_external(0),
 	block_weights_external(NULL), block_weights(NULL), type(E_EXTERNAL),
-	tries(d), poim_tries(d), tree_initialized(false), use_poim_tries(false)
+	tries(d), poim_tries(d), tree_initialized(false), use_poim_tries(false),
+	m_poim_distrib(NULL), m_poim(NULL), m_poim_num_sym(0), m_poim_num_feat(0),
+	m_poim_result_len(0), alphabet(NULL)
 {
 	properties |= KP_LINADD | KP_KERNCOMBINATION | KP_BATCHEVALUATION;
 
 	weights=new DREAL[d*(1+max_mismatch)];
-
-	ASSERT(weights);
 	for (INT i=0; i<d*(1+max_mismatch); i++)
 		weights[i]=w[i];
 
@@ -83,84 +86,69 @@ CWeightedDegreePositionStringKernel::CWeightedDegreePositionStringKernel(
 
 CWeightedDegreePositionStringKernel::CWeightedDegreePositionStringKernel(
 	CStringFeatures<CHAR>* l, CStringFeatures<CHAR>* r, INT d)
-	: CStringKernel<CHAR>(10), weights(NULL), position_weights(NULL),
+: CStringKernel<CHAR>(10), weights(NULL), position_weights(NULL),
 	position_weights_lhs(NULL), position_weights_rhs(NULL),
 	weights_buffer(NULL), mkl_stepsize(1), degree(d), length(0),
-	max_mismatch(0), seq_length(0), shift(NULL),
+	max_mismatch(0), seq_length(0), shift(NULL), shift_len(0),
 	initialized(false), use_normalization(true),
 	normalization_const(1.0), num_block_weights_external(0),
 	block_weights_external(NULL), block_weights(NULL), type(E_EXTERNAL),
-	tries(d), poim_tries(d), tree_initialized(false), use_poim_tries(false)
+	tries(d), poim_tries(d), tree_initialized(false), use_poim_tries(false),
+	m_poim_distrib(NULL), m_poim(NULL), m_poim_num_sym(0), m_poim_num_feat(0),
+	m_poim_result_len(0), alphabet(NULL)
 {
 	properties |= KP_LINADD | KP_KERNCOMBINATION | KP_BATCHEVALUATION;
 	set_wd_weights();
 	ASSERT(weights);
 
-	shift_len=l->get_vector_length(0);
-	INT *shifts=new INT[shift_len];
-	ASSERT(shifts);
-	for (INT i=0; i<shift_len; i++) {
-		shifts[i]=1;
-	}
-	set_shifts(shifts, shift_len);
-	delete[] shifts;
-
 	init(l, r);
 }
 
 
-CWeightedDegreePositionStringKernel::~CWeightedDegreePositionStringKernel() 
+CWeightedDegreePositionStringKernel::~CWeightedDegreePositionStringKernel()
 {
 	cleanup();
+	cleanup_POIM2();
 
 	delete[] shift;
-	shift = NULL;
+	shift=NULL;
 
-	delete[] weights ;
+	delete[] weights;
 	weights=NULL ;
 
-	delete[] position_weights ;
-	position_weights=NULL ;
+	delete[] block_weights;
+	block_weights=NULL;
 
-	delete[] position_weights_lhs ;
-	position_weights_lhs=NULL ;
+	delete[] position_weights;
+	position_weights=NULL;
 
-	delete[] position_weights_rhs ;
-	position_weights_rhs=NULL ;
+	delete[] position_weights_lhs;
+	position_weights_lhs=NULL;
 
-	delete[] weights_buffer ;
-	weights_buffer = NULL ;
+	delete[] position_weights_rhs;
+	position_weights_rhs=NULL;
+
+	delete[] weights_buffer;
+	weights_buffer=NULL;
 }
 
-void CWeightedDegreePositionStringKernel::remove_lhs() 
-{ 
+void CWeightedDegreePositionStringKernel::remove_lhs()
+{
 	SG_DEBUG( "deleting CWeightedDegreePositionStringKernel optimization\n");
 	delete_optimization();
+	initialized = false;
 
+	tries.destroy();
+	poim_tries.destroy();
 
-
-	lhs = NULL ; 
-	rhs = NULL ; 
-	initialized = false ;
-
-	tries.destroy() ;
-	poim_tries.destroy() ;
-} ;
-
-void CWeightedDegreePositionStringKernel::remove_rhs()
-{
-
-
-	rhs = lhs ;
+	CKernel::remove_lhs();
 }
 
-
 void CWeightedDegreePositionStringKernel::create_empty_tries()
 {
+	ASSERT(lhs);
 	seq_length = ((CStringFeatures<CHAR>*) lhs)->get_max_vector_length();
 
-	tries.destroy() ;
-	poim_tries.destroy() ;
 	if (opt_type==SLOWBUTMEMEFFICIENT)
 	{
 		tries.create(seq_length, true); 
@@ -185,16 +173,47 @@ bool CWeightedDegreePositionStringKernel::init(CFeatures* l, CFeatures* r)
 	SG_DEBUG( "lhs_changed: %i\n", lhs_changed) ;
 	SG_DEBUG( "rhs_changed: %i\n", rhs_changed) ;
 
-	ASSERT(((((CStringFeatures<CHAR>*) l)->get_alphabet()->get_alphabet()==DNA) || 
-				(((CStringFeatures<CHAR>*) l)->get_alphabet()->get_alphabet()==RNA)));
-	ASSERT(((((CStringFeatures<CHAR>*) r)->get_alphabet()->get_alphabet()==DNA) || 
-				(((CStringFeatures<CHAR>*) r)->get_alphabet()->get_alphabet()==RNA)));
+	CStringFeatures<CHAR>* sf_l=(CStringFeatures<CHAR>*) l;
+	CStringFeatures<CHAR>* sf_r=(CStringFeatures<CHAR>*) r;
 
-	if (lhs_changed) 
+	/* set shift */
+	if (shift_len==0) {
+		shift_len=sf_l->get_vector_length(0);
+		INT *shifts=new INT[shift_len];
+		for (INT i=0; i<shift_len; i++) {
+			shifts[i]=1;
+		}
+		set_shifts(shifts, shift_len);
+		delete[] shifts;
+	}
+
+
+	INT len=sf_l->get_max_vector_length();
+	if (lhs_changed && !sf_l->have_same_length(len))
+		SG_ERROR("All strings in WD kernel must have same length (lhs wrong)!\n");
+
+	if (rhs_changed && !sf_r->have_same_length(len))
+		SG_ERROR("All strings in WD kernel must have same length (rhs wrong)!\n");
+
+	delete alphabet;
+	alphabet= new CAlphabet(sf_l->get_alphabet());
+	CAlphabet* ralphabet=sf_r->get_alphabet();
+	if (!((alphabet->get_alphabet()==DNA) || (alphabet->get_alphabet()==RNA)))
+		properties &= ((ULONG) (-1)) ^ (KP_LINADD | KP_BATCHEVALUATION);
+
+	ASSERT(ralphabet->get_alphabet()==alphabet->get_alphabet());
+	SG_UNREF(ralphabet);
+
+	//whenever init is called also init tries and block weights
+	if (!initialized)
 	{
 		create_empty_tries();
 		init_block_weights();
+	}
 
+	//whenever lhs changes also recompute normalisation const
+	if (lhs_changed)
+	{
 		normalization_const=1.0;
 		if (use_normalization)
 			normalization_const=compute(0,0);
@@ -203,8 +222,7 @@ bool CWeightedDegreePositionStringKernel::init(CFeatures* l, CFeatures* r)
 	SG_DEBUG( "use normalization:%d (const:%f)\n", (use_normalization) ? 1 : 0,
 			normalization_const);
 
-
-	initialized = true ;
+	initialized = true;
 	return result;
 }
 
@@ -213,15 +231,20 @@ void CWeightedDegreePositionStringKernel::cleanup()
 	SG_DEBUG( "deleting CWeightedDegreePositionStringKernel optimization\n");
 	delete_optimization();
 
-	tries.destroy() ;
-	poim_tries.destroy() ;
+	delete[] block_weights;
+	block_weights=NULL;
 
-	lhs = NULL;
-	rhs = NULL;
+	tries.destroy();
+	poim_tries.destroy();
 
 	seq_length = 0;
 	initialized = false;
 	tree_initialized = false;
+
+	delete alphabet;
+	alphabet=NULL;
+
+	CKernel::cleanup();
 }
 
 bool CWeightedDegreePositionStringKernel::load_init(FILE* src)
@@ -236,9 +259,9 @@ bool CWeightedDegreePositionStringKernel::save_init(FILE* dest)
 
 bool CWeightedDegreePositionStringKernel::init_optimization(INT p_count, INT * IDX, DREAL * alphas, INT tree_num, INT upto_tree)
 {
-	ASSERT(position_weights_lhs==NULL) ;
-	ASSERT(position_weights_rhs==NULL) ;
-	
+	ASSERT(position_weights_lhs==NULL);
+	ASSERT(position_weights_rhs==NULL);
+
 	if (upto_tree<0)
 		upto_tree=tree_num;
 
@@ -250,6 +273,7 @@ bool CWeightedDegreePositionStringKernel::init_optimization(INT p_count, INT * I
 
 	if (tree_num<0)
 		SG_DEBUG( "deleting CWeightedDegreePositionStringKernel optimization\n");
+
 	delete_optimization();
 
 	if (tree_num<0)
@@ -272,7 +296,7 @@ bool CWeightedDegreePositionStringKernel::init_optimization(INT p_count, INT * I
 	}
 
 	if (tree_num<0)
-		SG_DEBUG( "done.           \n");
+		SG_DONE();
 
 	set_is_initialized(true) ;
 	return true ;
@@ -589,15 +613,14 @@ DREAL CWeightedDegreePositionStringKernel::compute(INT idx_a, INT idx_b)
 
 	CHAR* avec=((CStringFeatures<CHAR>*) lhs)->get_feature_vector(idx_a, alen);
 	CHAR* bvec=((CStringFeatures<CHAR>*) rhs)->get_feature_vector(idx_b, blen);
-
 	// can only deal with strings of same length
-	ASSERT(alen == blen);
-	ASSERT(shift_len == alen) ;
+	ASSERT(alen==blen);
+	ASSERT(shift_len==alen);
 
 	DREAL result = 0 ;
 	if (position_weights_lhs!=NULL || position_weights_rhs!=NULL)
 	{
-		ASSERT(max_mismatch==0) ;
+		ASSERT(max_mismatch==0);
 		result = compute_without_mismatch_position_weights(avec, &position_weights_lhs[idx_a*alen], alen, bvec, &position_weights_rhs[idx_b*blen], blen) ;
 	}
 	else if (max_mismatch > 0)
@@ -615,21 +638,23 @@ DREAL CWeightedDegreePositionStringKernel::compute(INT idx_a, INT idx_b)
 
 void CWeightedDegreePositionStringKernel::add_example_to_tree(INT idx, DREAL alpha)
 {
-	ASSERT(position_weights_lhs==NULL) ;
-	ASSERT(position_weights_rhs==NULL) ;
+	ASSERT(position_weights_lhs==NULL);
+	ASSERT(position_weights_rhs==NULL);
+	ASSERT(alphabet);
+	ASSERT(alphabet->get_alphabet()==DNA || alphabet->get_alphabet()==RNA);
 
-	INT len ;
+	INT len=0;
 	CHAR* char_vec=((CStringFeatures<CHAR>*) lhs)->get_feature_vector(idx, len);
-	ASSERT(max_mismatch==0) ;
+	ASSERT(max_mismatch==0);
 	INT *vec = new INT[len] ;
 
 	for (INT i=0; i<len; i++)
-		vec[i]=((CStringFeatures<CHAR>*) lhs)->get_alphabet()->remap_to_bin(char_vec[i]);
+		vec[i]=alphabet->remap_to_bin(char_vec[i]);
 
 	if (opt_type==FASTBUTMEMHUNGRY)
 	{
 		//TRIES(set_use_compact_terminal_nodes(false)) ;
-		ASSERT(!TRIES(get_use_compact_terminal_nodes())) ;
+		ASSERT(!TRIES(get_use_compact_terminal_nodes()));
 	}
 
 	for (INT i=0; i<len; i++)
@@ -664,28 +689,29 @@ void CWeightedDegreePositionStringKernel::add_example_to_tree(INT idx, DREAL alp
 
 void CWeightedDegreePositionStringKernel::add_example_to_single_tree(INT idx, DREAL alpha, INT tree_num) 
 {
-	ASSERT(position_weights_lhs==NULL) ;
-	ASSERT(position_weights_rhs==NULL) ;
+	ASSERT(position_weights_lhs==NULL);
+	ASSERT(position_weights_rhs==NULL);
+	ASSERT(alphabet);
+	ASSERT(alphabet->get_alphabet()==DNA || alphabet->get_alphabet()==RNA);
 
-	INT len ;
+	INT len=0;
 	CHAR* char_vec=((CStringFeatures<CHAR>*) lhs)->get_feature_vector(idx, len);
-	ASSERT(max_mismatch==0) ;
-	INT *vec = new INT[len] ;
-
+	ASSERT(max_mismatch==0);
+	INT *vec=new INT[len];
 	INT max_s=-1;
 
 	if (opt_type==SLOWBUTMEMEFFICIENT)
 		max_s=0;
 	else if (opt_type==FASTBUTMEMHUNGRY)
 	{
-		ASSERT(!tries.get_use_compact_terminal_nodes()) ;
+		ASSERT(!tries.get_use_compact_terminal_nodes());
 		max_s=shift[tree_num];
 	}
 	else {
 		SG_ERROR( "unknown optimization type\n");
 	}
 	for (INT i=CMath::max(0,tree_num-max_shift); i<CMath::min(len,tree_num+degree+max_shift); i++)
-		vec[i]=((CStringFeatures<CHAR>*) lhs)->get_alphabet()->remap_to_bin(char_vec[i]);
+		vec[i]=alphabet->remap_to_bin(char_vec[i]);
 
 	for (INT s=max_s; s>=0; s--)
 	{
@@ -713,17 +739,19 @@ void CWeightedDegreePositionStringKernel::add_example_to_single_tree(INT idx, DR
 
 DREAL CWeightedDegreePositionStringKernel::compute_by_tree(INT idx)
 {
-	ASSERT(position_weights_lhs==NULL) ;
-	ASSERT(position_weights_rhs==NULL) ;
+	ASSERT(position_weights_lhs==NULL);
+	ASSERT(position_weights_rhs==NULL);
+	ASSERT(alphabet);
+	ASSERT(alphabet->get_alphabet()==DNA || alphabet->get_alphabet()==RNA);
 
-	DREAL sum = 0 ;
-	INT len ;
+	DREAL sum=0;
+	INT len=0;
 	CHAR* char_vec=((CStringFeatures<CHAR>*) rhs)->get_feature_vector(idx, len);
-	ASSERT(max_mismatch==0) ;
-	INT *vec = new INT[len] ;
+	ASSERT(max_mismatch==0);
+	INT *vec=new INT[len];
 
 	for (INT i=0; i<len; i++)
-		vec[i]=((CStringFeatures<CHAR>*) lhs)->get_alphabet()->remap_to_bin(char_vec[i]);
+		vec[i]=alphabet->remap_to_bin(char_vec[i]);
 
 	for (INT i=0; i<len; i++)
 		sum += tries.compute_by_tree_helper(vec, len, i, i, i, weights, (length!=0)) ;
@@ -747,16 +775,18 @@ DREAL CWeightedDegreePositionStringKernel::compute_by_tree(INT idx)
 
 void CWeightedDegreePositionStringKernel::compute_by_tree(INT idx, DREAL* LevelContrib)
 {
-	ASSERT(position_weights_lhs==NULL) ;
-	ASSERT(position_weights_rhs==NULL) ;
+	ASSERT(position_weights_lhs==NULL);
+	ASSERT(position_weights_rhs==NULL);
+	ASSERT(alphabet);
+	ASSERT(alphabet->get_alphabet()==DNA || alphabet->get_alphabet()==RNA);
 
-	INT len ;
+	INT len=0;
 	CHAR* char_vec=((CStringFeatures<CHAR>*) rhs)->get_feature_vector(idx, len);
-	ASSERT(max_mismatch==0) ;
-	INT *vec = new INT[len] ;
+	ASSERT(max_mismatch==0);
+	INT *vec=new INT[len];
 
 	for (INT i=0; i<len; i++)
-		vec[i]=((CStringFeatures<CHAR>*) lhs)->get_alphabet()->remap_to_bin(char_vec[i]);
+		vec[i]=alphabet->remap_to_bin(char_vec[i]);
 
 	for (INT i=0; i<len; i++)
 		tries.compute_by_tree_helper(vec, len, i, i, i, LevelContrib, 1.0/normalization_const, mkl_stepsize, weights, (length!=0)) ;
@@ -797,7 +827,7 @@ bool CWeightedDegreePositionStringKernel::set_shifts(INT* shift_, INT shift_len_
 				max_shift = shift[i] ;
 		}
 
-		ASSERT(max_shift>=0 && max_shift<=shift_len) ;
+		ASSERT(max_shift>=0 && max_shift<=shift_len);
 	}
 	
 	return false;
@@ -915,14 +945,14 @@ bool CWeightedDegreePositionStringKernel::set_position_weights_lhs(DREAL* pws, I
 		SG_ERROR( "seq_length = %i, position_weights_length=%i\n", seq_length, len) ;
 		return false ;
 	}
-	if (!get_lhs())
+	if (!lhs)
 	{
-		SG_ERROR("get_lhs()=NULL\n") ;
+		SG_ERROR("lhs=NULL\n") ;
 		return false ;
 	}
-	if (get_lhs()->get_num_vectors()!=num)
+	if (lhs->get_num_vectors()!=num)
 	{
-		SG_ERROR("get_lhs()->get_num_vectors()=%i, num=%i\n", get_lhs()->get_num_vectors(), num) ;
+		SG_ERROR("lhs->get_num_vectors()=%i, num=%i\n", lhs->get_num_vectors(), num) ;
 		return false ;
 	}
 	
@@ -956,22 +986,22 @@ bool CWeightedDegreePositionStringKernel::set_position_weights_rhs(DREAL* pws, I
 		SG_ERROR( "seq_length = %i, position_weights_length=%i\n", seq_length, len) ;
 		return false ;
 	}
-	if (!get_rhs())
+	if (!rhs)
 	{
-		if (!get_lhs())
+		if (!lhs)
 		{
-			SG_ERROR("get_rhs()==0 and get_lhs()=NULL\n") ;
+			SG_ERROR("rhs==0 and lhs=NULL\n") ;
 			return false ;
 		}
-		if (get_lhs()->get_num_vectors()!=num)
+		if (lhs->get_num_vectors()!=num)
 		{
-			SG_ERROR("get_lhs()->get_num_vectors()=%i, num=%i\n", get_lhs()->get_num_vectors(), num) ;
+			SG_ERROR("lhs->get_num_vectors()=%i, num=%i\n", lhs->get_num_vectors(), num) ;
 			return false ;
 		}
 	} else
-		if (get_rhs()->get_num_vectors()!=num)
+		if (rhs->get_num_vectors()!=num)
 		{
-			SG_ERROR("get_rhs()->get_num_vectors()=%i, num=%i\n", get_rhs()->get_num_vectors(), num) ;
+			SG_ERROR("rhs->get_num_vectors()=%i, num=%i\n", rhs->get_num_vectors(), num) ;
 			return false ;
 		}
 
@@ -1196,9 +1226,14 @@ void* CWeightedDegreePositionStringKernel::compute_batch_helper(void* p)
 	for (INT i=params->start; i<params->end; i++)
 	{
 		INT len=0;
-		CHAR* char_vec=((CStringFeatures<CHAR>*) wd->get_rhs())->get_feature_vector(vec_idx[i], len);
+		CStringFeatures<CHAR>* rhs_feat=((CStringFeatures<CHAR>*) wd->get_rhs());
+		CAlphabet* alpha=wd->alphabet;
+
+		CHAR* char_vec=rhs_feat->get_feature_vector(vec_idx[i], len);
 		for (INT k=CMath::max(0,j-max_shift); k<CMath::min(len,j+wd->get_degree()+max_shift); k++)
-			vec[k]=((CStringFeatures<CHAR>*) wd->get_lhs())->get_alphabet()->remap_to_bin(char_vec[k]);
+			vec[k]=alpha->remap_to_bin(char_vec[k]);
+
+		SG_UNREF(rhs_feat);
 
 		result[i] += factor*tries->compute_by_tree_helper(vec, len, j, j, j, weights, (length!=0))/wd->normalization_const ;
 
@@ -1220,28 +1255,30 @@ void* CWeightedDegreePositionStringKernel::compute_batch_helper(void* p)
 
 void CWeightedDegreePositionStringKernel::compute_batch(INT num_vec, INT* vec_idx, DREAL* result, INT num_suppvec, INT* IDX, DREAL* alphas, DREAL factor)
 {
-	ASSERT(position_weights_lhs==NULL) ;
-	ASSERT(position_weights_rhs==NULL) ;
-
-	ASSERT(get_rhs());
-	ASSERT(num_vec<=get_rhs()->get_num_vectors());
+	ASSERT(alphabet);
+	ASSERT(alphabet->get_alphabet()==DNA || alphabet->get_alphabet()==RNA);
+	ASSERT(position_weights_lhs==NULL);
+	ASSERT(position_weights_rhs==NULL);
+	ASSERT(rhs);
+	ASSERT(num_vec<=rhs->get_num_vectors());
 	ASSERT(num_vec>0);
 	ASSERT(vec_idx);
 	ASSERT(result);
+	create_empty_tries();
 
-	INT num_feat=((CStringFeatures<CHAR>*) get_rhs())->get_max_vector_length();
+	INT num_feat=((CStringFeatures<CHAR>*) rhs)->get_max_vector_length();
 	ASSERT(num_feat>0);
 	INT num_threads=parallel.get_num_threads();
 	ASSERT(num_threads>0);
-	INT* vec= new INT[num_threads*num_feat];
-	ASSERT(vec);
+	INT* vec=new INT[num_threads*num_feat];
 
 	if (num_threads < 2)
 	{
 #ifdef WIN32
-		for (INT j=0; j<num_feat; j++)
+	   for (INT j=0; j<num_feat; j++)
 #else
-			for (INT j=0; j<num_feat && !CSignal::cancel_computations(); j++)
+       CSignal::clear() ;
+	   for (INT j=0; j<num_feat && !CSignal::cancel_computations(); j++)
 #endif
 			{
 				init_optimization(num_suppvec, IDX, alphas, j);
@@ -1267,6 +1304,8 @@ void CWeightedDegreePositionStringKernel::compute_batch(INT num_vec, INT* vec_id
 #ifndef WIN32
 	else
 	{
+
+		CSignal::clear() ;
 		for (INT j=0; j<num_feat && !CSignal::cancel_computations(); j++)
 		{
 			init_optimization(num_suppvec, IDX, alphas, j);
@@ -1324,39 +1363,36 @@ void CWeightedDegreePositionStringKernel::compute_batch(INT num_vec, INT* vec_id
 
 DREAL* CWeightedDegreePositionStringKernel::compute_scoring(INT max_degree, INT& num_feat, INT& num_sym, DREAL* result, INT num_suppvec, INT* IDX, DREAL* alphas)
 {
-	ASSERT(position_weights_lhs==NULL) ;
-	ASSERT(position_weights_rhs==NULL) ;
+	ASSERT(position_weights_lhs==NULL);
+	ASSERT(position_weights_rhs==NULL);
 
-	num_feat=((CStringFeatures<CHAR>*) get_rhs())->get_max_vector_length();
+	num_feat=((CStringFeatures<CHAR>*) rhs)->get_max_vector_length();
 	ASSERT(num_feat>0);
-	ASSERT(((CStringFeatures<CHAR>*) get_rhs())->get_alphabet()->get_alphabet() == DNA);
+	ASSERT(alphabet);
+	ASSERT(alphabet->get_alphabet()==DNA || alphabet->get_alphabet()==RNA);
+
 	num_sym=4; //for now works only w/ DNA
 
 	ASSERT(max_degree>0);
 
 	// === variables
-	INT* nofsKmers = new INT[ max_degree ];
-	DREAL** C = new DREAL*[ max_degree ];
-	DREAL** L = new DREAL*[ max_degree ];
-	DREAL** R = new DREAL*[ max_degree ];
-
-	ASSERT(nofsKmers);
-	ASSERT(C);
-	ASSERT(L);
-	ASSERT(R);
+	INT* nofsKmers=new INT[max_degree];
+	DREAL** C=new DREAL*[max_degree];
+	DREAL** L=new DREAL*[max_degree];
+	DREAL** R=new DREAL*[max_degree];
 
 	INT i;
 	INT k;
 
 	// --- return table
-	INT bigtabSize = 0;
-	for( k = 0; k < max_degree; ++k ) {
-		nofsKmers[k] = (INT) pow( num_sym, k+1 );
-		const INT tabSize = nofsKmers[k] * num_feat;
-		bigtabSize += tabSize;
+	INT bigtabSize=0;
+	for (k=0; k<max_degree; ++k )
+	{
+		nofsKmers[k]=(INT) pow(num_sym, k+1);
+		const INT tabSize=nofsKmers[k]*num_feat;
+		bigtabSize+=tabSize;
 	}
-	result= new DREAL[ bigtabSize ];
-	ASSERT(result);
+	result=new DREAL[bigtabSize];
 
 	// --- auxilliary tables
 	INT tabOffs=0;
@@ -1376,8 +1412,7 @@ DREAL* CWeightedDegreePositionStringKernel::compute_scoring(INT max_degree, INT&
 	}
 
 	// --- tree parsing info
-	DREAL* margFactors = new DREAL[ degree ];
-	ASSERT(margFactors);
+	DREAL* margFactors=new DREAL[degree];
 
 	INT* x = new INT[ degree+1 ];
 	INT* substrs = new INT[ degree+1 ];
@@ -1439,8 +1474,8 @@ DREAL* CWeightedDegreePositionStringKernel::compute_scoring(INT max_degree, INT&
 					for( sym = 0; sym < num_sym; ++sym ) {
 						const INT y_sym = num_sym*y + sym;
 						const INT sym_y = nofJmers*sym + y;
-						ASSERT( 0 <= y_sym && y_sym < nofKmers );
-						ASSERT( 0 <= sym_y && sym_y < nofKmers );
+						ASSERT(0<=y_sym && y_sym<nofKmers);
+						ASSERT(0<=sym_y && sym_y<nofKmers);
 						C[k][ y_sym + offsetK ] += L[j][ y + offsetJ ];
 						if( p < num_feat-1 ) {
 							C[k][ sym_y + offsetK ] += R[j][ y + offsetJ1 ];
@@ -1482,30 +1517,25 @@ DREAL* CWeightedDegreePositionStringKernel::compute_scoring(INT max_degree, INT&
 
 CHAR* CWeightedDegreePositionStringKernel::compute_consensus(INT &num_feat, INT num_suppvec, INT* IDX, DREAL* alphas)
 {
-	ASSERT(position_weights_lhs==NULL) ;
-	ASSERT(position_weights_rhs==NULL) ;
-
+	ASSERT(position_weights_lhs==NULL);
+	ASSERT(position_weights_rhs==NULL);
 	//only works for order <= 32
 	ASSERT(degree<=32);
 	ASSERT(!tries.get_use_compact_terminal_nodes());
-	num_feat=((CStringFeatures<CHAR>*) get_rhs())->get_max_vector_length();
+	num_feat=((CStringFeatures<CHAR>*) rhs)->get_max_vector_length();
 	ASSERT(num_feat>0);
-	ASSERT(((CStringFeatures<CHAR>*) get_rhs())->get_alphabet()->get_alphabet() == DNA);
+	ASSERT(alphabet);
+	ASSERT(alphabet->get_alphabet()==DNA || alphabet->get_alphabet()==RNA);
 
 	//consensus
-	CHAR* result= new CHAR[num_feat];
-	ASSERT(result);
+	CHAR* result=new CHAR[num_feat];
 
 	//backtracking and scoring table
 	INT num_tables=CMath::max(1,num_feat-degree+1);
-	CDynamicArray<ConsensusEntry>** table = new CDynamicArray<ConsensusEntry>*[num_tables];
-	ASSERT(table);
-	
+	CDynamicArray<ConsensusEntry>** table=new CDynamicArray<ConsensusEntry>*[num_tables];
+
 	for (INT i=0; i<num_tables; i++)
-	{
-		table[i] = new CDynamicArray<ConsensusEntry>(num_suppvec/10);
-		ASSERT(table[i]);
-	}
+		table[i]=new CDynamicArray<ConsensusEntry>(num_suppvec/10);
 
 	//compute consensus via dynamic programming
 	for (INT i=0; i<num_tables; i++)
@@ -1601,19 +1631,19 @@ CHAR* CWeightedDegreePositionStringKernel::compute_consensus(INT &num_feat, INT
 }
 
 
-DREAL* CWeightedDegreePositionStringKernel::extract_w( INT max_degree, INT& num_feat, INT& num_sym, DREAL* result, INT num_suppvec, INT* IDX, DREAL* alphas )
+DREAL* CWeightedDegreePositionStringKernel::extract_w( INT max_degree, INT& num_feat, INT& num_sym, DREAL* w_result, INT num_suppvec, INT* IDX, DREAL* alphas )
 {
   delete_optimization();
   use_poim_tries=true;
   poim_tries.delete_trees(false);
 
   // === check
-  ASSERT( position_weights_lhs == NULL );
-  ASSERT( position_weights_rhs == NULL );
-  num_feat=((CStringFeatures<CHAR>*) get_rhs())->get_max_vector_length();
-  ASSERT( num_feat > 0 );
-  ASSERT( ((CStringFeatures<CHAR>*) get_rhs())->get_alphabet()->get_alphabet() == DNA );
-  ASSERT( max_degree > 0 );
+  ASSERT(position_weights_lhs==NULL);
+  ASSERT(position_weights_rhs==NULL);
+  num_feat=((CStringFeatures<CHAR>*) rhs)->get_max_vector_length();
+  ASSERT(num_feat>0);
+  ASSERT(alphabet->get_alphabet()==DNA);
+  ASSERT(max_degree>0);
 
   // === general variables
   static const INT NUM_SYMS = poim_tries.NUM_SYMS;
@@ -1637,16 +1667,15 @@ DREAL* CWeightedDegreePositionStringKernel::extract_w( INT max_degree, INT& num_
   }
   // --- allocate memory
   const INT bigTabSize = offset;
-  result = new DREAL[ bigTabSize ];
-  ASSERT( result != NULL );
-  for( i = 0; i < bigTabSize; ++i ) {
-    result[i] = 0;
-  }
+  w_result=new DREAL[bigTabSize];
+  for (i=0; i<bigTabSize; ++i)
+    w_result[i]=0;
+
   // --- set pointers for tables
   subs = new DREAL*[ max_degree ];
   ASSERT( subs != NULL );
   for( k = 0; k < max_degree; ++k ) {
-    subs[k] = &result[ offsets[k] ];
+    subs[k] = &w_result[ offsets[k] ];
   }
   delete[] offsets;
 
@@ -1660,23 +1689,23 @@ DREAL* CWeightedDegreePositionStringKernel::extract_w( INT max_degree, INT& num_
   num_sym = bigTabSize;
   use_poim_tries=false;
   poim_tries.delete_trees(false);
-  return result;
+  return w_result;
 }
 
-DREAL* CWeightedDegreePositionStringKernel::compute_POIM( INT max_degree, INT& num_feat, INT& num_sym, DREAL* result, INT num_suppvec, INT* IDX, DREAL* alphas, DREAL* distrib )
+DREAL* CWeightedDegreePositionStringKernel::compute_POIM( INT max_degree, INT& num_feat, INT& num_sym, DREAL* poim_result, INT num_suppvec, INT* IDX, DREAL* alphas, DREAL* distrib )
 {
   delete_optimization();
   use_poim_tries=true;
   poim_tries.delete_trees(false);
 
   // === check
-  ASSERT( position_weights_lhs == NULL );
-  ASSERT( position_weights_rhs == NULL );
-  num_feat=((CStringFeatures<CHAR>*) get_rhs())->get_max_vector_length();
-  ASSERT( num_feat > 0 );
-  ASSERT( ((CStringFeatures<CHAR>*) get_rhs())->get_alphabet()->get_alphabet() == DNA );
-  ASSERT( max_degree != 0 );
-  ASSERT( distrib != NULL );
+  ASSERT(position_weights_lhs==NULL);
+  ASSERT(position_weights_rhs==NULL);
+  num_feat=((CStringFeatures<CHAR>*) rhs)->get_max_vector_length();
+  ASSERT(num_feat>0);
+  ASSERT(alphabet->get_alphabet()==DNA);
+  ASSERT(max_degree!=0);
+  ASSERT(distrib);
 
   // === general variables
   static const INT NUM_SYMS = poim_tries.NUM_SYMS;
@@ -1718,7 +1747,7 @@ DREAL* CWeightedDegreePositionStringKernel::compute_POIM( INT max_degree, INT& n
     }
     default: {
       printf( "POIM DEBUGGING: something is wrong (max order=%d)\n", max_degree );
-      ASSERT( 0 );
+      ASSERT(0);
       break;
     }
     }
@@ -1736,18 +1765,16 @@ DREAL* CWeightedDegreePositionStringKernel::compute_POIM( INT max_degree, INT& n
     offset += tabSize;
   }
   // --- allocate memory
-  const INT bigTabSize = offset;
-  result = new DREAL[ bigTabSize ];
-  ASSERT( result != NULL );
-  for( i = 0; i < bigTabSize; ++i ) {
-    result[i] = 0;
-  }
+  const INT bigTabSize=offset;
+  poim_result=new DREAL[bigTabSize];
+  for (i=0; i<bigTabSize; ++i )
+    poim_result[i]=0;
+
   // --- set pointers for tables
-  subs = new DREAL*[ max_degree ];
-  ASSERT( subs != NULL );
-  for( k = 0; k < max_degree; ++k ) {
-    subs[k] = &result[ offsets[k] ];
-  }
+  subs=new DREAL*[max_degree];
+  for (k=0; k<max_degree; ++k)
+    subs[k]=&poim_result[offsets[k]];
+
   delete[] offsets;
 
   // === init trees; precalc S, L and R
@@ -1794,5 +1821,71 @@ DREAL* CWeightedDegreePositionStringKernel::compute_POIM( INT max_degree, INT& n
   use_poim_tries=false;
   poim_tries.delete_trees(false);
   
-  return result;
+  return poim_result;
 }
+
+
+void CWeightedDegreePositionStringKernel::prepare_POIM2(DREAL* distrib, INT num_sym, INT num_feat)
+{
+	free(m_poim_distrib);
+	m_poim_distrib=(DREAL*)malloc(num_sym*num_feat*sizeof(DREAL));
+	ASSERT(m_poim_distrib);
+
+	memcpy(m_poim_distrib, distrib, num_sym*num_feat*sizeof(DREAL));
+	m_poim_num_sym=num_sym;
+	m_poim_num_feat=num_feat;
+}
+
+void CWeightedDegreePositionStringKernel::compute_POIM2(INT max_degree, CSVM* svm)
+{
+	ASSERT(svm);
+	INT num_suppvec=svm->get_num_support_vectors();
+	INT* sv_idx=new INT[num_suppvec];
+	DREAL* sv_weight=new DREAL[num_suppvec];
+
+	for (INT i=0; i<num_suppvec; i++)
+	{
+		sv_idx[i]=svm->get_support_vector(i);
+		sv_weight[i]=svm->get_alpha(i);
+	}
+	
+	if ((max_degree < 1) || (max_degree > 12))
+	{
+		//SG_WARNING( "max_degree out of range 1..12 (%d).\n", max_degree);
+		SG_WARNING( "max_degree out of range 1..12 (%d). setting to 1.\n", max_degree);
+		max_degree=1;
+	}
+	
+	int num_feat = m_poim_num_feat ;
+	int num_sym = m_poim_num_sym ;
+	free(m_poim) ;
+
+	m_poim = compute_POIM(max_degree, num_feat, num_sym, NULL,	num_suppvec, sv_idx, 
+						  sv_weight, m_poim_distrib);
+
+	ASSERT(num_feat==1);
+	m_poim_result_len=num_sym ;
+	
+	delete[] sv_weight ;
+	delete[] sv_idx ;
+}
+
+void CWeightedDegreePositionStringKernel::get_POIM2(DREAL** poim, INT* result_len)
+{
+	*poim=(DREAL*) malloc(m_poim_result_len*sizeof(DREAL));
+	ASSERT(*poim);
+	memcpy(*poim, m_poim, m_poim_result_len*sizeof(DREAL)) ;
+	*result_len=m_poim_result_len ;
+}
+
+void CWeightedDegreePositionStringKernel::cleanup_POIM2()
+{
+	free(m_poim) ;
+	m_poim=NULL ;
+	free(m_poim_distrib) ;
+	m_poim_distrib=NULL ;
+	m_poim_num_sym=0 ;
+	m_poim_num_sym=0 ;
+	m_poim_result_len=0 ;
+}
+
