@@ -1,12 +1,12 @@
 /*
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
- * Written (W) 1999-2007 Soeren Sonnenburg
- * Written (W) 1999-2007 Gunnar Raetsch
- * Copyright (C) 1999-2007 Fraunhofer Institute FIRST and Max-Planck-Society
+ * Written (W) 1999-2008 Soeren Sonnenburg
+ * Written (W) 1999-2008 Gunnar Raetsch
+ * Copyright (C) 1999-2008 Fraunhofer Institute FIRST and Max-Planck-Society
  */
 
 #include "lib/common.h"
@@ -38,133 +38,136 @@ struct S_THREAD_PARAM
 	INT* vec_idx;
 };
 
-CWeightedDegreeStringKernel::CWeightedDegreeStringKernel(INT size, EWDKernType
-		typ, INT deg, INT max_mismatch_, bool use_norm, bool block, INT
-		mkl_stepsize_, INT which_deg)
-	: CStringKernel<CHAR>(size),weights(NULL),position_weights(NULL),
-	weights_buffer(NULL), mkl_stepsize(mkl_stepsize_),degree(deg), length(0),
-	max_mismatch(max_mismatch_), seq_length(0), initialized(false),
-	block_computation(block), use_normalization(use_norm),
+CWeightedDegreeStringKernel::CWeightedDegreeStringKernel (
+	INT degree_, EWDKernType type_)
+: CStringKernel<CHAR>(10),weights(NULL),position_weights(NULL),
+	weights_buffer(NULL), mkl_stepsize(1),degree(degree_), length(0),
+	max_mismatch(0), seq_length(0), initialized(false),
+	use_normalization(true),block_computation(true),
 	normalization_const(1.0), num_block_weights_external(0),
-	block_weights_external(NULL), block_weights(NULL), type(typ),
-	which_degree(which_deg), tries(deg,max_mismatch_==0),
-	tree_initialized(false)
+	block_weights_external(NULL), block_weights(NULL), type(type_),
+	which_degree(-1), tries(NULL), tree_initialized(false), alphabet(NULL)
 {
 	properties |= KP_LINADD | KP_KERNCOMBINATION | KP_BATCHEVALUATION;
 	lhs=NULL;
 	rhs=NULL;
 
-	// weights will be set using set_wd_weights
-	if (typ != E_EXTERNAL)
+	if (type!=E_EXTERNAL)
 		set_wd_weights_by_type(type);
 }
 
-CWeightedDegreeStringKernel::CWeightedDegreeStringKernel(
-	INT size, double* w, INT d, INT mm, bool un, bool bc, INT mkls, INT wd) 
-	: CStringKernel<CHAR>(size),weights(NULL),position_weights(NULL),weights_buffer(NULL),
-	mkl_stepsize(mkls), degree(d), length(0),
-	max_mismatch(mm), seq_length(0), initialized(false),
-	block_computation(bc), use_normalization(un),
+CWeightedDegreeStringKernel::CWeightedDegreeStringKernel (
+	DREAL *weights_, INT degree_)
+: CStringKernel<CHAR>(10), weights(NULL), position_weights(NULL),
+	weights_buffer(NULL), mkl_stepsize(1), degree(degree_), length(0),
+	max_mismatch(0), seq_length(0), initialized(false),
+	use_normalization(true), block_computation(true),
 	normalization_const(1.0), num_block_weights_external(0),
 	block_weights_external(NULL), block_weights(NULL), type(E_EXTERNAL),
-	which_degree(wd), tries(d, mm==0), tree_initialized(false)
+	which_degree(-1), tries(NULL), tree_initialized(false), alphabet(NULL)
 {
 	properties |= KP_LINADD | KP_KERNCOMBINATION | KP_BATCHEVALUATION;
 	lhs=NULL;
 	rhs=NULL;
 
-	weights=new DREAL[d*(1+max_mismatch)];
-	ASSERT(weights!=NULL);
-	for (INT i=0; i<d*(1+max_mismatch); i++)
-		weights[i]=w[i];
+	weights=new DREAL[degree*(1+max_mismatch)];
+	for (INT i=0; i<degree*(1+max_mismatch); i++)
+		weights[i]=weights_[i];
 }
 
 CWeightedDegreeStringKernel::CWeightedDegreeStringKernel(
-	CStringFeatures<CHAR>* l, CStringFeatures<CHAR>* r, INT d, INT mm)
-	: CStringKernel<CHAR>(10),weights(NULL),position_weights(NULL),weights_buffer(NULL),
-	mkl_stepsize(1), degree(d), length(0),
-	max_mismatch(mm), seq_length(0), initialized(false),
-	block_computation(false), use_normalization(true),
+	CStringFeatures<CHAR>* l, CStringFeatures<CHAR>* r, INT degree_)
+: CStringKernel<CHAR>(10), weights(NULL), position_weights(NULL),
+	weights_buffer(NULL), mkl_stepsize(1), degree(degree_), length(0),
+	max_mismatch(0), seq_length(0), initialized(false),
+	use_normalization(true), block_computation(true),
 	normalization_const(1.0), num_block_weights_external(0),
-	block_weights_external(NULL), block_weights(NULL), type(E_EXTERNAL),
-	which_degree(-1), tries(d, mm==0), tree_initialized(false)
+	block_weights_external(NULL), block_weights(NULL), type(E_WD),
+	which_degree(-1), tries(NULL), tree_initialized(false), alphabet(NULL)
 {
 	properties |= KP_LINADD | KP_KERNCOMBINATION | KP_BATCHEVALUATION;
 
-	set_wd_weights_by_type(E_WD);
-	ASSERT(weights);
-
+	set_wd_weights_by_type(type);
 	init(l, r);
 }
 
-CWeightedDegreeStringKernel::~CWeightedDegreeStringKernel() 
+CWeightedDegreeStringKernel::~CWeightedDegreeStringKernel()
 {
 	cleanup();
 
 	delete[] weights;
 	weights=NULL;
 
-	delete[] position_weights ;
-	position_weights=NULL ;
+	delete[] block_weights;
+	block_weights=NULL;
 
-	delete[] weights_buffer ;
-	weights_buffer = NULL ;
+	delete[] position_weights;
+	position_weights=NULL;
 
+	delete[] weights_buffer;
+	weights_buffer=NULL;
 }
 
 
-void CWeightedDegreeStringKernel::remove_lhs() 
+void CWeightedDegreeStringKernel::remove_lhs()
 { 
 	SG_DEBUG( "deleting CWeightedDegreeStringKernel optimization\n");
 	delete_optimization();
+	initialized = false;
 
-#ifdef SVMLIGHT
-	if (lhs)
-		cache_reset();
-#endif
+	if (tries!=NULL)
+		tries->destroy();
 
-	lhs = NULL ; 
-	rhs = NULL ; 
-	initialized = false ;
-
-	tries.destroy() ;
-}
-
-void CWeightedDegreeStringKernel::remove_rhs()
-{
-#ifdef SVMLIGHT
-	if (rhs)
-		cache_reset() ;
-#endif
-	rhs = lhs ;
+	CKernel::remove_lhs();
 }
 
 void CWeightedDegreeStringKernel::create_empty_tries()
 {
 	seq_length=((CStringFeatures<CHAR>*) lhs)->get_max_vector_length();
 
-	tries.destroy() ;
-	tries.create(seq_length, max_mismatch==0) ;
+	if (tries!=NULL)
+	{
+		tries->destroy() ;
+		tries->create(seq_length, max_mismatch==0) ;
+	}
 }
   
 bool CWeightedDegreeStringKernel::init(CFeatures* l, CFeatures* r)
 {
-	INT lhs_changed = (lhs!=l) ;
-	INT rhs_changed = (rhs!=r) ;
+	INT lhs_changed=(lhs!=l);
+	INT rhs_changed=(rhs!=r);
 
 	bool result=CStringKernel<CHAR>::init(l,r);
-	initialized = false;
+	initialized=false;
 
-	SG_DEBUG( "lhs_changed: %i\n", lhs_changed);
-	SG_DEBUG( "rhs_changed: %i\n", rhs_changed);
+	SG_DEBUG("lhs_changed: %i\n", lhs_changed);
+	SG_DEBUG("rhs_changed: %i\n", rhs_changed);
 
-	ASSERT(((((CStringFeatures<CHAR>*) l)->get_alphabet()->get_alphabet()==DNA) || 
-				 (((CStringFeatures<CHAR>*) l)->get_alphabet()->get_alphabet()==RNA)));
-	ASSERT(((((CStringFeatures<CHAR>*) r)->get_alphabet()->get_alphabet()==DNA) || 
-				 (((CStringFeatures<CHAR>*) r)->get_alphabet()->get_alphabet()==RNA)));
-	
-	if (lhs_changed) 
-		create_empty_tries();
+	CStringFeatures<CHAR>* sf_l=(CStringFeatures<CHAR>*) l;
+	CStringFeatures<CHAR>* sf_r=(CStringFeatures<CHAR>*) r;
+
+	INT len=sf_l->get_max_vector_length();
+	if (lhs_changed && !sf_l->have_same_length(len))
+		SG_ERROR("All strings in WD kernel must have same length (lhs wrong)!\n");
+
+	if (rhs_changed && !sf_r->have_same_length(len))
+		SG_ERROR("All strings in WD kernel must have same length (rhs wrong)!\n");
+
+	delete alphabet;
+	alphabet=new CAlphabet(sf_l->get_alphabet());
+	CAlphabet* ralphabet=(sf_r->get_alphabet());
+	if (!((alphabet->get_alphabet()==DNA) || (alphabet->get_alphabet()==RNA)))
+		properties &= ((ULONG) (-1)) ^ (KP_LINADD | KP_BATCHEVALUATION);
+
+	ASSERT(ralphabet->get_alphabet()==alphabet->get_alphabet());
+	SG_UNREF(ralphabet);
+
+	if (tries!=NULL) {
+		tries->delete_trees(max_mismatch==0);
+		delete tries;
+	}
+	tries=new CTrie<DNATrie>(degree, max_mismatch==0);
+	create_empty_tries();
 
 	init_block_weights();
 
@@ -173,29 +176,33 @@ bool CWeightedDegreeStringKernel::init(CFeatures* l, CFeatures* r)
 	else
 		normalization_const=1.0;
 	
-	this->lhs=(CStringFeatures<CHAR>*) l;
-	this->rhs=(CStringFeatures<CHAR>*) r;
-
-	initialized = true ;
+	initialized=true;
 	return result;
 }
 
 void CWeightedDegreeStringKernel::cleanup()
 {
-	SG_DEBUG( "deleting CWeightedDegreeStringKernel optimization\n");
+	SG_DEBUG("deleting CWeightedDegreeStringKernel optimization\n");
 	delete_optimization();
 
 	delete[] block_weights;
 	block_weights=NULL;
 
-	tries.destroy() ;
-
-	lhs = NULL;
-	rhs = NULL;
+	if (tries!=NULL)
+	{
+		tries->destroy();
+		delete tries;
+		tries=NULL;
+	}
 
 	seq_length=0;
 	initialized=false;
 	tree_initialized = false;
+
+	delete alphabet;
+	alphabet=NULL;
+
+	CKernel::cleanup();
 }
 
 bool CWeightedDegreeStringKernel::load_init(FILE* src)
@@ -213,6 +220,7 @@ bool CWeightedDegreeStringKernel::init_optimization(INT count, INT* IDX, DREAL*
 {
 	if (tree_num<0)
 		SG_DEBUG( "deleting CWeightedDegreeStringKernel optimization\n");
+
 	delete_optimization();
 
 	if (tree_num<0)
@@ -243,7 +251,7 @@ bool CWeightedDegreeStringKernel::init_optimization(INT count, INT* IDX, DREAL*
 	}
 	
 	if (tree_num<0)
-		SG_PRINT( "done.           \n");
+		SG_DONE();
 
 	//tries.compact_nodes(NO_CHILD, 0, weights) ;
 
@@ -251,11 +259,12 @@ bool CWeightedDegreeStringKernel::init_optimization(INT count, INT* IDX, DREAL*
 	return true ;
 }
 
-bool CWeightedDegreeStringKernel::delete_optimization() 
+bool CWeightedDegreeStringKernel::delete_optimization()
 { 
 	if (get_is_initialized())
 	{
-		tries.delete_trees(max_mismatch==0); 
+		if (tries!=NULL)
+			tries->delete_trees(max_mismatch==0); 
 		set_is_initialized(false);
 		return true;
 	}
@@ -293,12 +302,11 @@ DREAL CWeightedDegreeStringKernel::compute_with_mismatch(CHAR* avec, INT alen, C
 
 DREAL CWeightedDegreeStringKernel::compute_using_block(CHAR* avec, INT alen, CHAR* bvec, INT blen)
 {
-	DREAL sum=0;
-
-	INT match_len=-1;
-
 	ASSERT(alen==blen);
 
+	DREAL sum=0;
+	INT match_len=-1;
+
 	for (INT i=0; i<alen; i++)
 	{
 		if (avec[i]==bvec[i])
@@ -364,64 +372,64 @@ DREAL CWeightedDegreeStringKernel::compute_without_mismatch_matrix(CHAR* avec, I
 
 DREAL CWeightedDegreeStringKernel::compute(INT idx_a, INT idx_b)
 {
-  INT alen, blen;
+	INT alen, blen;
+	CHAR* avec=((CStringFeatures<CHAR>*) lhs)->get_feature_vector(idx_a, alen);
+	CHAR* bvec=((CStringFeatures<CHAR>*) rhs)->get_feature_vector(idx_b, blen);
+	DREAL result=0;
 
-  CHAR* avec=((CStringFeatures<CHAR>*) lhs)->get_feature_vector(idx_a, alen);
-  CHAR* bvec=((CStringFeatures<CHAR>*) rhs)->get_feature_vector(idx_b, blen);
+	if (max_mismatch==0 && length==0 && block_computation)
+		result=compute_using_block(avec, alen, bvec, blen);
+	else
+	{
+		if (max_mismatch>0)
+			result=compute_with_mismatch(avec, alen, bvec, blen);
+		else if (length==0)
+			result=compute_without_mismatch(avec, alen, bvec, blen);
+		else
+			result=compute_without_mismatch_matrix(avec, alen, bvec, blen);
+	}
 
-  // can only deal with strings of same length
-  ASSERT(alen==blen);
-
-  DREAL result=0;
-
-  if (max_mismatch == 0 && length == 0 && block_computation)
-	  result = compute_using_block(avec, alen, bvec, blen) ;
-  else
-  {
-	  if (max_mismatch > 0)
-		  result = compute_with_mismatch(avec, alen, bvec, blen) ;
-	  else if (length==0)
-		  result = compute_without_mismatch(avec, alen, bvec, blen) ;
-	  else
-		  result = compute_without_mismatch_matrix(avec, alen, bvec, blen) ;
-  }
-  
-  return result/normalization_const;
+	return result/normalization_const;
 }
 
 
-void CWeightedDegreeStringKernel::add_example_to_tree(INT idx, DREAL alpha) 
+void CWeightedDegreeStringKernel::add_example_to_tree(INT idx, DREAL alpha)
 {
-	INT len ;
+	ASSERT(alphabet);
+	ASSERT(alphabet->get_alphabet()==DNA || alphabet->get_alphabet()==RNA);
+
+	INT len=0;
 	CHAR* char_vec=((CStringFeatures<CHAR>*) lhs)->get_feature_vector(idx, len);
-	ASSERT(max_mismatch==0) ;
-	INT *vec = new INT[len] ;
+	ASSERT(max_mismatch==0);
+	INT *vec=new INT[len];
 
 	for (INT i=0; i<len; i++)
-		vec[i]=((CStringFeatures<CHAR>*) lhs)->get_alphabet()->remap_to_bin(char_vec[i]);
+		vec[i]=alphabet->remap_to_bin(char_vec[i]);
 	
 	if (length == 0 || max_mismatch > 0)
 	{
 		for (INT i=0; i<len; i++)
 		{
-			DREAL alpha_pw = alpha ;
+			DREAL alpha_pw=alpha;
 			/*if (position_weights!=NULL)
 			  alpha_pw *= position_weights[i] ;*/
 			if (alpha_pw==0.0)
-				continue ;
-			tries.add_to_trie(i, 0, vec, alpha_pw, weights, (length!=0)) ;
+				continue;
+			ASSERT(tries);
+			tries->add_to_trie(i, 0, vec, alpha_pw, weights, (length!=0));
 		}
 	}
 	else
 	{
 		for (INT i=0; i<len; i++)
 		{
-			DREAL alpha_pw = alpha ;
+			DREAL alpha_pw=alpha;
 			/*if (position_weights!=NULL) 
 			  alpha_pw = alpha*position_weights[i] ;*/
 			if (alpha_pw==0.0)
 				continue ;
-			tries.add_to_trie(i, 0, vec, alpha_pw, weights, (length!=0)) ;		
+			ASSERT(tries);
+			tries->add_to_trie(i, 0, vec, alpha_pw, weights, (length!=0));
 		}
 	}
 	delete[] vec ;
@@ -430,120 +438,120 @@ void CWeightedDegreeStringKernel::add_example_to_tree(INT idx, DREAL alpha)
 
 void CWeightedDegreeStringKernel::add_example_to_single_tree(INT idx, DREAL alpha, INT tree_num) 
 {
+	ASSERT(alphabet);
+	ASSERT(alphabet->get_alphabet()==DNA || alphabet->get_alphabet()==RNA);
+
 	INT len ;
 	CHAR* char_vec=((CStringFeatures<CHAR>*) lhs)->get_feature_vector(idx, len);
-	ASSERT(max_mismatch==0) ;
+	ASSERT(max_mismatch==0);
 	INT *vec = new INT[len] ;
 
 	for (INT i=tree_num; i<tree_num+degree && i<len; i++)
-		vec[i]=((CStringFeatures<CHAR>*) lhs)->get_alphabet()->remap_to_bin(char_vec[i]);
+		vec[i]=alphabet->remap_to_bin(char_vec[i]);
 	
-	if (length == 0 || max_mismatch > 0)
-	{
-		DREAL alpha_pw = alpha ;
-		/*if (position_weights!=NULL)
-		  alpha_pw = alpha*position_weights[tree_num] ;*/
-		if (alpha_pw!=0.0)
-			tries.add_to_trie(tree_num, 0, vec, alpha_pw, weights, (length!=0)) ;
-	}
-	else
-	{
-		DREAL alpha_pw = alpha ;
-		/*if (position_weights!=NULL) 
-		  alpha_pw = alpha*position_weights[tree_num] ;*/
-		if (alpha_pw!=0.0)
-			tries.add_to_trie(tree_num, 0, vec, alpha_pw, weights, (length!=0)) ;
-	}
+	ASSERT(tries);
+	if (alpha!=0.0)
+		tries->add_to_trie(tree_num, 0, vec, alpha, weights, (length!=0));
+
 	delete[] vec ;
 	tree_initialized=true ;
 }
 
-void CWeightedDegreeStringKernel::add_example_to_tree_mismatch(INT idx, DREAL alpha) 
+void CWeightedDegreeStringKernel::add_example_to_tree_mismatch(INT idx, DREAL alpha)
 {
+	ASSERT(tries);
+	ASSERT(alphabet);
+	ASSERT(alphabet->get_alphabet()==DNA || alphabet->get_alphabet()==RNA);
+
 	INT len ;
 	CHAR* char_vec=((CStringFeatures<CHAR>*) lhs)->get_feature_vector(idx, len);
 	
 	INT *vec = new INT[len] ;
 	
 	for (INT i=0; i<len; i++)
-		vec[i]=((CStringFeatures<CHAR>*) lhs)->get_alphabet()->remap_to_bin(char_vec[i]);
-	
-	/*for (INT q=0; q<40; q++)
-	  fprintf(stderr, "w[%i]=%f\n", q,weights[q]) ;*/
+		vec[i]=alphabet->remap_to_bin(char_vec[i]);
 	
 	for (INT i=0; i<len; i++)
 	{
-		DREAL alpha_pw = alpha ;
-		/*if (position_weights!=NULL)
-		  alpha_pw = alpha*position_weights[i] ;*/
-		if (alpha_pw==0.0)
-			continue ;
-		tries.add_example_to_tree_mismatch_recursion(NO_CHILD, i, alpha_pw, &vec[i], len-i, 0, 0, max_mismatch, weights) ;
+		if (alpha!=0.0)
+			tries->add_example_to_tree_mismatch_recursion(NO_CHILD, i, alpha, &vec[i], len-i, 0, 0, max_mismatch, weights);
 	}
 	
 	delete[] vec ;
 	tree_initialized=true ;
 }
 
-void CWeightedDegreeStringKernel::add_example_to_single_tree_mismatch(INT idx, DREAL alpha, INT tree_num) 
+void CWeightedDegreeStringKernel::add_example_to_single_tree_mismatch(INT idx, DREAL alpha, INT tree_num)
 {
-	INT len ;
-	CHAR* char_vec=((CStringFeatures<CHAR>*) lhs)->get_feature_vector(idx, len);
+	ASSERT(tries);
+	ASSERT(alphabet);
+	ASSERT(alphabet->get_alphabet()==DNA || alphabet->get_alphabet()==RNA);
 
-	INT *vec = new INT[len] ;
+	INT len=0;
+	CHAR* char_vec=((CStringFeatures<CHAR>*) lhs)->get_feature_vector(idx, len);
+	INT *vec=new INT[len];
 
 	for (INT i=tree_num; i<len && i<tree_num+degree; i++)
-		vec[i]=((CStringFeatures<CHAR>*) lhs)->get_alphabet()->remap_to_bin(char_vec[i]);
+		vec[i]=alphabet->remap_to_bin(char_vec[i]);
 
-	DREAL alpha_pw = alpha ;
-	/*if (position_weights!=NULL)
-	  alpha_pw = alpha*position_weights[tree_num] ;*/
-	if (alpha_pw!=0.0)
-		tries.add_example_to_tree_mismatch_recursion(NO_CHILD, tree_num, alpha_pw, &vec[tree_num], len-tree_num, 0, 0, max_mismatch, weights) ;
+	if (alpha!=0.0)
+	{
+		tries->add_example_to_tree_mismatch_recursion(
+			NO_CHILD, tree_num, alpha, &vec[tree_num], len-tree_num,
+			0, 0, max_mismatch, weights);
+	}
 
-	delete[] vec ;
-	tree_initialized=true ;
+	delete[] vec;
+	tree_initialized=true;
 }
 
 
-DREAL CWeightedDegreeStringKernel::compute_by_tree(INT idx) 
+DREAL CWeightedDegreeStringKernel::compute_by_tree(INT idx)
 {
-	INT len ;
+	ASSERT(alphabet);
+	ASSERT(alphabet->get_alphabet()==DNA || alphabet->get_alphabet()==RNA);
+
+	INT len=0;
 	CHAR* char_vec=((CStringFeatures<CHAR>*) rhs)->get_feature_vector(idx, len);
 	ASSERT(char_vec && len>0);
-	INT *vec = new INT[len] ;
-	
-	for (INT i=0; i<len; i++)
-		vec[i]=((CStringFeatures<CHAR>*) lhs)->get_alphabet()->remap_to_bin(char_vec[i]);
+	INT *vec=new INT[len];
 
-	DREAL sum=0 ;
 	for (INT i=0; i<len; i++)
-		sum += tries.compute_by_tree_helper(vec, len, i, i, i, weights, (length!=0));
-	
-	delete[] vec ;
-	
+		vec[i]=alphabet->remap_to_bin(char_vec[i]);
+
+	DREAL sum=0;
+	ASSERT(tries);
+	for (INT i=0; i<len; i++)
+		sum+=tries->compute_by_tree_helper(vec, len, i, i, i, weights, (length!=0));
+
+	delete[] vec;
 	return sum/normalization_const;
 }
 
-void CWeightedDegreeStringKernel::compute_by_tree(INT idx, DREAL* LevelContrib) 
+void CWeightedDegreeStringKernel::compute_by_tree(INT idx, DREAL* LevelContrib)
 {
+	ASSERT(alphabet);
+	ASSERT(alphabet->get_alphabet()==DNA || alphabet->get_alphabet()==RNA);
+
 	INT len ;
 	CHAR* char_vec=((CStringFeatures<CHAR>*) rhs)->get_feature_vector(idx, len);
 
 	INT *vec = new INT[len] ;
 
 	for (INT i=0; i<len; i++)
-		vec[i]=((CStringFeatures<CHAR>*) lhs)->get_alphabet()->remap_to_bin(char_vec[i]);
+		vec[i]=alphabet->remap_to_bin(char_vec[i]);
 
+	ASSERT(tries);
 	for (INT i=0; i<len; i++)
-	  tries.compute_by_tree_helper(vec, len, i, i, i, LevelContrib, 1.0/normalization_const, mkl_stepsize, weights, (length!=0));
+	  tries->compute_by_tree_helper(vec, len, i, i, i, LevelContrib, 1.0/normalization_const, mkl_stepsize, weights, (length!=0));
 
 	delete[] vec ;
 }
 
-DREAL *CWeightedDegreeStringKernel::compute_abs_weights(int &len) 
+DREAL *CWeightedDegreeStringKernel::compute_abs_weights(int &len)
 {
-	return tries.compute_abs_weights(len) ;
+	ASSERT(tries);
+	return tries->compute_abs_weights(len);
 }
 
 bool CWeightedDegreeStringKernel::set_wd_weights_by_type(EWDKernType p_type)
@@ -598,21 +606,22 @@ bool CWeightedDegreeStringKernel::set_wd_weights_by_type(EWDKernType p_type)
 
 bool CWeightedDegreeStringKernel::set_weights(DREAL* ws, INT d, INT len)
 {
-	SG_DEBUG( "degree = %i  d=%i\n", degree, d) ;
-	degree = d ;
-	tries.set_degree(d);
+	SG_DEBUG("degree = %i  d=%i\n", degree, d);
+	degree=d;
+	ASSERT(tries);
+	tries->set_degree(degree);
 	length=len;
 	
-	if (len == 0)
-		len=1;
-	
+	if (length==0) length=1;
+	INT num_weights=degree*(length+max_mismatch);
 	delete[] weights;
-	weights=new DREAL[d*len];
-	
+	weights=new DREAL[num_weights];
 	if (weights)
 	{
-		for (int i=0; i<degree*len; i++)
-			weights[i]=ws[i];
+		for (INT i=0; i<num_weights; i++) {
+			if (ws[i]) // len(ws) might be != num_weights?
+				weights[i]=ws[i];
+		}
 		return true;
 	}
 	else
@@ -623,19 +632,19 @@ bool CWeightedDegreeStringKernel::set_position_weights(DREAL* pws, INT len)
 {
 	if (len==0)
 	{
-		delete[] position_weights ;
-		position_weights = NULL ;
-		tries.set_position_weights(position_weights) ;
-	}
-	
-    if (seq_length!=len) 
-	{
-      SG_ERROR( "seq_length = %i, position_weights_length=%i\n", seq_length, len) ;
-		return false ;
+		delete[] position_weights;
+		position_weights=NULL;
+		ASSERT(tries);
+		tries->set_position_weights(position_weights);
 	}
+
+	if (seq_length!=len)
+		SG_ERROR("seq_length = %i, position_weights_length=%i\n", seq_length, len);
+
 	delete[] position_weights;
 	position_weights=new DREAL[len];
-	tries.set_position_weights(position_weights) ;
+	ASSERT(tries);
+	tries->set_position_weights(position_weights);
 	
 	if (position_weights)
 	{
@@ -849,39 +858,52 @@ void* CWeightedDegreeStringKernel::compute_batch_helper(void* p)
 	DREAL factor=params->factor;
 	INT* vec_idx=params->vec_idx;
 
+	CStringFeatures<CHAR>* rhs_feat=((CStringFeatures<CHAR>*) wd->get_rhs());
+	CAlphabet* alpha=wd->alphabet;
+
 	for (INT i=params->start; i<params->end; i++)
 	{
 		INT len=0;
-		CHAR* char_vec=((CStringFeatures<CHAR>*) wd->get_rhs())->get_feature_vector(vec_idx[i], len);
+		CHAR* char_vec=rhs_feat->get_feature_vector(vec_idx[i], len);
 		for (INT k=j; k<CMath::min(len,j+wd->get_degree()); k++)
-			vec[k]=((CStringFeatures<CHAR>*) wd->get_lhs())->get_alphabet()->remap_to_bin(char_vec[k]);
+			vec[k]=alpha->remap_to_bin(char_vec[k]);
 
-		result[i] += factor*tries->compute_by_tree_helper(vec, len, j, j, j, weights, (length!=0))/wd->get_normalization_const();
+		ASSERT(tries);
+
+		result[i]+=factor*
+			tries->compute_by_tree_helper(vec, len, j, j, j, weights, (length!=0))/
+			wd->get_normalization_const();
 	}
 
+	SG_UNREF(rhs_feat);
+
 	return NULL;
 }
 
 void CWeightedDegreeStringKernel::compute_batch(INT num_vec, INT* vec_idx, DREAL* result, INT num_suppvec, INT* IDX, DREAL* alphas, DREAL factor)
 {
-	ASSERT(get_rhs());
-    ASSERT(num_vec<=get_rhs()->get_num_vectors());
+	ASSERT(tries);
+	ASSERT(alphabet);
+	ASSERT(alphabet->get_alphabet()==DNA || alphabet->get_alphabet()==RNA);
+	ASSERT(rhs);
+	ASSERT(num_vec<=rhs->get_num_vectors());
 	ASSERT(num_vec>0);
 	ASSERT(vec_idx);
 	ASSERT(result);
+	create_empty_tries();
 
-	INT num_feat=((CStringFeatures<CHAR>*) get_rhs())->get_max_vector_length();
+	INT num_feat=((CStringFeatures<CHAR>*) rhs)->get_max_vector_length();
 	ASSERT(num_feat>0);
 	INT num_threads=parallel.get_num_threads();
 	ASSERT(num_threads>0);
-	INT* vec= new INT[num_threads*num_feat];
-	ASSERT(vec);
+	INT* vec=new INT[num_threads*num_feat];
 
 	if (num_threads < 2)
 	{
 #ifdef CYGWIN
 		for (INT j=0; j<num_feat; j++)
 #else
+        CSignal::clear() ;
 		for (INT j=0; j<num_feat && !CSignal::cancel_computations(); j++)
 #endif
 		{
@@ -891,7 +913,7 @@ void CWeightedDegreeStringKernel::compute_batch(INT num_vec, INT* vec_idx, DREAL
 			params.result=result;
 			params.weights=weights;
 			params.kernel=this;
-			params.tries=&tries;
+			params.tries=tries;
 			params.factor=factor;
 			params.j=j;
 			params.start=0;
@@ -906,6 +928,7 @@ void CWeightedDegreeStringKernel::compute_batch(INT num_vec, INT* vec_idx, DREAL
 #ifndef WIN32
 	else
 	{
+        CSignal::clear() ;
 		for (INT j=0; j<num_feat && !CSignal::cancel_computations(); j++)
 		{
 			init_optimization(num_suppvec, IDX, alphas, j);
@@ -920,7 +943,7 @@ void CWeightedDegreeStringKernel::compute_batch(INT num_vec, INT* vec_idx, DREAL
 				params[t].result=result;
 				params[t].weights=weights;
 				params[t].kernel=this;
-				params[t].tries=&tries;
+				params[t].tries=tries;
 				params[t].factor=factor;
 				params[t].j=j;
 				params[t].start = t*step;
@@ -933,7 +956,7 @@ void CWeightedDegreeStringKernel::compute_batch(INT num_vec, INT* vec_idx, DREAL
 			params[t].result=result;
 			params[t].weights=weights;
 			params[t].kernel=this;
-			params[t].tries=&tries;
+			params[t].tries=tries;
 			params[t].factor=factor;
 			params[t].j=j;
 			params[t].start=t*step;
@@ -955,3 +978,18 @@ void CWeightedDegreeStringKernel::compute_batch(INT num_vec, INT* vec_idx, DREAL
 	//using the combined kernel
 	create_empty_tries();
 }
+
+bool CWeightedDegreeStringKernel::set_max_mismatch(INT max)
+{
+	if (type==E_EXTERNAL && max!=0) {
+		return false;
+	}
+
+	max_mismatch=max;
+
+	if (lhs!=NULL && rhs!=NULL)
+		return init(lhs, rhs);
+	else
+		return true;
+}
+
