@@ -1,12 +1,12 @@
 /*
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
- * Written (W) 1999-2007 Soeren Sonnenburg
- * Written (W) 1999-2007 Gunnar Raetsch
- * Copyright (C) 1999-2007 Fraunhofer Institute FIRST and Max-Planck-Society
+ * Written (W) 1999-2008 Soeren Sonnenburg
+ * Written (W) 1999-2008 Gunnar Raetsch
+ * Copyright (C) 1999-2008 Fraunhofer Institute FIRST and Max-Planck-Society
  */
 
 // Math.cpp: implementation of the CMath class.
@@ -19,31 +19,10 @@
 #include "lib/lapack.h"
 #include "lib/io.h"
 
-#include <sys/time.h>
-#include <sys/types.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <math.h>
-#include <time.h>
-#include <unistd.h>
 
-//#if defined(HAVE_LAPACK) && defined(DARWIN)
-//int clapack_dpotrf(const enum CBLAS_ORDER Order, const enum CBLAS_UPLO Uplo,
-//			                   const int N, double *A, const int LDA)
-//{
-//	ASSERT(Order==CblasColMajor);
-//        //call dgemm ( 'T', 'T', alpha, B, ldb, A, lda, beta, C, ldc )
-//	char uplo = 'U';
-//	__CLPK_integer info=0;
-//	__CLPK_integer n=N;
-//	__CLPK_integer lda=LDA;
-//
-//	if (Uplo==CblasLower)
-//		uplo='L';
-//	dpotrf_(&uplo, &n, A, &lda, &info);
-//	return info;
-//}
-//#endif
 //////////////////////////////////////////////////////////////////////
 // Construction/Destruction
 //////////////////////////////////////////////////////////////////////
@@ -66,21 +45,20 @@ INT CMath::LOGRANGE            = 0; // range for logtable: log(1+exp(x))  -25 <=
 const DREAL CMath::INFTY            =  -log(0.0);	// infinity
 const DREAL CMath::ALMOST_INFTY		=  +1e+20;		//a large number
 const DREAL CMath::ALMOST_NEG_INFTY =  -1000;	
-
-CHAR CMath::rand_state[256];
+#ifdef USE_LOGCACHE
+DREAL* CMath::logtable = NULL;
+#endif
+CHAR* CMath::rand_state = NULL;
 UINT CMath::seed = 0;
 
-CMath::CMath() : CSGObject()
+CMath::CMath()
+: CSGObject()
 {
 #ifndef HAVE_SWIG
 	CSGObject::version.print_version();
 #endif
-	struct timeval tv;
-	gettimeofday(&tv, NULL);
-	seed=(UINT) (4223517*getpid()*tv.tv_sec*tv.tv_usec);
-#ifndef CYGWIN
-	initstate(seed, CMath::rand_state, sizeof(CMath::rand_state));
-#endif
+	CMath::rand_state=new CHAR[256];
+	init_random();
 #ifndef HAVE_SWIG
 	SG_PRINT( "( seeding random number generator with %u, ", seed);
 #endif
@@ -107,8 +85,11 @@ CMath::CMath() : CSGObject()
 
 CMath::~CMath()
 {
+	delete[] CMath::rand_state;
+	CMath::rand_state=NULL;
 #ifdef USE_LOGCACHE
-	delete[] logtable;
+	delete[] CMath::logtable;
+	CMath::logtable=NULL;
 #endif
 }
 
@@ -439,9 +420,7 @@ double CMath::entropy(DREAL* p, INT len)
 DREAL* CMath::pinv(DREAL* matrix, INT rows, INT cols, DREAL* target)
 {
 	if (!target)
-		target= new DREAL[rows*cols];
-
-	ASSERT(target);
+		target=new DREAL[rows*cols];
 
 	char jobu='A';
 	char jobvt='A';
@@ -453,22 +432,11 @@ DREAL* CMath::pinv(DREAL* matrix, INT rows, INT cols, DREAL* target)
 	int info=-1;
 	int lsize=CMath::min(m,n);
 	double* s=new double[lsize];
-	ASSERT(s);
 	double* u=new double[m*m];
-	ASSERT(u);
 	double* vt=new double[n*n];
-	ASSERT(vt);
-	int lwork=-1;
-	double work1;
 
-	dgesvd_(&jobu, &jobvt, &m, &n, matrix, &lda, s, u, &ldu, vt, &ldvt, &work1, &lwork, &info);
-	ASSERT(info == 0);
-	ASSERT(work1>0);
-	lwork=(int) work1;
-	double* work=new double[lwork];
-	ASSERT(work);
-	dgesvd_(&jobu, &jobvt, &m, &n, matrix, &lda, s, u, &ldu, vt, &ldvt, work, &lwork, &info);
-	ASSERT(info == 0);
+	wrap_dgesvd(jobu, jobvt, m, n, matrix, lda, s, u, ldu, vt, ldvt, &info);
+	ASSERT(info==0);
 
 	for (INT i=0; i<n; i++)
 	{
@@ -480,13 +448,22 @@ DREAL* CMath::pinv(DREAL* matrix, INT rows, INT cols, DREAL* target)
 
 	delete[] u;
 	delete[] vt;
-	delete[] work;
 	delete[] s;
 
 	return target;
 }
 #endif
 
+template <>
+void CMath::display_vector(BYTE* vector, INT n, const char* name)
+{
+	ASSERT(n>=0);
+	SG_SPRINT("%s=[", name);
+	for (INT i=0; i<n; i++)
+		SG_SPRINT("%d%s", vector[i], i==n-1? "" : ",");
+	SG_SPRINT("]\n");
+}
+
 template <>
 void CMath::display_vector(INT* vector, INT n, const char* name)
 {
@@ -507,6 +484,16 @@ void CMath::display_vector(LONG* vector, INT n, const char* name)
 	SG_SPRINT("]\n");
 }
 
+template <>
+void CMath::display_vector(SHORTREAL* vector, INT n, const char* name)
+{
+	ASSERT(n>=0);
+	SG_SPRINT("%s=[", name);
+	for (INT i=0; i<n; i++)
+		SG_SPRINT("%10.10f%s", vector[i], i==n-1? "" : ",");
+	SG_SPRINT("]\n");
+}
+
 template <>
 void CMath::display_vector(DREAL* vector, INT n, const char* name)
 {
