@@ -1,11 +1,11 @@
 /*
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
- * Written (W) 1999-2007 Soeren Sonnenburg
- * Copyright (C) 1999-2007 Fraunhofer Institute FIRST and Max-Planck-Society
+ * Written (W) 1999-2008 Soeren Sonnenburg
+ * Copyright (C) 1999-2008 Fraunhofer Institute FIRST and Max-Planck-Society
  */
 
 #include "lib/config.h"
@@ -19,7 +19,10 @@
 
 #include "lib/io.h"
 #include "lib/Signal.h"
-
+#include "lib/matlab.h"
+#include "lib/octave.h"
+#include "lib/python.h"
+#include "lib/r.h"
 
 int CSignal::signals[NUMTRAPPEDSIGS]={SIGINT, SIGURG};
 struct sigaction CSignal::oldsigaction[NUMTRAPPEDSIGS];
@@ -27,6 +30,7 @@ bool CSignal::active=false;
 bool CSignal::cancel_computation=false;
 
 CSignal::CSignal()
+: CSGObject()
 {
 }
 
@@ -38,7 +42,9 @@ CSignal::~CSignal()
 
 void CSignal::handler(int signal)
 {
-#if defined(HAVE_MATLAB) || defined(HAVE_OCTAVE) || defined(HAVE_R)
+	unset_handler();
+
+#if !defined(HAVE_SWIG) && (defined(HAVE_MATLAB) || defined(HAVE_OCTAVE) || defined(HAVE_R))
 	if (signal == SIGINT)
 	{
 		SG_SPRINT("\nImmediately return to matlab prompt / Prematurely finish computations / Do nothing (I/P/D)? ");
@@ -46,13 +52,31 @@ void CSignal::handler(int signal)
 
 		if (answer == 'I')
 		{
-			unset_handler();
-			SG_SERROR("sg stopped by SIGINT\n");
+#if defined(HAVE_MATLAB)
+			mexErrMsgTxt("sg stopped by SIGINT\n");
+#elif defined(HAVE_OCTAVE)
+			SG_PRINT("sg stopped by SIGINT\n");
+			BEGIN_INTERRUPT_IMMEDIATELY_IN_FOREIGN_CODE;
+			octave_throw_interrupt_exception();
+			END_INTERRUPT_IMMEDIATELY_IN_FOREIGN_CODE;
+#elif defined(HAVE_PYTHON)
+			SG_PRINT("sg stopped by SIGINT\n");
+			PyErr_SetInterrupt();
+			PyErr_CheckSignals();
+#elif defined(HAVE_R)
+			R_Suicide((char*) "sg stopped by SIGINT\n");
+#endif
+
 		}
-		else if (answer == 'P')
-			cancel_computation=true;
 		else
-			SG_SPRINT("\n");
+		{
+			set_handler();
+
+			if (answer == 'P')
+				cancel_computation=true;
+			else
+				SG_SPRINT("\n");
+		}
 	}
 	else if (signal == SIGURG)
 		cancel_computation=true;
