@@ -1,18 +1,19 @@
 /*
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
- * Written (W) 1999-2007 Soeren Sonnenburg
- * Copyright (C) 1999-2007 Fraunhofer Institute FIRST and Max-Planck-Society
+ * Written (W) 1999-2008 Soeren Sonnenburg
+ * Copyright (C) 1999-2008 Fraunhofer Institute FIRST and Max-Planck-Society
  */
 
 #include "lib/Time.h"
 #include "lib/io.h"
 #include "lib/common.h"
 
-CTime::CTime(bool st) : CSGObject()
+CTime::CTime(bool st)
+: CSGObject()
 {
 	start_time=0;
 	stop_time=0;
