@@ -1,15 +1,15 @@
 /*
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
- * Written (W) 1999-2007 Soeren Sonnenburg
- * Written (W) 1999-2007 Gunnar Raetsch
+ * Written (W) 1999-2008 Soeren Sonnenburg
+ * Written (W) 1999-2008 Gunnar Raetsch
  * Written (W) 2006 Fabio De Bona
  * Written (W) 2006 Konrad Rieck
  * Written (W) 2006-2007 Christian Gehl
- * Copyright (C) 1999-2007 Fraunhofer Institute FIRST and Max-Planck-Society
+ * Copyright (C) 1999-2008 Fraunhofer Institute FIRST and Max-Planck-Society
  */
 
 #ifndef __COMMON_H__
@@ -18,6 +18,7 @@
 #include <stdlib.h> 
 #include <stdio.h> 
 #include "lib/config.h"
+#include "lib/memory.h"
 
 #ifdef SUNOS
 #define bool int
@@ -172,12 +173,10 @@ enum EKernelType
 	K_CHI2 = 180,
 	K_DIAG = 190,
 	K_CONST = 200,
-	K_HAMMINGWORD = 210,
-	K_MANHATTANWORD = 220,
-	K_CANBERRAWORD = 230,
-	K_MINDYGRAM = 240,
-	K_DISTANCE = 250,
-	K_LOCALALIGNMENT= 260,
+	K_MINDYGRAM = 210,
+	K_DISTANCE = 220,
+	K_LOCALALIGNMENT= 230,
+	K_PYRAMIDCHI2= 240,
 };
 
 enum EClassifierType
@@ -209,6 +208,8 @@ enum EClassifierType
 	CT_KMEANS = 240,
 	CT_HIERARCHICAL = 250,
 	CT_SVMOCAS = 260,
+	CT_WDSVMOCAS = 270,
+	CT_SVMSGD = 280,
 };
 
 enum EDistanceType
@@ -223,8 +224,8 @@ enum EDistanceType
 	D_MANHATTANWORD = 70,
 	D_HAMMINGWORD = 80 ,
 	D_CANBERRAWORD = 90,
-	D_SPARSENORMSQUARED = 100,
-	D_NORMSQUARED = 110
+	D_SPARSEEUCLIDIAN = 100,
+	D_EUCLIDIAN = 110
 };
 
 enum ERegressionType
@@ -234,6 +235,17 @@ enum ERegressionType
 	RT_LIBSVM = 20
 };
 
+enum EPreProcType
+{
+	P_UNKNOWN=0,
+	P_NORMONE=10,
+	P_LOGPLUSONE=20,
+	P_SORTWORDSTRING=30,
+	P_SORTULONGSTRING=40,
+	P_SORTWORD=50,
+	P_PRUNEVARSUBMEAN=60
+};
+
 enum EKernelProperty
 {
 	KP_NONE = 0,
@@ -253,9 +265,9 @@ enum EFeatureType
 	F_UINT = 60,
 	F_LONG = 70,
 	F_ULONG = 80,
-	F_SREAL = 90,
+	F_SHORTREAL = 90,
 	F_DREAL = 100,
-	F_LREAL = 110,
+	F_LONGREAL = 110,
 	F_ANY = 1000
 };
 
@@ -276,29 +288,35 @@ enum E_ALPHABET
 	/// DNA - letters A,C,G,T,*,N,n
 	DNA=0,
 
-	/// DNA - letters A,C,G,U,*,N,n
-	RNA=1,
+	/// RAWDNA - letters 0,1,2,3
+	RAWDNA=1,
+
+	/// RNA - letters A,C,G,U,*,N,n
+	RNA=2,
 
 	/// PROTEIN - letters a-z
-	PROTEIN=2,
+	PROTEIN=3,
 
 	/// ALPHANUM - [0-9a-z]
-	ALPHANUM=3,
+	ALPHANUM=5,
 
 	/// CUBE - [1-6]
-	CUBE=4,
+	CUBE=6,
 
 	/// RAW BYTE - [0-255]
-	RAWBYTE=5,
+	RAWBYTE=7,
 
 	/// IUPAC_NUCLEIC_ACID
-	IUPAC_NUCLEIC_ACID=6,
+	IUPAC_NUCLEIC_ACID=8,
 
 	/// IUPAC_AMINO_ACID
-	IUPAC_AMINO_ACID=7,
+	IUPAC_AMINO_ACID=9,
 
 	/// NONE - type has no alphabet
-	NONE=8
+	NONE=10,
+
+	/// unknown alphabet
+	UNKNOWN=11,
 };
 
 //@}
