@@ -1,31 +1,20 @@
 /*
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
- * Written (W) 1999-2007 Soeren Sonnenburg
- * Written (W) 1999-2007 Gunnar Raetsch
- * Copyright (C) 1999-2007 Fraunhofer Institute FIRST and Max-Planck-Society
+ * Written (W) 1999-2008 Soeren Sonnenburg
+ * Written (W) 1999-2008 Gunnar Raetsch
+ * Copyright (C) 1999-2008 Fraunhofer Institute FIRST and Max-Planck-Society
  */
 
 #include "lib/config.h"
-
-#ifdef HAVE_OCTAVE
-#include <octave/config.h>
-#include <octave/error.h>
-#include <octave/lo-error.h>
-#endif
-
 #include "lib/matlab.h"
-
-#ifdef HAVE_R
-#include <R.h>
-#include <Rinternals.h>
-#include <R_ext/RS.h>
-#endif
-
+#include "lib/octave.h"
+#include "lib/r.h"
 #include "lib/python.h"
+
 #include "lib/io.h"
 #include "lib/ShogunException.h"
 #include "lib/Signal.h"
@@ -52,186 +41,126 @@ CHAR file_buffer[FBUFSIZE];
 /// directory name buffer
 CHAR directory_name[FBUFSIZE];
 
-CIO::CIO() : target(stdout), last_progress_time(0), progress_start_time(0), last_progress(1), loglevel(M_WARN)
+CIO::CIO() : target(stdout), last_progress_time(0), progress_start_time(0),
+	last_progress(1), show_progress(false), loglevel(M_WARN)
 {
 }
 
-CIO::CIO(const CIO& orig) : target(orig.get_target()), last_progress_time(0), progress_start_time(0), last_progress(1), loglevel(orig.get_loglevel())
+CIO::CIO(const CIO& orig) : target(orig.get_target()), last_progress_time(0),
+	progress_start_time(0), last_progress(1), show_progress(orig.get_show_progress()),
+	loglevel(orig.get_loglevel())
 {
 }
 
 void CIO::message(EMessageType prio, const CHAR *fmt, ... ) const
 {
-#if defined(HAVE_MATLAB) || defined(HAVE_PYTHON) || defined(HAVE_OCTAVE) || defined(HAVE_R)
-	char str[4096];
-    va_list list;
-    va_start(list,fmt);
+	const CHAR* msg_intro=get_msg_intro(prio);
+	if (!msg_intro)
+		return;
+
+	CHAR str[4096];
+	va_list list;
+	va_start(list,fmt);
 	vsnprintf(str, sizeof(str), fmt, list);
-    va_end(list);
+	va_end(list);
 
-	int p=get_prio_string(prio);
-	if (p>=0)
+	switch (prio)
 	{
-#ifdef HAVE_MATLAB
-		switch (prio)
-		{
-			case M_DEBUG:
-			case M_INFO:
-			case M_NOTICE:
-			case M_MESSAGEONLY:
-				fprintf(target, "%s", message_strings[p]);
-#ifdef WIN32
-				mexPrintf("%s", str);
-#else
-				fprintf(target, "%s", str);
-#endif
-				break;
-
-			case M_WARN:
-				mexWarnMsgTxt(str);
-				break;
-
-			case M_ERROR:
-			case M_CRITICAL:
-			case M_ALERT:
-			case M_EMERGENCY:
-#ifndef CYGWIN
-				CSignal::unset_handler();
-#endif
-				mexErrMsgTxt(str);
-				break;
-			default:
-				break;
-		}
-#elif defined(HAVE_OCTAVE)
-		switch (prio)
-		{
-			case M_DEBUG:
-			case M_INFO:
-			case M_NOTICE:
-			case M_MESSAGEONLY:
-				fprintf(target, "%s", message_strings[p]);
-				fprintf(target, "%s", str);
-				break;
-
-			case M_WARN:
-				::warning(str);
-				break;
-
-			case M_ERROR:
-			case M_CRITICAL:
-			case M_ALERT:
-			case M_EMERGENCY:
-				CSignal::unset_handler();
-				error("%s", str);
-				break;
-			default:
-				break;
-		}
-#elif defined(HAVE_PYTHON) && !defined(HAVE_SWIG)
-		switch (prio)
-		{
-			case M_DEBUG:
-			case M_INFO:
-			case M_NOTICE:
-			case M_MESSAGEONLY:
-				fprintf(target, "%s", message_strings[p]);
-				fprintf(target, "%s", str);
-				break;
-
-			case M_WARN:
-				PyErr_Warn(NULL, str);
-				break;
-
-			case M_ERROR:
-			case M_CRITICAL:
-			case M_ALERT:
-			case M_EMERGENCY:
-				fprintf(target, "%s", message_strings[p]);
-				fprintf(target, "%s\n", str);
-				PyErr_SetString(PyExc_RuntimeError,str);
-				break;
-			default:
-				break;
-		}
-#elif defined(HAVE_PYTHON) && defined(HAVE_SWIG)
-		switch (prio)
-		{
-			case M_DEBUG:
-			case M_INFO:
-			case M_NOTICE:
-			case M_MESSAGEONLY:
-				fprintf(target, "%s", message_strings[p]);
-				fprintf(target, "%s", str);
-				break;
-
-			case M_WARN:
-				PyErr_Warn(NULL, str);
-				break;
-
-			case M_ERROR:
-			case M_CRITICAL:
-			case M_ALERT:
-			case M_EMERGENCY:
-				throw ShogunException(str);
-				break;
-			default:
-				break;
-		}
-
+		case M_DEBUG:
+		case M_INFO:
+		case M_NOTICE:
+		case M_MESSAGEONLY:
+#if defined(WIN32) && defined(HAVE_MATLAB)
+			fprintf(target, "%s", msg_intro);
+			mexPrintf("%s", str);
 #elif defined(HAVE_R)
-		switch (prio)
-		{
-			case M_DEBUG:
-			case M_INFO:
-			case M_NOTICE:
-			case M_MESSAGEONLY:
-			case M_WARN:
-				Rprintf((char*) "%s",message_strings[p]);
-				Rprintf((char*) "%s",str);
-				break;
-			case M_ERROR:
-			case M_CRITICAL:
-			case M_ALERT:
-			case M_EMERGENCY:
-				error("%s", str);
-				break;
-			default:
-				break;
-		}
-
-#endif
-		fflush(target);
-	}
+			if (target==stdout)
+			{
+				Rprintf((char*) "%s", msg_intro);
+				Rprintf((char*) "%s", str);
+			}
+			else
+			{
+				fprintf(target, "%s", msg_intro);
+				fprintf(target, "%s", str);
+			}
 #else
-	int p=get_prio_string(prio);
-	if (p>=0)
-	{
-		fprintf(target, "%s", message_strings[p]);
-		va_list list;
-		va_start(list,fmt);
-		vfprintf(target,fmt,list);
-		va_end(list);
-		fflush(target);
-	}
+			fprintf(target, "%s", msg_intro);
+			fprintf(target, "%s", str);
 #endif
+			break;
+
+		case M_WARN:
+#if defined(HAVE_MATLAB)
+			mexWarnMsgTxt(str);
+#elif defined(HAVE_OCTAVE)
+			::warning(str);
+#elif defined(HAVE_PYTHON) // no check for swig necessary
+			PyErr_Warn(NULL, str);
+#elif defined(HAVE_R)
+			if (target==stdout)
+			{
+				Rprintf((char*) "%s", msg_intro);
+				Rprintf((char*) "%s", str);
+			}
+			else
+			{
+				fprintf(target, "%s", msg_intro);
+				fprintf(target, "%s", str);
+			}
+#else
+			fprintf(target, "%s", msg_intro);
+			fprintf(target, "%s", str);
+#endif
+			break;
+
+		case M_ERROR:
+		case M_CRITICAL:
+		case M_ALERT:
+		case M_EMERGENCY:
+#ifndef WIN32
+			CSignal::unset_handler();
+#endif
+#if defined(WIN32) && defined(HAVE_MATLAB)
+			mexPrintf("%s", str);
+#elif defined(HAVE_OCTAVE)
+			error("%s", str);
+#elif defined(HAVE_PYTHON) && !defined(HAVE_SWIG)
+			PyErr_SetString(PyExc_RuntimeError, str);
+#elif defined(HAVE_PYTHON) && defined(HAVE_SWIG)
+			// nop
+#elif defined(HAVE_R)
+			error("%s", str);
+#else
+			fprintf(target, "%s", str);
+#endif
+			throw ShogunException(str);
+			break;
+		default:
+			break;
+	}
+
+	fflush(target);
 }
 
 void CIO::buffered_message(EMessageType prio, const CHAR *fmt, ... ) const
 {
-	int p=get_prio_string(prio);
-	if (p>=0)
-	{
-		fprintf(target, "%s", message_strings[p]);
-		va_list list;
-		va_start(list,fmt);
-		vfprintf(target,fmt,list);
-		va_end(list);
-	}
+	const CHAR* msg_intro=get_msg_intro(prio);
+	if (!msg_intro)
+		return;
+
+	fprintf(target, "%s", msg_intro);
+	va_list list;
+	va_start(list,fmt);
+	vfprintf(target,fmt,list);
+	va_end(list);
 }
 
 void CIO::progress(DREAL current_val, DREAL min_val, DREAL max_val, INT decimals, const char* prefix)
 {
+	if (!show_progress)
+		return;
+
 	LONG runtime = CTime::get_runtime() ;
 
 	char str[1000];
@@ -252,10 +181,10 @@ void CIO::progress(DREAL current_val, DREAL min_val, DREAL max_val, INT decimals
 	else
 	{
 		if (v>100) v=100.0 ;
-		if (v<=0) v=1e-6 ;
-		last_progress = v-1e-5 ; ;
+		if (v<=0) v=1e-5 ;
+		last_progress = v-1e-6 ; ;
 
-		if ((v!=100.0) && (runtime - last_progress_time<100))
+		if ((v!=100.0) && (runtime - last_progress_time<10))
 			return ;
 
 		last_progress_time = runtime ;
@@ -279,6 +208,9 @@ void CIO::progress(DREAL current_val, DREAL min_val, DREAL max_val, INT decimals
 
 void CIO::absolute_progress(DREAL current_val, DREAL val, DREAL min_val, DREAL max_val, INT decimals, const char* prefix)
 {
+	if (!show_progress)
+		return;
+
 	LONG runtime = CTime::get_runtime() ;
 
 	char str[1000];
@@ -324,6 +256,14 @@ void CIO::absolute_progress(DREAL current_val, DREAL val, DREAL min_val, DREAL m
     fflush(target);
 }
 
+void CIO::done()
+{
+	if (!show_progress)
+		return;
+
+	message(M_INFO, "done.\n");
+}
+
 CHAR* CIO::skip_spaces(CHAR* str)
 {
 	INT i=0;
@@ -338,6 +278,20 @@ CHAR* CIO::skip_spaces(CHAR* str)
 		return str;
 }
 
+CHAR* CIO::skip_blanks(CHAR* str)
+{
+	INT i=0;
+
+	if (str)
+	{
+		for (i=0; isblank(str[i]); i++);
+
+		return &str[i];
+	}
+	else
+		return str;
+}
+
 EMessageType CIO::get_loglevel() const
 {
 	return loglevel;
@@ -348,36 +302,30 @@ void CIO::set_loglevel(EMessageType level)
 	loglevel=level;
 }
 
+bool CIO::get_show_progress() const
+{
+	return show_progress;
+}
+
 void CIO::set_target(FILE* t)
 {
 	target=t;
 }
 
-
-int CIO::get_prio_string(EMessageType prio) const
+const CHAR* CIO::get_msg_intro(EMessageType prio) const
 {
-	int i=0;
-	int idx=-1;
-
-	while (i<NUM_LOG_LEVELS)
+	for (INT i=NUM_LOG_LEVELS-1; i>=0; i--)
 	{
-		if (levels[i]==loglevel)
-		{
-			while (i<NUM_LOG_LEVELS)
-			{
-				if (levels[i]==prio)
-				{
-					idx=i;
-					break;
-				}
-				i++;
-			}
-			break;
-		}
-		i++;
+		// ignore msg if prio's level is under loglevel,
+		// but not if prio's level higher than M_WARN
+		if (levels[i]<loglevel && prio<=M_WARN)
+			return NULL;
+
+		if (levels[i]==prio)
+			return message_strings[i];
 	}
 
-	return idx;
+	return NULL;
 }
 
 CHAR* CIO::concat_filename(const CHAR* filename)
