@@ -1,21 +1,21 @@
 /*
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
- * Written (W) 1999-2007 Soeren Sonnenburg
- * Copyright (C) 1999-2007 Fraunhofer Institute FIRST and Max-Planck-Society
+ * Written (W) 1999-2008 Soeren Sonnenburg
+ * Copyright (C) 1999-2008 Fraunhofer Institute FIRST and Max-Planck-Society
  */
 
 #include "preproc/NormOne.h"
-#include "preproc/RealPreProc.h"
+#include "preproc/SimplePreProc.h"
 #include "lib/Mathematics.h"
 #include "features/Features.h"
 #include "features/RealFeatures.h"
 
 CNormOne::CNormOne()
-  : CRealPreProc("NormOne", "NRM1") 
+: CSimplePreProc<DREAL>("NormOne", "NRM1")
 {
 }
 
@@ -26,8 +26,9 @@ CNormOne::~CNormOne()
 /// initialize preprocessor from features
 bool CNormOne::init(CFeatures* f)
 {
-	ASSERT(f->get_feature_class() == C_SIMPLE);
-	ASSERT(f->get_feature_type() == F_DREAL);
+	ASSERT(f->get_feature_class()==C_SIMPLE);
+	ASSERT(f->get_feature_type()==F_DREAL);
+
 	return true;
 }
 
@@ -72,7 +73,7 @@ DREAL* CNormOne::apply_to_feature_matrix(CFeatures* f)
 		}
 
 		norm=sqrt(sqnorm);
-		
+
 		for (j=0; j<num_feat; j++)
 			vec[j]/=norm;
 	}
@@ -90,7 +91,7 @@ DREAL* CNormOne::apply_to_feature_vector(DREAL* f, INT& len)
 
 	for (i=0; i<len; i++)
 		sqnorm+=f[i]*f[i];
-	
+
 	norm=sqrt(sqnorm);
 
 	for (i=0; i<len; i++)
