@@ -1,21 +1,22 @@
 /*
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
- * Written (W) 1999-2007 Soeren Sonnenburg
- * Written (W) 1999-2007 Gunnar Raetsch
- * Copyright (C) 1999-2007 Fraunhofer Institute FIRST and Max-Planck-Society
+ * Written (W) 1999-2008 Soeren Sonnenburg
+ * Written (W) 1999-2008 Gunnar Raetsch
+ * Copyright (C) 1999-2008 Fraunhofer Institute FIRST and Max-Planck-Society
  */
 
 #include "preproc/SortWord.h"
-#include "preproc/WordPreProc.h"
+#include "preproc/SimplePreProc.h"
 #include "features/Features.h"
 #include "lib/Mathematics.h"
 #include "features/WordFeatures.h"
 
-CSortWord::CSortWord() : CWordPreProc("SortWord", "SRTW") 
+CSortWord::CSortWord()
+: CSimplePreProc<WORD>("SortWord", "SRTW")
 {
 }
 
@@ -26,8 +27,9 @@ CSortWord::~CSortWord()
 /// initialize preprocessor from features
 bool CSortWord::init(CFeatures* f)
 {
-	ASSERT(f->get_feature_class() == C_SIMPLE);
-	ASSERT(f->get_feature_type() == F_WORD);
+	ASSERT(f->get_feature_class()==C_SIMPLE);
+	ASSERT(f->get_feature_type()==F_WORD);
+
 	return true;
 }
 
