@@ -1,12 +1,12 @@
 /*
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
- * Written (W) 1999-2007 Soeren Sonnenburg
- * Written (W) 1999-2007 Gunnar Raetsch
- * Copyright (C) 1999-2007 Fraunhofer Institute FIRST and Max-Planck-Society
+ * Written (W) 1999-2008 Soeren Sonnenburg
+ * Written (W) 1999-2008 Gunnar Raetsch
+ * Copyright (C) 1999-2008 Fraunhofer Institute FIRST and Max-Planck-Society
  */
 
 #include "preproc/SortWordString.h"
@@ -14,7 +14,8 @@
 #include "features/StringFeatures.h"
 #include "lib/Mathematics.h"
 
-CSortWordString::CSortWordString() : CStringPreProc<WORD>("SortWordString", "STWS") 
+CSortWordString::CSortWordString()
+: CStringPreProc<WORD>("SortWordString", "STWS") 
 {
 }
 
@@ -25,8 +26,9 @@ CSortWordString::~CSortWordString()
 /// initialize preprocessor from features
 bool CSortWordString::init(CFeatures* f)
 {
-	ASSERT(f->get_feature_class() == C_STRING);
-	ASSERT(f->get_feature_type() == F_WORD);
+	ASSERT(f->get_feature_class()==C_STRING);
+	ASSERT(f->get_feature_type()==F_WORD);
+
 	return true;
 }
 
