@@ -1,11 +1,11 @@
 /*
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
- * Written (W) 1999-2007 Soeren Sonnenburg
- * Copyright (C) 1999-2007 Fraunhofer Institute FIRST and Max-Planck-Society
+ * Written (W) 1999-2008 Soeren Sonnenburg
+ * Copyright (C) 1999-2008 Fraunhofer Institute FIRST and Max-Planck-Society
  */
 
 #ifndef _CSORTWORDSTRING__H__
@@ -17,14 +17,14 @@
 
 #include <stdio.h>
 
+/** Preprocessor SortWordString */
 class CSortWordString : public CStringPreProc<WORD>
 {
 public:
+	/** default constructor */
 	CSortWordString();
 	virtual ~CSortWordString();
 
-	virtual EFeatureType get_feature_type() { return F_WORD; }
-
 	/// initialize preprocessor from features
 	virtual bool init(CFeatures* f);
 
