@@ -1,11 +1,11 @@
 /*
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
- * Written (W) 1999-2007 Gunnar Raetsch
- * Copyright (C) 1999-2007 Fraunhofer Institute FIRST and Max-Planck-Society
+ * Written (W) 1999-2008 Gunnar Raetsch
+ * Copyright (C) 1999-2008 Fraunhofer Institute FIRST and Max-Planck-Society
  */
 
 #include "lib/config.h"
@@ -22,34 +22,35 @@
 //#define PLIF_DEBUG
 
 CPlif::CPlif(INT l)
+: CPlifBase()
 {
-	limits=NULL ;
-	penalties=NULL ;
-	cum_derivatives=NULL ;
-	id=-1 ;
-	transform = T_LINEAR ;
-	name = NULL ;
-	max_value=0 ;
-	min_value=0 ;
-	cache=NULL ;
-	use_svm=0 ;
-	use_cache=false ;
+	limits=NULL;
+	penalties=NULL;
+	cum_derivatives=NULL;
+	id=-1;
+	transform=T_LINEAR;
+	name=NULL;
+	max_value=0;
+	min_value=0;
+	cache=NULL;
+	use_svm=0;
+	use_cache=false;
 	len=0;
-
+	do_calc = true;
 	if (l>0)
 		set_plif_length(l);
 }
 
 CPlif::~CPlif()
 {
-	delete[] limits ;
-	delete[] penalties ;
-	delete[] name ;
-	delete[] cache ;
-	delete[] cum_derivatives ; 
+	delete[] limits;
+	delete[] penalties;
+	delete[] name;
+	delete[] cache;
+	delete[] cum_derivatives;
 }
 
-bool CPlif::set_transform_type(const char *type_str) 
+bool CPlif::set_transform_type(const char *type_str)
 {
 	delete[] cache ;
 	cache=NULL ;
@@ -73,7 +74,6 @@ bool CPlif::set_transform_type(const char *type_str)
 	}
 	return true ;
 }
-
 void CPlif::init_penalty_struct_cache()
 {
 	if (!use_cache)
@@ -227,6 +227,7 @@ CPlif** read_penalty_struct_from_cell(const mxArray * mx_penalty_info, INT P)
 			return NULL ;
 		}
 		PEN[id]->set_min_value(min_value) ;
+		//SG_PRINT("id: %i, min_value: %i,  max_value: %i\n",id,min_value, max_value);
 		
 		if (PEN[id]->get_id()!=-1)
 		{
@@ -270,6 +271,7 @@ CPlif** read_penalty_struct_from_cell(const mxArray * mx_penalty_info, INT P)
 	}
 	return PEN ;
 }
+#endif
 
 void delete_penalty_struct(CPlif** PEN, INT P) 
 {
@@ -278,16 +280,16 @@ void delete_penalty_struct(CPlif** PEN, INT P)
 	delete[] PEN ;
 }
 
-#endif
-
 DREAL CPlif::lookup_penalty_svm(DREAL p_value, DREAL *d_values) const
 {	
-	ASSERT(use_svm>0) ;
+	ASSERT(use_svm>0);
 	DREAL d_value=d_values[use_svm-1] ;
 #ifdef PLIF_DEBUG
 	SG_PRINT("%s.lookup_penalty_svm(%f)\n", get_name(), d_value) ;
 #endif
 
+	if (!do_calc)
+		return d_value;
 	switch (transform)
 	{
 	case T_LINEAR:
@@ -347,7 +349,8 @@ DREAL CPlif::lookup_penalty(INT p_value, DREAL* svm_values) const
 
 	if ((p_value<min_value) || (p_value>max_value))
 		return -CMath::INFTY ;
-	
+	if (!do_calc)
+		return p_value;
 	if (cache!=NULL && (p_value>=0) && (p_value<=max_value))
 	{
 		DREAL ret=cache[p_value] ;
@@ -365,9 +368,13 @@ DREAL CPlif::lookup_penalty(DREAL p_value, DREAL* svm_values) const
 	SG_PRINT("%s.lookup_penalty(%f)\n", get_name(), p_value) ;
 #endif
 
+
 	if ((p_value<min_value) || (p_value>max_value))
 		return -CMath::INFTY ;
-	
+
+	if (!do_calc)
+		return p_value;
+
 	DREAL d_value = (DREAL) p_value ;
 	switch (transform)
 	{
@@ -442,8 +449,9 @@ void CPlif::penalty_add_derivative(DREAL p_value, DREAL* svm_values)
 	}
 	
 	if ((p_value<min_value) || (p_value>max_value))
+	{
 		return ;
-	
+	}
 	DREAL d_value = (DREAL) p_value ;
 	switch (transform)
 	{
@@ -486,8 +494,11 @@ void CPlif::penalty_add_derivative(DREAL p_value, DREAL* svm_values)
 
 void CPlif::penalty_add_derivative_svm(DREAL p_value, DREAL *d_values) 
 {	
-	ASSERT(use_svm>0) ;
+	ASSERT(use_svm>0);
 	DREAL d_value=d_values[use_svm-1] ;
+
+	if (d_value<-1e+20)
+		return;
 	
 	switch (transform)
 	{
@@ -527,3 +538,23 @@ void CPlif::penalty_add_derivative_svm(DREAL p_value, DREAL *d_values)
 		cum_derivatives[idx-1]+=(limits[idx]-d_value)/(limits[idx]-limits[idx-1]) ;
 	}
 }
+void CPlif::get_used_svms(INT* num_svms, INT* svm_ids)
+{
+	if (use_svm)
+	{
+		svm_ids[(*num_svms)] = use_svm;
+		(*num_svms)++;
+	}
+	SG_PRINT("->use_svm:%i plif_id:%i name:%s trans_type:%s  ",use_svm, get_id(), get_name(), get_transform_type());
+}
+bool CPlif::get_do_calc()
+{
+	return do_calc;
+}
+void CPlif::set_do_calc(bool b)
+{
+	do_calc = b;;
+}
+
+
+
